using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.RecordsManagement.InformationPolicy;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Workflow;
using System.Runtime.InteropServices;

namespace Edfinancial.SharePoint.STSADM.Commands.ContentTypes
{
    public class CopyContentTypes : SPOperation
    {
        private static bool copyWorkflows = true;
        private static bool copyColumns = true;
        private static bool copyDocConversions = true;
        private static bool copyDocInfoPanel = true;
        private static bool copyPolicies = true;
        private static bool copyDocTemplate = true;

        /// <summary>
        /// Initializes a new instance of the <see cref="CopyContentTypes"/> class.
        /// </summary>
        public CopyContentTypes()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sourceurl", "sourceurl", true, null, new SPUrlValidator(), "Please specify the source site collection."));
            parameters.Add(new SPParam("targeturl", "targeturl", true, null, new SPUrlValidator(), "Please specify the target site collection"));
            parameters.Add(new SPParam("sourcename", "sourcename", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("noworkflows", "noworkflows", false, null, null));
            parameters.Add(new SPParam("nocolumns", "nocolumns", false, null, null));
            parameters.Add(new SPParam("nodocconversions", "nodocconversions", false, null, null));
            parameters.Add(new SPParam("nodocinfopanel", "nodocinfopanel", false, null, null));
            parameters.Add(new SPParam("nopolicies", "nopolicies", false, null, null));
            parameters.Add(new SPParam("nodoctemplate", "nodoctemplate", false, null, null));
            parameters.Add(new SPParam("verbose", "v"));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nCopies all Content Types from one gallery to another.\r\n\r\nParameters:");
            sb.Append("\r\n\t-sourceurl <site collection url containing the source content types>");
            sb.Append("\r\n\t-targeturl <site collection url where the content types will be copied to>");
            sb.Append(
                "\r\n\t[-sourcename <name of an individual content type to copy - if ommitted all content types are copied if they don't already exist>");
            sb.Append("\r\n\t[-noworkflows]");
            sb.Append("\r\n\t[-nocolumns]");
            sb.Append("\r\n\t[-nodocconversions]");
            sb.Append("\r\n\t[-nodocinfopanel]");
            sb.Append("\r\n\t[-nopolicies]");
            sb.Append("\r\n\t[-nodoctemplate]");
            sb.Append("\r\n\t[-verbose]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string sourceUrl = Params["sourceurl"].Value.TrimEnd('/');
            string targetUrl = Params["targeturl"].Value.TrimEnd('/');
            string sourceContentTypeName = null;
            if (Params["sourcename"].UserTypedIn)
                sourceContentTypeName = Params["sourcename"].Value;
            bool verbose = Params["verbose"].UserTypedIn;


            copyWorkflows = !Params["noworkflows"].UserTypedIn;
            copyColumns = !Params["nocolumns"].UserTypedIn;
            copyDocConversions = !Params["nodocconversions"].UserTypedIn;
            copyDocInfoPanel = !Params["nodocinfopanel"].UserTypedIn;
            copyPolicies = !Params["nopolicies"].UserTypedIn;
            copyDocTemplate = !Params["nodoctemplate"].UserTypedIn;

            SPContentTypeCollection availableTargetContentTypes;
            SPContentTypeCollection availableSourceContentTypes;
            SPContentTypeCollection targetContentTypes;
            SPFieldCollection sourceFields;
            SPFieldCollection targetFields;

            if (verbose)
                Console.WriteLine("Start Time: {0}", DateTime.Now);

            // Get the source content type and fields.
            using (SPSite site = new SPSite(sourceUrl))
            {
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(sourceUrl)])
                {
                    if (verbose)
                        Log("Getting existing source fields and content types...");

                    availableSourceContentTypes = web.AvailableContentTypes;
                    sourceFields = web.Fields;
                }
            }

            // Get the target content type and fields.
            using (SPSite site = new SPSite(targetUrl))
            {
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(targetUrl)])
                {
                    if (verbose)
                        Log("Getting existing target fields and content types...");
                    
                    availableTargetContentTypes = web.AvailableContentTypes;
                    targetContentTypes = web.ContentTypes;
                    targetFields = web.Fields;
                }
            }

            // Make sure the source exists if it was specified.
            if (sourceContentTypeName != null)
            {
                SPContentType sourceCT = availableSourceContentTypes[sourceContentTypeName];
                if (sourceCT == null)
                {
                    output += "The source content type could not be found.";
                    return 0;
                }
                else
                {
                    if (verbose)
                        Log("Source content type found.");

                    if (availableTargetContentTypes[sourceCT.Name] == null)
                    {
                        if (verbose)
                            Log(string.Format("Source content type '{0}' does not exist on target - creating content type...", sourceCT.Name));

                        CreateContentType(sourceCT, availableTargetContentTypes, targetContentTypes, sourceFields, targetFields, verbose);
                    }
                    else
                    {
                        throw new SPException("Content type already exists at target.");
                    }
                }
            }
            else
            {
                // Loop through all the source content types and create them at the target.
                foreach (SPContentType sourceCT in availableSourceContentTypes)
                {
                    if (availableTargetContentTypes[sourceCT.Name] == null)
                    {
                        if (verbose)
                            Log(string.Format("Source content type '{0}' does not exist on target - creating content type...", sourceCT.Name));

                        CreateContentType(
                            sourceCT, 
                            availableTargetContentTypes, 
                            targetContentTypes, 
                            sourceFields,
                            targetFields,
                            verbose);
                    }
                    else
                    {
                        if (verbose)
                            Log(string.Format("Source content type '{0}' exists on target - skipping.", sourceCT.Name));
                    }
                }
            }

            if (verbose)
                Console.WriteLine("Finish Time: {0}", DateTime.Now);

            return 1;
        }

        #endregion

        /// <summary>
        /// Logs the specified message.
        /// </summary>
        /// <param name="message">The message.</param>
        private static void Log(string message)
        {
            Console.WriteLine("Progress: {0}", message);
        }

        /// <summary>
        /// Creates the content type.
        /// </summary>
        /// <param name="sourceCT">The source content type.</param>
        /// <param name="availableTargetContentTypes">The available target content types.</param>
        /// <param name="targetContentTypes">The target content types.</param>
        /// <param name="sourceFields">The source fields.</param>
        /// <param name="targetFields">The target fields.</param>
        /// <param name="verbose">if set to <c>true</c> [verbose].</param>
        private static void CreateContentType(SPContentType sourceCT, SPContentTypeCollection availableTargetContentTypes, SPContentTypeCollection targetContentTypes, SPFieldCollection sourceFields, SPFieldCollection targetFields, bool verbose)
        {
            // Make sure any parent content types exist - they have to be there before we can create this content type.
            if (availableTargetContentTypes[sourceCT.Parent.Id] == null)
            {
                if (verbose)
                    Log(string.Format("Parent of content type '{0}' does not exist - creating...", sourceCT.Name));

                CreateContentType(sourceCT.Parent, availableTargetContentTypes, targetContentTypes, sourceFields,
                                  targetFields, verbose);
            }

            if (verbose)
                Log(string.Format("Creating content type '{0}'...", sourceCT.Name));

            // Create a new content type using information from the source content type.
            SPContentType newCT =
                new SPContentType(availableTargetContentTypes[sourceCT.Parent.Id], targetContentTypes, sourceCT.Name);

            if (verbose)
                Log(string.Format("Setting fields for content type '{0}'...", sourceCT.Name));

            // Set all the core properties for the content type.
            newCT.Group = sourceCT.Group;
            newCT.Hidden = sourceCT.Hidden;
            newCT.NewDocumentControl = sourceCT.NewDocumentControl;
            newCT.NewFormTemplateName = sourceCT.NewFormTemplateName;
            newCT.NewFormUrl = sourceCT.NewFormUrl;
            newCT.ReadOnly = sourceCT.ReadOnly;
            newCT.RequireClientRenderingOnNew = sourceCT.RequireClientRenderingOnNew;
            newCT.Description = sourceCT.Description;
            newCT.DisplayFormTemplateName = sourceCT.DisplayFormTemplateName;
            newCT.DisplayFormUrl = sourceCT.DisplayFormUrl;
            newCT.EditFormTemplateName = sourceCT.EditFormTemplateName;
            newCT.EditFormUrl = sourceCT.EditFormUrl;

            if (verbose)
                Log(string.Format("Adding content type '{0}' to collection...", sourceCT.Name));

            // Add the content type to the content types collection and update all the settings.
            targetContentTypes.Add(newCT);
            newCT.Update();

            // Add all the peripheral items

            if (copyColumns)
            {
                if (verbose)
                    Log(string.Format("Adding site columns for content type '{0}'...", sourceCT.Name));

                AddSiteColumns(newCT, sourceCT, targetFields, sourceFields, verbose);
            }

            if (copyWorkflows)
            {
                if (verbose)
                    Log(string.Format("Adding workflow associations for content type '{0}'...", sourceCT.Name));

                AddWorkflowAssociations(newCT, sourceCT, verbose);
            }

            if (copyDocTemplate)
            {
                if (verbose)
                    Log(string.Format("Adding document template for content type '{0}'...", sourceCT.Name));

                AddDocumentTemplate(newCT, sourceCT);
            }

            if (copyDocConversions)
            {
                if (verbose)
                    Log(string.Format("Adding document conversion settings for content type '{0}'...", sourceCT.Name));

                AddDocumentConversionSettings(newCT, sourceCT);
            }

            if (copyPolicies)
            {
                if (verbose)
                    Log(string.Format("Adding information rights policies for content type '{0}'...", sourceCT.Name));

                AddInformationRightsPolicies(newCT, sourceCT, verbose);
            }

            if (copyDocInfoPanel)
            {
                if (verbose)
                    Log(string.Format("Adding document information panel for content type '{0}'...", sourceCT.Name));

                AddDocumentInfoPanelToContentType(sourceCT, newCT);
            }
        }

        /// <summary>
        /// Adds the document template.
        /// </summary>
        /// <param name="targetCT">The target content type.</param>
        /// <param name="sourceCT">The source content type.</param>
        private static void AddDocumentTemplate(SPContentType targetCT, SPContentType sourceCT)
        {
            if (string.IsNullOrEmpty(sourceCT.DocumentTemplate))
                return;

            // Add the document template.
            SPFile sourceFile = sourceCT.ResourceFolder.Files[sourceCT.DocumentTemplate];
            if (sourceFile != null && !string.IsNullOrEmpty(sourceFile.Name))
            {
                SPFile targetFile = targetCT.ResourceFolder.Files.Add(sourceFile.Name, sourceFile.OpenBinary(), true);
                targetCT.DocumentTemplate = targetFile.Name;
                targetCT.Update();
            }
        }

        /// <summary>
        /// Adds the workflow associations.
        /// </summary>
        /// <param name="targetCT">The target content type.</param>
        /// <param name="sourceCT">The source content type.</param>
        /// <param name="verbose">if set to <c>true</c> [verbose].</param>
        private static void AddWorkflowAssociations(SPContentType targetCT, SPContentType sourceCT, bool verbose)
        {
            // Remove the default workflows - we're going to add from the source.
            while (targetCT.WorkflowAssociations.Count > 0)
            {
                targetCT.RemoveWorkflowAssociation(targetCT.WorkflowAssociations[0]);
            }

            // Add workflows.
            foreach (SPWorkflowAssociation wf in sourceCT.WorkflowAssociations)
            {
                if (verbose)
                    Log(string.Format("Adding workflow '{0}' to content type...", wf.Name));

                targetCT.AddWorkflowAssociation(wf);
            }
            targetCT.Update();
        }

        /// <summary>
        /// Adds the site columns.
        /// </summary>
        /// <param name="targetCT">The target content type.</param>
        /// <param name="sourceCT">The source content type.</param>
        /// <param name="targetFields">The target fields.</param>
        /// <param name="sourceFields">The source fields.</param>
        /// <param name="verbose">if set to <c>true</c> [verbose].</param>
        private static void AddSiteColumns(SPContentType targetCT, SPContentType sourceCT, SPFieldCollection targetFields, SPFieldCollection sourceFields, bool verbose)
        {
            // Store the field order so that we can reorder after adding all the fields.
            List<string> fields = new List<string>();
            foreach (SPField field in sourceCT.Fields)
            {
                if (!field.Hidden && field.Reorderable)
                    fields.Add(field.InternalName);
            }
            // Add any columns associated with the content type.
            foreach (SPFieldLink field in sourceCT.FieldLinks)
            {
                // First we need to see if the column already exists as a Site Column.
                SPField sourceField;
                try
                {
                    // First try and find the column via the ID
                    sourceField = sourceFields[field.Id];
                }
                catch
                {
                    try
                    {
                        // Couldn't locate via ID so now try the name
                        sourceField = sourceFields[field.Name];
                    }
                    catch
                    {
                        sourceField = null;
                    }
                }
                if (sourceField == null)
                {
                    // Couldn't locate by ID or name - it could be due to casing issues between the linked version of the name and actual field
                    // (for example, the Contact content type stores the name for email differently: EMail for the field and Email for the link)
                    foreach (SPField f in sourceCT.Fields)
                    {
                        if (field.Name.ToLowerInvariant() == f.InternalName.ToLowerInvariant())
                        {
                            sourceField = f;
                            break;
                        }
                    }
                }
                if (sourceField == null)
                {
                    if (verbose)
                        Log(string.Format("WARNING: Unable to add column '{0}' to content type.", field.Name));
                    continue;
                }

                if (!targetFields.ContainsField(sourceField.InternalName))
                {
                    if (verbose)
                        Log(string.Format("Adding column '{0}' to site columns...", sourceField.InternalName));

                    // The column does not exist so add the Site Column.
                    targetFields.Add(sourceField);
                }

                // Now that we know the column exists we can add it to our content type.
                if (targetCT.FieldLinks[sourceField.InternalName] == null) // This should always be true if we're here but I'm keeping it in as a safety check.
                {
                    if (verbose)
                        Log(string.Format("Associating content type with site column '{0}'...", sourceField.InternalName));
                    
                    // Now add the reference to the site column for this content type.
                    targetCT.FieldLinks.Add(field);
                }
            }
            // Save the fields so that we can reorder them.
            targetCT.Update(true);

            // Reorder the fields.
            try
            {
                targetCT.FieldLinks.Reorder(fields.ToArray());
            }
            catch
            {
                Console.WriteLine("WARNING: Unable to set field order.");
            }
            targetCT.Update(true);
        }

        /// <summary>
        /// Adds the information rights policies.
        /// </summary>
        /// <param name="targetCT">The target content type.</param>
        /// <param name="sourceCT">The source content type.</param>
        /// <param name="verbose">if set to <c>true</c> [verbose].</param>
        private static void AddInformationRightsPolicies(SPContentType targetCT, SPContentType sourceCT, bool verbose)
        {
            // Set information rights policy - must be done after the new content type is added to the collection.
            Policy sourcePolicy = Policy.GetPolicy(sourceCT);

            if (sourcePolicy != null)
            {
                PolicyCatalog catalog = new PolicyCatalog(targetCT.ParentWeb.Site);
                PolicyCollection policyList = catalog.PolicyList;
                if (policyList[sourcePolicy.Id] == null)
                {
                    XmlDocument exportedSourcePolicy = sourcePolicy.Export();
                    try
                    {
                        if (verbose)
                            Log(string.Format("Adding policy '{0}' to content type...", sourcePolicy.Name));

                        PolicyCollection.Add(targetCT.ParentWeb.Site, exportedSourcePolicy.OuterXml);
                    }
                    catch (Exception ex)
                    {
                        if (ex is NullReferenceException || ex is SEHException)
                            throw;
                        // Policy already exists
                    }
                }
                if (verbose)
                    Log(string.Format("Associating content type with policy '{0}'...", sourcePolicy.Name));
                // Associate the policy with the content type.
                Policy.CreatePolicy(targetCT, sourcePolicy);
            }
            targetCT.Update();
        }

        /// <summary>
        /// Adds the document conversion settings.
        /// </summary>
        /// <param name="targetCT">The target content type.</param>
        /// <param name="sourceCT">The source content type.</param>
        private static void AddDocumentConversionSettings(SPContentType targetCT, SPContentType sourceCT)
        {
            // Add document conversion settings if document converisons are enabled.
            if (targetCT.ParentWeb.Site.WebApplication.DocumentConversionsEnabled)
            {
                // First, handle the xml that describes what is enabled and each setting
                string sourceDocConversionXml = sourceCT.XmlDocuments["urn:sharePointPublishingRcaProperties"];
                if (sourceDocConversionXml != null)
                {
                    XmlDocument sourceDocConversionXmlDoc = new XmlDocument();
                    sourceDocConversionXmlDoc.LoadXml(sourceDocConversionXml);

                    targetCT.XmlDocuments.Delete("urn:sharePointPublishingRcaProperties");
                    targetCT.XmlDocuments.Add(sourceDocConversionXmlDoc);
                }
                // Second, handle the xml that describes what is excluded (disabled).
                sourceDocConversionXml =
                    sourceCT.XmlDocuments["http://schemas.microsoft.com/sharepoint/v3/contenttype/transformers"];
                if (sourceDocConversionXml != null)
                {
                    XmlDocument sourceDocConversionXmlDoc = new XmlDocument();
                    sourceDocConversionXmlDoc.LoadXml(sourceDocConversionXml);

                    targetCT.XmlDocuments.Delete("http://schemas.microsoft.com/sharepoint/v3/contenttype/transformers");
                    targetCT.XmlDocuments.Add(sourceDocConversionXmlDoc);
                }
                targetCT.Update();
            }
        }

        /// <summary>
        /// Adds the document info panel to the content type.
        /// </summary>
        /// <param name="sourceCT">The source content type.</param>
        /// <param name="targetCT">The target content type.</param>
        private static void AddDocumentInfoPanelToContentType(SPContentType sourceCT, SPContentType targetCT)
        {
            XmlDocument sourceXmlDoc = null;
            string sourceXsnLocation;
            bool isCached;
            bool openByDefault;
            string scope;

            // We first need to get the XML which describes the custom information panel.
            string str = sourceCT.XmlDocuments["http://schemas.microsoft.com/office/2006/metadata/customXsn"];
            if (!string.IsNullOrEmpty(str))
            {
                sourceXmlDoc = new XmlDocument();
                sourceXmlDoc.LoadXml(str);
            }
            if (sourceXmlDoc != null)
            {
                // We found settings for a custom information panel so grab those settings for later use.
                XmlNode node;
                string innerText;
                XmlNamespaceManager nsmgr = new XmlNamespaceManager(sourceXmlDoc.NameTable);
                nsmgr.AddNamespace("cust", "http://schemas.microsoft.com/office/2006/metadata/customXsn");
                sourceXsnLocation = sourceXmlDoc.SelectSingleNode("/cust:customXsn/cust:xsnLocation", nsmgr).InnerText;
                node = sourceXmlDoc.SelectSingleNode("/cust:customXsn/cust:cached", nsmgr);
                isCached = (node != null) && (node.InnerText == bool.TrueString);
                innerText = sourceXmlDoc.SelectSingleNode("/cust:customXsn/cust:openByDefault", nsmgr).InnerText;
                openByDefault = !string.IsNullOrEmpty(innerText) && (innerText == bool.TrueString);
            }
            else
                return;

            // This should never be null but just in case...
            if (sourceXsnLocation == null)
                return;

            // Grab the source file and add it to the target resource folder.
            SPFile sourceFile = sourceCT.ResourceFolder.Files[sourceXsnLocation];
            SPFile file2 = targetCT.ResourceFolder.Files.Add(targetCT.ParentWeb.Url + "/" + sourceFile.Url, sourceFile.OpenBinary(), true);
            
            // Get the target and scope to use in the xsn for the custom information panel.
            string targetXsnLocation = targetCT.ParentWeb.Url + "/" + file2.Url;
            scope = targetCT.ParentWeb.Site.MakeFullUrl(targetCT.Scope);

            XmlDocument targetXmlDoc = BuildCustomInformationPanelXml(targetXsnLocation, isCached, openByDefault, scope);

            // Delete the existing doc so that we can add the new one.
            targetCT.XmlDocuments.Delete("http://schemas.microsoft.com/office/2006/metadata/customXsn");
            targetCT.XmlDocuments.Add(targetXmlDoc);
            targetCT.Update();
        }

        /// <summary>
        /// Builds the custom information panel XML.
        /// </summary>
        /// <param name="targetXsnLocation">The target XSN location.</param>
        /// <param name="isCached">if set to <c>true</c> [is cached].</param>
        /// <param name="openByDefault">if set to <c>true</c> [open by default].</param>
        /// <param name="scope">The scope.</param>
        /// <returns></returns>
        private static XmlDocument BuildCustomInformationPanelXml(string targetXsnLocation, bool isCached, bool openByDefault, string scope)
        {
            XmlDocument document = new XmlDocument();
            XmlNode newChild = document.CreateNode(XmlNodeType.Element, "customXsn", "http://schemas.microsoft.com/office/2006/metadata/customXsn");
            document.AppendChild(newChild);
            XmlNode node2 = document.CreateNode(XmlNodeType.Element, "xsnLocation", "http://schemas.microsoft.com/office/2006/metadata/customXsn");
            newChild.AppendChild(node2);
            XmlNode node3 = document.CreateNode(XmlNodeType.Text, "xsnLocationText", "http://schemas.microsoft.com/office/2006/metadata/customXsn");
            node3.Value = targetXsnLocation;
            node2.AppendChild(node3);
            node2 = document.CreateNode(XmlNodeType.Element, "cached", "http://schemas.microsoft.com/office/2006/metadata/customXsn");
            newChild.AppendChild(node2);
            node3 = document.CreateNode(XmlNodeType.Text, "cachedText", "http://schemas.microsoft.com/office/2006/metadata/customXsn");
            node3.Value = isCached.ToString();
            node2.AppendChild(node3);
            node2 = document.CreateNode(XmlNodeType.Element, "openByDefault", "http://schemas.microsoft.com/office/2006/metadata/customXsn");
            newChild.AppendChild(node2);
            node3 = document.CreateNode(XmlNodeType.Text, "openByDefaultText", "http://schemas.microsoft.com/office/2006/metadata/customXsn");
            node3.Value = openByDefault.ToString();
            node2.AppendChild(node3);
            node2 = document.CreateNode(XmlNodeType.Element, "xsnScope", "http://schemas.microsoft.com/office/2006/metadata/customXsn");
            newChild.AppendChild(node2);
            node3 = document.CreateNode(XmlNodeType.Text, "xsnScopeText", "http://schemas.microsoft.com/office/2006/metadata/customXsn");
            node3.Value = scope;
            node2.AppendChild(node3);
            return document;
        }


    }
}
