using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class AddListItem : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="AddListItem"/> class.
        /// </summary>
        public AddListItem()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the list view url to add the items to."));
            parameters.Add(new SPParam("filename", "f", false, null, new SPFileExistsValidator()));
            parameters.Add(new SPParam("leafname", "ln", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("folderurl", "fu", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("fielddata", "fd", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("datafile", "df", false, null, new SPFileExistsValidator()));
            parameters.Add(new SPParam("publish", "p"));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nAdds a list item or items to an existing list.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <list view url to import into>");
            sb.Append("\r\n\t[-filename <file to add to the list (will add as attachment if not document library)>]");
            sb.Append("\r\n\t[-leafname <name to give the file if specified (be sure to include the file extension)>]");
            sb.Append("\r\n\t[-folderurl <url to place the specified file if not at the root (list relative)>]");
            sb.Append("\r\n\t{[-fielddata <semi-colon separated list of key value pairs: \"Field1=Val1;Field2=Val2\"> (use ';;' to escape semi-colons in data values)] | ");
            sb.Append("\r\n\t [-datafile <path to a file with xml settings matching the following format: <Items><Item File=\"{optional file path}\" FolderUrl=\"{optional folder url}\" LeafName=\"{optional display name of file}\" Author=\"{optional username}\" CreatedDate=\"{optional create date}\"><Fields><Field Name=\"Name1\">Value1</Field><Field Name=\"Name2\">Value2</Field></Fields><Attachments><Attachment>{path to file}</Attachment></Attachments></Item></Items> >]}");
            sb.Append("\r\n\t[-publish]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            bool publish = Params["publish"].UserTypedIn;


            using (SPSite site = new SPSite(url))
            using (SPWeb web = site.OpenWeb())
            {
                SPList list = Utilities.GetListFromViewUrl(web, url);

                if (list == null)
                    throw new SPException("List was not found.");

                List<ItemInfo> itemData;
                if (Params["fielddata"].UserTypedIn)
                {
                    itemData = GetItemData(Params["filename"].Value, Params["leafname"].Value, Params["folderurl"].Value, Params["fielddata"].Value);
                }
                else if (Params["datafile"].UserTypedIn)
                {
                    itemData = GetItemDataFromXml(list, Params["datafile"].Value);
                }
                else if (Params["filename"].UserTypedIn)
                {
                    itemData = GetItemData(Params["filename"].Value, Params["leafname"].Value, Params["folderurl"].Value, null);
                }
                else
                    throw new SPSyntaxException("Either fielddata, datafile, or filename must be provided.");
                
                AddItem(web, list, itemData, publish);
            }

            return 1;
        }

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["fielddata"].UserTypedIn)
            {
                string fieldData = Params["fielddata"].Value;
                fieldData = fieldData.Replace(";;", "_STSADM_ADDLISTITEM_SEMICOLON_");

                foreach (string prop in fieldData.Split(';'))
                {
                    if (prop.Split(new char[] { '=' }, 2).Length != 2)
                        throw new SPSyntaxException(
                            "The format of the fielddata parameter is incorrect: \"Field1=Val1;Field2=Val2\" (use ';;' to escape semi-colons in data values)");
                }
            }
            if (Params["fielddata"].UserTypedIn && Params["datafile"].UserTypedIn)
                throw new SPSyntaxException("The fielddata parameter and datafile parameters are incompatible.");

            if (Params["datafile"].UserTypedIn && Params["filename"].UserTypedIn)
                throw new SPSyntaxException("The datafile parameter and filename parameters are incompatible.  Specify the filename using the 'File' attribute of the 'Item' node.");

            if (Params["leafname"].UserTypedIn && !Params["filename"].UserTypedIn)
                throw new SPSyntaxException("The leafname parameter is only valid if the filename parameter is also specified.");

            if (Params["folderurl"].UserTypedIn && Params["datafile"].UserTypedIn)
                throw new SPSyntaxException("The folderurl parameter and datafile parameters are incompatible.");

            base.Validate(keyValues);
        }

        #endregion

        /// <summary>
        /// Adds the item.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="list">The list.</param>
        /// <param name="itemData">The item data.</param>
        /// <param name="publish">if set to <c>true</c> [publish].</param>
        internal static void AddItem(SPWeb web, SPList list, List<ItemInfo> itemData, bool publish)
        {
            bool itemsAdded = false;
            foreach (ItemInfo itemInfo in itemData)
            {
                SPListItem item;

                SPFolder folder = GetFolder(web, list, itemInfo);
                if (itemInfo.File == null)
                {

                    if (list.BaseType == SPBaseType.DocumentLibrary)
                    {
                        string title = Guid.NewGuid().ToString();
                        if (!string.IsNullOrEmpty(itemInfo.LeafName))
                            title = itemInfo.LeafName;

                        SPFile file;
                        if (itemInfo.Author == null)
                            file = folder.Files.Add(title, new byte[] { });
                        else
                            file = folder.Files.Add(title, new byte[] {}, itemInfo.Author, itemInfo.Author, itemInfo.CreatedDate, DateTime.Now);

                        item = file.Item;
                    }
                    else
                        item = list.Items.Add(folder.ServerRelativeUrl, SPFileSystemObjectType.File);
                }
                else
                {
                    // We have a file so we need to handle it.
                    // Make sure the leaf and folder properties are set.
                    if (string.IsNullOrEmpty(itemInfo.LeafName))
                        itemInfo.LeafName = itemInfo.File.Name;

                    if (list.BaseType != SPBaseType.DocumentLibrary)
                    {
                        // The list is not a document library so we'll add the file as an attachment.
                        item = list.Items.Add(folder.ServerRelativeUrl, SPFileSystemObjectType.File);

                        item.Attachments.Add(itemInfo.LeafName, File.ReadAllBytes(itemInfo.File.FullName));
                    }
                    else
                    {
                        // We've got the right folder so now add the file.
                        SPFile file;
                        if (itemInfo.Author == null)
                            file = folder.Files.Add(folder.ServerRelativeUrl + "/" + itemInfo.LeafName,
                                                       File.ReadAllBytes(itemInfo.File.FullName));
                        else
                            file = folder.Files.Add(folder.ServerRelativeUrl + "/" + itemInfo.LeafName, File.ReadAllBytes(itemInfo.File.FullName), itemInfo.Author, itemInfo.Author, itemInfo.CreatedDate, DateTime.Now);

                        // Get the SPListItem object from the added file.
                        item = file.Item;
                    }

                }
                // Set the field values
                foreach (string fieldName in itemInfo.FieldData.Keys)
                {
                    SPField field = item.Fields.GetFieldByInternalName(fieldName);

                    if (field.Type == SPFieldType.URL)
                        item[fieldName] = new SPFieldUrlValue(itemInfo.FieldData[fieldName]);
                    else
                        item[fieldName] = itemInfo.FieldData[fieldName];

                }
                
                if (itemInfo.Author != null && item.Fields.ContainsField("Created By"))
                    item["Created By"] = itemInfo.Author;

                if (item.Fields.ContainsField("Created"))
                    item["Created"] = itemInfo.CreatedDate;
                
                item.Update();
                
                if (list.BaseType != SPBaseType.DocumentLibrary)
                {
                    try
                    {
                        // Add any attachments
                        foreach (FileInfo att in itemInfo.Attachments)
                        {
                            item.Attachments.Add(att.Name, File.ReadAllBytes(att.FullName));
                        }
                    }
                    catch (ArgumentException)
                    {
                        throw new SPException("List does not support use of attachments.  Item added but not published.");
                    }
                }
                else if (itemInfo.Attachments.Count > 0)
                {
                    throw new SPException("List does not support use of attachments.  Item added but not published.");
                }
                item.Update();


                // Publish the changes
                if (publish)
                {
                    PublishItems.Settings settings = new PublishItems.Settings();
                    settings.Test = false;
                    settings.Quiet = true;
                    settings.LogFile = null;
                    PublishItems.PublishListItem(item, list, settings, "stsadm -o addlistitem");
                }
                itemsAdded = true;
            }
            if (itemsAdded)
                list.Update();
        }

        /// <summary>
        /// Gets the folder.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="list">The list.</param>
        /// <param name="itemInfo">The item info.</param>
        /// <returns></returns>
        internal static SPFolder GetFolder(SPWeb web, SPList list, ItemInfo itemInfo)
        {
            if (string.IsNullOrEmpty(itemInfo.FolderUrl))
                return list.RootFolder;

            SPFolder folder = web.GetFolder(list.RootFolder.Url + "/" + itemInfo.FolderUrl);

            if (!folder.Exists)
            {
                if (!list.EnableFolderCreation)
                {
                    list.EnableFolderCreation = true;
                    list.Update();
                }

                // We couldn't find the folder so create it
                string[] folders = itemInfo.FolderUrl.Trim('/').Split('/');

                string folderPath = string.Empty;
                for (int i = 0; i < folders.Length; i++)
                {
                    folderPath += "/" + folders[i];
                    folder = web.GetFolder(list.RootFolder.Url + folderPath);
                    if (!folder.Exists)
                    {
                        SPListItem newFolder = list.Items.Add("", SPFileSystemObjectType.Folder, folderPath.Trim('/'));
                        newFolder.Update();
                        folder = newFolder.Folder;
                    }
                }
            }
            // Still no folder so error out
            if (folder == null)
                throw new SPException(string.Format("The folder '{0}' could not be found.", itemInfo.FolderUrl));
            return folder;
        }


        /// <summary>
        /// Gets the item data from a comma delimited string (field values separated by "=").
        /// </summary>
        /// <param name="file">The file.</param>
        /// <param name="leafName">Name of the leaf.</param>
        /// <param name="folderUrl">The folder URL.</param>
        /// <param name="fieldData">The field data.</param>
        /// <returns></returns>
        private static List<ItemInfo> GetItemData(string file, string leafName, string folderUrl, string fieldData)
        {
            if (string.IsNullOrEmpty(fieldData) && string.IsNullOrEmpty(file))
                return null;

            List<ItemInfo> items = new List<ItemInfo>();
            ItemInfo item = new ItemInfo();

            items.Add(item);

            if (!string.IsNullOrEmpty(folderUrl))
                item.FolderUrl = folderUrl.TrimStart('/', '\\');

            if (!string.IsNullOrEmpty(leafName))
                item.LeafName = leafName;

            if (!string.IsNullOrEmpty(file))
                item.File = new FileInfo(file);

            if (!string.IsNullOrEmpty(fieldData))
            {
                fieldData = fieldData.Replace(";;", "_STSADM_ADDLISTITEM_SEMICOLON_");
                foreach (string s in fieldData.Split(';'))
                {
                    string[] data = s.Split(new char[] { '=' }, 2);
                    item.FieldData.Add(data[0].Trim(), data[1].Trim().Replace("_STSADM_ADDLISTITEM_SEMICOLON_", ";"));
                }
            }
            return items;
        }

        /// <summary>
        /// Gets the item data from XML.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="dataFile">The data file.</param>
        /// <returns></returns>
        private static List<ItemInfo> GetItemDataFromXml(SPList list, string dataFile)
        {
            if (string.IsNullOrEmpty(dataFile))
                return null;

            List<ItemInfo> items = new List<ItemInfo>();
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(dataFile);
            foreach (XmlElement itemElement in xmlDoc.DocumentElement.ChildNodes)
            {
                ItemInfo item = new ItemInfo();

                if (itemElement.HasAttribute("FolderUrl") && !string.IsNullOrEmpty(itemElement.Attributes["FolderUrl"].Value))
                    item.FolderUrl = itemElement.Attributes["FolderUrl"].Value.TrimStart('/', '\\');

                if (itemElement.HasAttribute("LeafName") && !string.IsNullOrEmpty(itemElement.Attributes["LeafName"].Value))
                    item.LeafName = itemElement.Attributes["LeafName"].Value;

                if (itemElement.HasAttribute("File") && !string.IsNullOrEmpty(itemElement.Attributes["File"].Value))
                    item.File = new FileInfo(itemElement.Attributes["File"].Value);

                if (itemElement.HasAttribute("CreatedDate") && !string.IsNullOrEmpty(itemElement.Attributes["CreatedDate"].Value))
                    item.CreatedDate = DateTime.Parse(itemElement.Attributes["CreatedDate"].Value);

                if (itemElement.HasAttribute("Author") && !string.IsNullOrEmpty(itemElement.Attributes["Author"].Value))
                {
                    string author = itemElement.Attributes["Author"].Value;
                    try
                    {
                        SPUser user;
                        using (SPWeb web = list.ParentWeb)
                            user = web.AllUsers[author];

                        item.Author = user;
                    }
                    catch (SPException) { }
                }

                foreach (XmlElement fieldElement in itemElement.SelectNodes("./Fields/Field"))
                {
                    item.FieldData.Add(fieldElement.Attributes["Name"].Value, fieldElement.InnerText.Trim());
                }
                foreach (XmlElement attElement in itemElement.SelectNodes("./Attachments/Attachment"))
                {
                    item.Attachments.Add(new FileInfo(attElement.InnerText.Trim()));
                }
                // Only add if we actually have something to insert
                if (item.File != null || item.FieldData.Count > 0)
                    items.Add(item);
            }
            return items;
        }
    }

    internal class ItemInfo
    {
        public FileInfo File = null;
        public string LeafName = null;
        public string FolderUrl = null;
        public SPUser Author = null;
        public DateTime CreatedDate = DateTime.Now;
        public List<FileInfo> Attachments = new List<FileInfo>();
        public Dictionary<string, string> FieldData = new Dictionary<string, string>();
    }
}
