using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Edfinancial.SharePoint.STSADM.Commands.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using System.Text.RegularExpressions;
using Microsoft.SharePoint.Utilities;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class ApplyUpgradeAreaUrlMappings : SPOperation
    {
        private static List<ReplacementSettings> upgradeUrls = null;

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ApplyUpgradeAreaUrlMappings"/> class.
        /// </summary>
        public ApplyUpgradeAreaUrlMappings()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPNonEmptyValidator(), "Please specify the url to search."));
            parameters.Add(new SPParam("scope", "scope", true, null, new SPRegexValidator("(?i:^WebApplication$|^Site$|^Web$|^List$|^Page$)")));
            parameters.Add(new SPParam("field", "f", false, null, new SPNonEmptyValidator(), "Please enter the field name."));
            parameters.Add(new SPParam("useinternalfieldname", "userinternal", false, null, null));
            parameters.Add(new SPParam("quiet", "q", false, null, null));
            parameters.Add(new SPParam("test", "t", false, null, null));
            parameters.Add(new SPParam("publish", "p", false, null, null));
            parameters.Add(new SPParam("logfile", "log", false, null, new SPDirectoryExistsAndValidFileNameValidator()));
            parameters.Add(new SPParam("webpartsonly", "webparts"));
            parameters.Add(new SPParam("listsonly", "lists"));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nReplaces all occurances of the V2 server relative URLs with the corresponding V3 server relative URLs as specified in the Upgrade Area Url Mappings list: \"http://[portal]/Lists/98d3057cd9024c27b2007643c1/AllItems.aspx\"\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <url to search>");
            sb.Append("\r\n\t-scope <WebApplication | Site | Web | List | Page>");
            sb.Append("\r\n\t[-field <field name>]");
            sb.Append("\r\n\t[-useinternalfieldname (if not present then the display name will be used)]");
            sb.Append("\r\n\t[-quiet]");
            sb.Append("\r\n\t[-test]");
            sb.Append("\r\n\t[-logfile <log file>]");
            sb.Append("\r\n\t[-publish]");
            sb.Append("\r\n\t[-webpartsonly]");
            sb.Append("\r\n\t[-listsonly]");

            Init(parameters, sb.ToString());
        }

        #endregion

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            if (url != null)
                url = url.TrimEnd('/');

            Settings settings = new Settings();
            settings.LogFile = Params["logfile"].Value;
            settings.FieldName = Params["field"].Value;
            settings.Publish = Params["publish"].UserTypedIn;
            settings.Quiet = Params["quiet"].UserTypedIn;
            settings.Test = Params["test"].UserTypedIn;
            settings.UseInternalFieldName = Params["useinternalfieldname"].UserTypedIn;
            settings.WebPartsOnly = Params["webpartsonly"].UserTypedIn;
            settings.ListsOnly = Params["listsonly"].UserTypedIn;

            string scope = Params["scope"].Value.ToLowerInvariant();

            using (SPSite site = new SPSite(url))
            using (SPSite rootSite = site.WebApplication.Sites[""])
            {
                // The upgrade list should always be at the root site collection.
                try
                {
                    settings.UpgradeList =
                        SiteCollectionSettings.UpdateV2ToV3UpgradeAreaUrlMappings.GetUpgradeAreaUrlMappingsList(rootSite);
                }
                catch(Exception)
                {
                    settings.UpgradeList = null;
                }
                if (settings.UpgradeList == null)
                    throw new SPException("Web application does not contain an Upgrade Area Url Mapping list.");
            }

            Log(settings, "Building replacement URL listing...");
            BuildReplacementUrlsList(settings);
            Log(settings, "Finished building replacement URL listing.\r\n");

            if (scope == "webapplication")
            {
                SPWebApplication webApp = SPWebApplication.Lookup(new Uri(url));
                ReplaceValues(webApp, settings);
            }
            else if (scope == "site")
            {
                using (SPSite site = new SPSite(url))
                {
                    ReplaceValues(site, settings);
                }
            }
            else if (scope == "web")
            {
                using (SPSite site = new SPSite(url))
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    ReplaceValues(web, settings);
                }
            }
            else if (scope == "list")
            {
                SPList list = Utilities.GetListFromViewUrl(url);
                ReplaceValues(list, settings);
            }
            else if (scope == "page")
            {
                using (SPSite site = new SPSite(url))
                using (SPWeb web = site.OpenWeb())
                {
                    ReplaceValues(web, web.GetFile(url), settings);
                }

            }

            return 1;
        }

        #endregion

        #region SPOperation Misc Overrides

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["quiet"].UserTypedIn && Params["test"].UserTypedIn && !Params["logfile"].UserTypedIn)
                throw new SPSyntaxException("The quiet parameter and the test parameter are incompatible when no log file is specified.");

            if (Params["webpartsonly"].UserTypedIn && Params["scope"].UserTypedIn && Params["scope"].Value.ToLowerInvariant() == "list")
                throw new SPSyntaxException("The webpartsonly parameter is incompatible with a scope of list.");

            if (Params["listsonly"].UserTypedIn && Params["scope"].UserTypedIn && Params["scope"].Value.ToLowerInvariant() == "page")
                throw new SPSyntaxException("The listsonly parameter is incompatible with a scope of page.");

            if (Params["webpartsonly"].UserTypedIn && Params["listsonly"].UserTypedIn)
                throw new SPSyntaxException("The webpartsonly parameter and the listsonly parameter are incompatible.");

            base.Validate(keyValues);
        }

        #endregion

        #region ReplaceValues Methods

        /// <summary>
        /// Replaces the values.
        /// </summary>
        /// <param name="webApp">The web app.</param>
        /// <param name="settings">The settings.</param>
        private static void ReplaceValues(SPWebApplication webApp, Settings settings)
        {
            Log(settings, "Processing Web Application: " + webApp.DisplayName);
            
            foreach (SPSite site in webApp.Sites)
            {
                try
                {
                    ReplaceValues(site, settings);
                }
                finally
                {
                    site.Dispose();
                }
            }

            Log(settings, "Finished Processing Web Application: " + webApp.DisplayName + "\r\n");
        }

        /// <summary>
        /// Replaces the values.
        /// </summary>
        /// <param name="site">The site.</param>
        /// <param name="settings">The settings.</param>
        private static void ReplaceValues(SPSite site, Settings settings)
        {
            Log(settings, "Processing Site: " + site.ServerRelativeUrl);
            
            foreach (SPWeb web in site.AllWebs)
            {
                try
                {
                    ReplaceValues(web, settings);
                }
                finally
                {
                    web.Dispose();
                }
            }

            Log(settings, "Finished Processing Site: " + site.ServerRelativeUrl + "\r\n");
        }

        /// <summary>
        /// Replaces the values.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="settings">The settings.</param>
        private static void ReplaceValues(SPWeb web, Settings settings)
        {
            if (!settings.WebPartsOnly)
            {
                Log(settings, "Beginning List Replacements...");
                Log(settings, "Processing Web: " + web.ServerRelativeUrl);

                foreach (SPList list in web.Lists)
                {
                    ReplaceValues(list, settings);
                }

                Log(settings, "Finished Processing Web: " + web.ServerRelativeUrl + "\r\n");
                Log(settings, "Finished List Replacements.");
            }

            if (!string.IsNullOrEmpty(settings.FieldName))
                return;

            if (!settings.ListsOnly)
            {
                Log(settings, "Beginning Web Part Replacements...");
                ReplaceWebPartContent.Settings settings2 = new ReplaceWebPartContent.Settings();
                settings2.LogFile = settings.LogFile;
                settings2.Publish = settings.Publish;
                settings2.Quiet = settings.Quiet;
                settings2.Test = settings.Test;
                settings2.Verbose = false;

                foreach (ReplacementSettings rs in upgradeUrls)
                {
                    Log(settings, "Processing \"" + settings2.SearchString + "\"...");
                    settings2.SearchString = rs.Regex.ToString();
                    settings2.ReplaceString = rs.ReplaceString;
                    ReplaceWebPartContent.ReplaceValues(web, settings2);
                }
                Log(settings, "Finished Web Part Replacements.");
            }
        }

        /// <summary>
        /// Replaces the values.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="file">The file.</param>
        /// <param name="settings">The settings.</param>
        private static void ReplaceValues(SPWeb web, SPFile file, Settings settings)
        {
            ReplaceWebPartContent.Settings settings2 = new ReplaceWebPartContent.Settings();
            settings2.LogFile = settings.LogFile;
            settings2.Publish = settings.Publish;
            settings2.Quiet = settings.Quiet;
            settings2.Test = settings.Test;
            settings2.Verbose = false;

            foreach (ReplacementSettings rs in upgradeUrls)
            {
                Log(settings, "Processing \"" + settings2.SearchString + "\"...");
                settings2.SearchString = rs.Regex.ToString();
                settings2.ReplaceString = rs.ReplaceString;
                ReplaceWebPartContent.ReplaceValues(web, file, settings2);
            }
        }

        /// <summary>
        /// Replaces the values.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="settings">The settings.</param>
        private static void ReplaceValues(SPList list, Settings settings)
        {
            if (list.Title.ToLowerInvariant() == "upgrade area url mapping" || list.DefaultViewUrl.ToLowerInvariant().IndexOf(UPGRADE_AREA_URL_LIST) >= 0)
                return; // We don't want to process this list as it's the source of our data.

            Log(settings, "Processing List: " + list.DefaultViewUrl);

            // We've got the replacement list built - now we need to check every field of every item.
            foreach (SPListItem item in list.Items)
            {
                if (item.File != null && Utilities.IsCheckedOut(item) && !Utilities.IsCheckedOutByCurrentUser(item))
                {
                    continue;
                }
                bool wasCheckedOut = true;
                bool modified = false;

                foreach (SPField field in list.Fields)
                {
                    if (item[field.Id] == null || field.ReadOnlyField)
                        continue;

                    Type fieldType = item[field.Id].GetType();

                    if (fieldType != typeof(string))
                        continue; // We're only going to work with strings.

                    string fieldName = field.Title.ToLowerInvariant();
                    if (settings.UseInternalFieldName)
                        fieldName = field.InternalName.ToLowerInvariant();

                    if (settings.FieldName == null || settings.FieldName.ToLowerInvariant() == fieldName)
                    {
                        // We've found a field that we need to consider.  Check all possibilities to see if a match is found.
                        foreach (ReplacementSettings rs in upgradeUrls)
                        {
                            bool isMatch = rs.Regex.IsMatch((string)item[field.Id]);

                            if (!isMatch)
                                continue;

                            string result = rs.Regex.Replace((string)item[field.Id], rs.ReplaceString);

                            Log(settings,
                                string.Format("Match found: List={0}, Field={1}, Replacement={2} => {3}", item.Url,
                                              field.Title, item[field.Id], result));

                            if (!settings.Test)
                            {
                                if (item.File != null && item.File.CheckOutStatus == SPFile.SPCheckOutStatus.None)
                                {
                                    item.File.CheckOut();
                                    wasCheckedOut = false;
                                }
                                item[field.Id] = result;
                                modified = true;
                            }
                        }
                    }
                }
                if (modified && !settings.Test)
                    item.Update();

                if (modified && item.File != null)
                    item.File.CheckIn("Checking in changes to list item due to automated upgrade area url mappings being applied.");
                else if (!wasCheckedOut && item.File != null)
                    item.File.UndoCheckOut();

                if (modified && settings.Publish && !wasCheckedOut)
                    PublishItems.PublishListItem(item, list,
                                                 new PublishItems.Settings(settings.Quiet, settings.Test,
                                                                           settings.LogFile),
                                                 "\"stsadm.exe -o replacefieldvalues\"");
            }
            Log(settings, "Finished Processing List: " + list.DefaultViewUrl + "\r\n");
        }

        #endregion

        #region Utility Methods

        /// <summary>
        /// Builds the replacement urls list.
        /// </summary>
        /// <param name="settings">The settings.</param>
        private static void BuildReplacementUrlsList(Settings settings)
        {
            // We need to build a local copy of the replacement urls so that we don't have to keep referring back
            // to the list for the information.
            upgradeUrls = new List<ReplacementSettings>();
            foreach (SPListItem item in settings.UpgradeList.Items)
            {
                if ((bool)item["ShouldRedirect"])
                {
                    ReplacementSettings rs = new ReplacementSettings();
                    rs.ReplaceString = (string)item["V3ServerRelativeUrl"];
                    string str = (string) item["V2ServerRelativeUrl"];

                    // Need to find out if the redirect points under an existing item:
                    // "/Divisions" pointing to "/Topics/Divisions" will cause the resultant urls to be "/Topics/Topics/Divisions"
                    // which is incorrect and must be prevented
                    //string web = str.Substring(str.LastIndexOf('/') + 1);
                    string testUrl = rs.ReplaceString.Substring(0, rs.ReplaceString.LastIndexOf('/'));
                    bool excludeItem = false;
                    foreach (SPListItem item2 in settings.UpgradeList.Items)
                    {
                        if (((string)item2["V2ServerRelativeUrl"]).ToLowerInvariant() == testUrl.ToLowerInvariant())
                        {
                            if (Regex.IsMatch(str, "(?i:^/c[0-9]+/)"))
                                continue;  // Allow the item to be processed if it's a bucket web item.

                            excludeItem = true;
                            Log(settings, string.Format("Excluding \"{0}\" from search due to possible sub-site issues with {1}.", str, item2["V2ServerRelativeUrl"]));
                            break;
                        }
                    }
                    if (excludeItem)
                        continue;
                    
                    rs.Regex = new Regex("(?i:" + str + "|" + SPEncode.UrlEncodeAsUrl(str) + ")");

                    upgradeUrls.Add(rs);
                }
            }
        }


        /// <summary>
        /// Logs the specified message using the provided settings.
        /// </summary>
        /// <param name="settings">The settings.</param>
        /// <param name="message">The message.</param>
        private static void Log(Settings settings, string message)
        {
            if (!settings.Quiet)
                Console.WriteLine(message);

            if (settings.LogFile != null)
            {
                File.AppendAllText(settings.LogFile, DateTime.Now + ": " + message + "\r\n");
            }
        }

        #endregion

        #region Internal Classes

        /// <summary>
        /// Data class for storing the V2ServerRelativeUrl (as a Regex) and the 
        /// V3ServerRelativeUrl (or replacement string)
        /// </summary>
        private class ReplacementSettings
        {
            public string ReplaceString;
            public Regex Regex;
        }

        /// <summary>
        /// Data class used for passing user provided settings into the various worker methods.
        /// </summary>
        private class Settings
        {
            public string FieldName;
            public bool Publish;
            public bool Quiet;
            public bool Test;
            public string LogFile;
            public bool UseInternalFieldName;
            public SPList UpgradeList;
            public bool WebPartsOnly;
            public bool ListsOnly;
        }

        #endregion
    }
}
