using System;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class CopyListSecurity : SPOperation
    {
                /// <summary>
        /// Initializes a new instance of the <see cref="CopyListSecurity"/> class.
        /// </summary>
        public CopyListSecurity()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sourceurl", "sourceurl", true, null, new SPUrlValidator(), "Please specify the list view URL to copy security from."));
            parameters.Add(new SPParam("targeturl", "targeturl", true, null, new SPUrlValidator(), "Please specify the list view URL to copy security to."));
            parameters.Add(new SPParam("quiet", "quiet", false, null, null));
            parameters.Add(new SPParam("includeitemsecurity", "items"));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nCopies a list's security settings from one list to another.\r\n\r\nParameters:\r\n\t");
            sb.Append("-sourceurl <list view url to copy security from>\r\n\t");
            sb.Append("-targeturl <list view url to copy security to>\r\n\t");
            sb.Append("[-quiet]\r\n\t");
            sb.Append("[-includeitemsecurity]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string sourceUrl = Params["sourceurl"].Value;
            string targetUrl = Params["targeturl"].Value;
            bool quiet = Params["quiet"].UserTypedIn;
            bool includeItemSecurity = Params["includeitemsecurity"].UserTypedIn;

            using (SPSite sourceSite = new SPSite(sourceUrl))
            using (SPSite targetSite = new SPSite(targetUrl))
            using (SPWeb sourceWeb = sourceSite.OpenWeb())
            using (SPWeb targetWeb = targetSite.OpenWeb())
            {
                SPList sourceList = Utilities.GetListFromViewUrl(sourceWeb, sourceUrl);
                SPList targetList = Utilities.GetListFromViewUrl(targetWeb, targetUrl);

                if (sourceList == null)
                    throw new SPException("Source list was not found.");
                if (targetList == null)
                    throw new SPException("Target list was not found.");

                CopySecurity(sourceList, targetList, targetWeb, includeItemSecurity, quiet);
            }
            return 1;
        }


        #endregion


        /// <summary>
        /// Copies the security.
        /// </summary>
        /// <param name="sourceList">The source list.</param>
        /// <param name="targetList">The target list.</param>
        /// <param name="targetWeb">The target web.</param>
        /// <param name="includeItemSecurity">if set to <c>true</c> [include item security].</param>
        /// <param name="quiet">if set to <c>true</c> [quiet].</param>
        internal static void CopySecurity(SPList sourceList, SPList targetList, SPWeb targetWeb, bool includeItemSecurity, bool quiet)
        {
            if (!quiet)
                Console.WriteLine("Start Time: {0}.", DateTime.Now);

            try
            {
                if (sourceList.WriteSecurity != targetList.WriteSecurity)
                    targetList.WriteSecurity = sourceList.WriteSecurity;

                if (sourceList.ReadSecurity != targetList.ReadSecurity)
                    targetList.ReadSecurity = sourceList.ReadSecurity;

                // Set the security on the list itself.
                SetObjectSecurity(targetWeb, sourceList, targetList, quiet, targetList.RootFolder.ServerRelativeUrl);

                // Set the security on any folders in the list.
                SetFolderSecurity(targetWeb, sourceList, targetList, quiet);

                if (includeItemSecurity)
                {
                    // Set the security on list items.
                    SetListItemSecurity(targetWeb, sourceList, targetList, quiet);
                }

                targetList.Update();
            }
            finally
            {
                if (!quiet)
                    Console.WriteLine("Finish Time: {0}.\r\n", DateTime.Now);
            }
        }

        /// <summary>
        /// Sets the list item security.
        /// </summary>
        /// <param name="targetWeb">The target web.</param>
        /// <param name="sourceList">The source list.</param>
        /// <param name="targetList">The target list.</param>
        /// <param name="quiet">if set to <c>true</c> [quiet].</param>
        private static void SetListItemSecurity(SPWeb targetWeb, SPList sourceList, SPList targetList, bool quiet)
        {
            foreach (SPListItem sourceItem in sourceList.Items)
            {
                SPListItem targetItem = null;
                foreach (SPListItem i in targetList.Items)
                {
                    if (sourceItem.Name == i.Name)
                    {
                        targetItem = i;
                        break;
                    }
                }
                if (targetItem == null)
                    continue;

                SetObjectSecurity(targetWeb, sourceItem, targetItem, quiet, sourceItem.Name);
            }
        }

        /// <summary>
        /// Sets the folder security.
        /// </summary>
        /// <param name="targetWeb">The target web.</param>
        /// <param name="sourceList">The source list.</param>
        /// <param name="targetList">The target list.</param>
        /// <param name="quiet">if set to <c>true</c> [quiet].</param>
        private static void SetFolderSecurity(SPWeb targetWeb, SPList sourceList, SPList targetList, bool quiet)
        {
            foreach (SPListItem sourceFolder in sourceList.Folders)
            {
                SPListItem targetFolder = null;
                foreach (SPListItem f in targetList.Folders)
                {
                        
                    if (f.Folder.ServerRelativeUrl.Substring(targetList.RootFolder.ServerRelativeUrl.Length) ==
                        sourceFolder.Folder.ServerRelativeUrl.Substring(sourceList.RootFolder.ServerRelativeUrl.Length))
                    {
                        targetFolder = f;
                        break;
                    }
                }
                if (targetFolder == null)
                    continue;

                SetObjectSecurity(targetWeb, sourceFolder, targetFolder, quiet, targetFolder.Folder.ServerRelativeUrl);
            }
        }

        /// <summary>
        /// Sets the object security.
        /// </summary>
        /// <param name="targetWeb">The target web.</param>
        /// <param name="sourceObject">The source object.</param>
        /// <param name="targetObject">The target object.</param>
        /// <param name="quiet">if set to <c>true</c> [quiet].</param>
        /// <param name="itemName">Name of the item.</param>
        private static void SetObjectSecurity(SPWeb targetWeb, ISecurableObject sourceObject, ISecurableObject targetObject, bool quiet, string itemName)
        {
            if (!sourceObject.HasUniqueRoleAssignments && targetObject.HasUniqueRoleAssignments)
            {
                if (!quiet)
                    Console.WriteLine("Progress: Setting target object to inherit permissions from parent for \"{0}\".", itemName);
                targetObject.ResetRoleInheritance();
                return;
            }
            else if (sourceObject.HasUniqueRoleAssignments && !targetObject.HasUniqueRoleAssignments)
            {
                if (!quiet)
                    Console.WriteLine("Progress: Breaking target object inheritance from parent for \"{0}\".", itemName);
                targetObject.BreakRoleInheritance(false);
            }
            else if (!sourceObject.HasUniqueRoleAssignments && !targetObject.HasUniqueRoleAssignments)
            {
                if (!quiet)
                    Console.WriteLine("Progress: Ignoring \"{0}\".  Target object and source object both inheritance from parent.", itemName);
                return; // Both are inheriting so don't change.
            }

            foreach (SPRoleAssignment ra in sourceObject.RoleAssignments)
            {
                SPRoleAssignment existingRoleAssignment = GetRoleAssignement(targetObject, ra.Member);

                if (existingRoleAssignment == null)
                {

                    targetObject.RoleAssignments.Add(ra);
                        
                    existingRoleAssignment = GetRoleAssignement(targetObject, ra.Member);
                    if (existingRoleAssignment != null)
                    {
                        if (!quiet)
                            Console.WriteLine("Progress: Added \"{0}\" to target object \"{1}\".", ra.Member, itemName);
                        continue;
                    }

                    SPRoleAssignment newRA = new SPRoleAssignment(ra.Member);
                    foreach (SPRoleDefinition rd in ra.RoleDefinitionBindings)
                    {
                        if (rd.Name == "Limited Access")
                            continue;

                        SPRoleDefinition existingRoleDef = targetWeb.RoleDefinitions[rd.Name];
                        if (existingRoleDef == null)
                        {
                            existingRoleDef = new SPRoleDefinition();
                            existingRoleDef.BasePermissions = rd.BasePermissions;
                            existingRoleDef.Description = rd.Description;
                            existingRoleDef.Update();
                            targetWeb.RoleDefinitions.Add(existingRoleDef);
                        }
                        newRA.RoleDefinitionBindings.Add(existingRoleDef);
                    }
                    if (newRA.RoleDefinitionBindings.Count == 0)
                    {
                        if (!quiet)
                            Console.WriteLine("Progress: Unable to add \"{0}\" to target object \"{1}\" (principals with only \"Limited Access\" cannot be added).", ra.Member, itemName);
                        continue;
                    }

                    targetObject.RoleAssignments.Add(newRA);

                    existingRoleAssignment = GetRoleAssignement(targetObject, ra.Member);
                    if (existingRoleAssignment == null)
                    {
                        Console.WriteLine("Progress: Unable to add \"{0}\" to target object \"{1}\".", ra.Member, itemName);
                    }
                    else
                    {
                        if (!quiet)
                            Console.WriteLine("Progress: Added \"{0}\" to target object \"{1}\".", ra.Member, itemName);
                    }
                }
            }
        }

        /// <summary>
        /// Gets the role assignement.
        /// </summary>
        /// <param name="securableObject">The securable object.</param>
        /// <param name="principal">The principal.</param>
        /// <returns></returns>
        private static SPRoleAssignment GetRoleAssignement(ISecurableObject securableObject, SPPrincipal principal)
        {
            SPRoleAssignment ra = null;
            try
            {
                ra = securableObject.RoleAssignments.GetAssignmentByPrincipal(principal);
            }
            catch (ArgumentException)
            { }
            return ra;
        }
    }
}
