using System;
using System.Collections;
using System.IO;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Deployment;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class ExportList : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ExportList"/> class.
        /// </summary>
        public ExportList()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the list view URL."));
            parameters.Add(new SPParam("filename", "f", true, null, new SPNonEmptyValidator(), "Please specify the filename."));
            parameters.Add(new SPParam("overwrite", "over", false, null, null));
            parameters.Add(new SPParam("quiet", "quiet", false, null, null));
            parameters.Add(new SPParam("includeusersecurity", "security", false, null, null));
            parameters.Add(new SPParam("haltonwarning", "warning", false, null, null));
            parameters.Add(new SPParam("haltonfatalerror", "error", false, null, null));
            parameters.Add(new SPParam("nologfile", "nolog", false, null, null));
            parameters.Add(new SPParam("versions", "v", false, SPIncludeVersions.All.ToString(), new SPIntRangeValidator(1, 4), "Please specify the version settings."));
            parameters.Add(new SPParam("cabsize", "csize", false, null, new SPIntRangeValidator(1, 0x400), "Please specify the cab size."));
            parameters.Add(new SPParam("nofilecompression", "nofilecompression", false, null, null));
            
            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nExports a list.\r\n\r\nParameters:\r\n\t");
            sb.Append("-url <list view url>\r\n\t");
            sb.Append("-filename <export file name>\r\n\t");
            sb.Append("[-overwrite]\r\n\t");
            sb.Append("[-includeusersecurity]\r\n\t");
            sb.Append("[-haltonwarning]\r\n\t");
            sb.Append("[-haltonfatalerror]\r\n\t");
            sb.Append("[-nologfile]\r\n\t");
            sb.Append("[-versions <1-4>\r\n");
            sb.Append("\t\t1 - Last major version for files and list items\r\n");
            sb.Append("\t\t2 - The current version, either the last major or the last minor\r\n");
            sb.Append("\t\t3 - Last major and last minor version for files and list items\r\n");
            sb.Append("\t\t4 - All versions for files and list items (default)]\r\n\t");
            sb.Append("[-cabsize <integer from 1-1024 megabytes> (default: 25)]\r\n\t");
            sb.Append("[-nofilecompression]\r\n\t");
            sb.Append("[-quiet]");
            
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);


            string url = Params["url"].Value;
            bool compressFile = !Params["nofilecompression"].UserTypedIn;
            string filename = Params["filename"].Value;
            bool overwrite = Params["overwrite"].UserTypedIn;
            bool quiet = Params["quiet"].UserTypedIn;
            bool haltOnWarning = Params["haltonwarning"].UserTypedIn;
            bool haltOnFatalError = Params["haltonfatalerror"].UserTypedIn;
            bool includeusersecurity = Params["includeusersecurity"].UserTypedIn;
            bool logFile = !Params["nologfile"].UserTypedIn;
            SPIncludeVersions versions = SPIncludeVersions.All;
            SPExportObject exportObject = new SPExportObject();
            SPExportSettings settings = new SPExportSettings();
            SPExport export = new SPExport(settings);
            int cabSize = 0;

            if (Params["cabsize"].UserTypedIn)
            {
                cabSize = int.Parse(Params["cabsize"].Value);
            }

            if (Params["versions"].UserTypedIn)
                versions = (SPIncludeVersions)Enum.Parse(typeof(SPIncludeVersions), Params["versions"].Value);


            exportObject.Type = SPDeploymentObjectType.List;
            exportObject.IncludeDescendants = SPIncludeDescendants.All;
            ExportHelper.SetupExportObjects(settings, cabSize, compressFile, filename, haltOnFatalError, haltOnWarning, includeusersecurity, logFile, overwrite, quiet, versions);

            PerformExport(export, exportObject, settings, logFile, quiet, url);


            return 1;
        }

        #endregion

        /// <summary>
        /// Performs the export.
        /// </summary>
        /// <param name="export">The export.</param>
        /// <param name="exportObject">The export object.</param>
        /// <param name="settings">The settings.</param>
        /// <param name="logFile">if set to <c>true</c> [log file].</param>
        /// <param name="quiet">if set to <c>true</c> [quiet].</param>
        /// <param name="url">The URL.</param>
        internal static void PerformExport(SPExport export, SPExportObject exportObject, SPExportSettings settings, bool logFile, bool quiet, string url)
        {
            using (SPSite site = new SPSite(url))
            {
                ValidateUser(site);

                using (SPWeb web = site.OpenWeb())
                {
                    SPList list = Utilities.GetListFromViewUrl(web, url);

                    if (list == null)
                    {
                        throw new Exception("List not found.");
                    }

                    settings.SiteUrl = web.Url;
                    exportObject.Id = list.ID;
                }
            }

            settings.ExportObjects.Add(exportObject);


            try
            {
                export.Run();
                if (!quiet)
                {
                    ArrayList dataFiles = settings.DataFiles;
                    if (dataFiles != null)
                    {
                        Console.WriteLine();
                        Console.WriteLine("File(s) generated: ");
                        for (int i = 0; i < dataFiles.Count; i++)
                        {
                            Console.WriteLine("\t{0}", Path.Combine(settings.FileLocation, (string)dataFiles[i]));
                            Console.WriteLine();
                        }
                        Console.WriteLine();

                    }
                }
            }
            finally
            {
                if (logFile)
                {
                    Console.WriteLine();
                    Console.WriteLine("Log file generated: ");
                    Console.WriteLine("\t{0}", settings.LogFilePath);
                }
            }
        }


        /// <summary>
        /// Validates the user.
        /// </summary>
        /// <param name="site">The site.</param>
        internal static void ValidateUser(SPSite site)
        {
            if (site == null)
                throw new ArgumentNullException("site");

            bool isSiteAdmin = false;
            if ((site.RootWeb != null) && (site.RootWeb.CurrentUser != null))
            {
                isSiteAdmin = site.RootWeb.CurrentUser.IsSiteAdmin;
            }
            if (isSiteAdmin)
            {
                return;
            }
            try
            {
                isSiteAdmin = SPFarm.Local.CurrentUserIsAdministrator();
            }
            catch
            {
                isSiteAdmin = false;
            }
            if (!isSiteAdmin)
                throw new SPException(SPResource.GetString("AccessDenied", new object[0]));
        }


    }
}
