using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class ExportListItem2 : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ExportListItem2"/> class.
        /// </summary>
        public ExportListItem2()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the list view url to export items from."));
            parameters.Add(new SPParam("path", "p", true, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("id", "id", false, null, new SPNonEmptyValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nExports list items to disk (exported results can be used with addlistitem).\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <list view url to export from>");
            sb.Append("\r\n\t-path <export path>");
            sb.Append("\r\n\t[-id <list item ID (separate multiple items with a comma)>]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            string manifestPath = Params["path"].Value.TrimEnd('\\', '/');

            List<int> ids = new List<int>();
            if (Params["id"].UserTypedIn)
            {
                string[] ids2 = Params["id"].Value.Split(',');
                ids = new List<int>(ids2.Length);
                for (int i = 0; i < ids2.Length; i++)
                    if (!string.IsNullOrEmpty(ids2[i]))
                        ids.Add(int.Parse(ids2[i]));
            }

            using (SPSite site = new SPSite(url))
            using (SPWeb web = site.OpenWeb())
            {
                SPList list = Utilities.GetListFromViewUrl(web, url);

                if (list == null)
                    throw new SPException("List was not found.");

                SaveItemData(GetItemData(web, list, ids), manifestPath);
            }
            return 1;
        }

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Directory.Exists(Params["path"].Value))
                throw new SPException("The path specified already exists.  Specify a new path for the path parameter.");

            base.Validate(keyValues);
        }

        #endregion


        /// <summary>
        /// Gets the item data.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="list">The list.</param>
        /// <param name="ids">The ids.</param>
        /// <returns></returns>
        private static List<ItemInfo> GetItemData(SPWeb web, SPList list, List<int> ids)
        {
            List<ItemInfo> itemData = new List<ItemInfo>();

            foreach (SPListItem item in list.Items)
            {
                if (!(ids.Count == 0 || ids.Contains(item.ID)))
                    continue;

                ItemInfo info = new ItemInfo();
                itemData.Add(info);
                info.ID = item.ID;
                
                if (item.File != null)
                {
                    info.File = new FileDetails(item.File.OpenBinary(), item.File.Name, item.File.Author, item.File.TimeCreated);
                    info.Title = item.File.Name;
                }
                else
                    info.Title = item.Title;

                info.FolderUrl = item.Url.Substring(list.RootFolder.Url.ToString().Length, item.Url.LastIndexOf("/") - list.RootFolder.Url.ToString().Length);

                try
                {
                    foreach (string fileName in item.Attachments)
                    {
                        SPFile file = web.GetFile(item.Attachments.UrlPrefix + fileName);
                        info.Attachments.Add(new FileDetails(file.OpenBinary(), file.Name, file.Author, file.TimeCreated));
                    }
                }
                catch (ArgumentException)
                {}

                foreach (SPField field in list.Fields)
                {
                    if (!field.ReadOnlyField && 
                        field.InternalName != "Attachments" && 
                        field.InternalName != "FileLeafRef" &&
                        item[field.InternalName] != null)
                    {
                        info.FieldData.Add(field.InternalName, item[field.InternalName].ToString());
                    }
                }
            }
            return itemData;
        }

        /// <summary>
        /// Gets the item data from XML.
        /// </summary>
        /// <param name="itemData">The item data.</param>
        /// <param name="manifestPath">The manifest path.</param>
        private static void SaveItemData(List<ItemInfo> itemData, string manifestPath)
        {
            if (string.IsNullOrEmpty(manifestPath))
                throw new ArgumentNullException("manifest", "No directory was specified for the manifest.");

            if (!Directory.Exists(manifestPath))
                Directory.CreateDirectory(manifestPath);

            string dataPath = Path.Combine(manifestPath, "Data");

            StringBuilder sb = new StringBuilder();

            XmlTextWriter xmlWriter = new XmlTextWriter(new StringWriter(sb));
            xmlWriter.Formatting = Formatting.Indented;

            xmlWriter.WriteStartElement("Items");

            foreach (ItemInfo info in itemData)
            {
                xmlWriter.WriteStartElement("Item");

                if (info.File != null)
                {
                    string folder = Path.Combine(dataPath, info.FolderUrl.Trim('\\', '/')).Replace("/", "\\");
                    if (!Directory.Exists(folder))
                        Directory.CreateDirectory(folder);

                    xmlWriter.WriteAttributeString("File", Path.Combine(folder, info.File.Name));
                    xmlWriter.WriteAttributeString("Author", info.File.Author.LoginName);
                    xmlWriter.WriteAttributeString("CreatedDate", info.File.CreatedDate.ToString());
                    File.WriteAllBytes(Path.Combine(folder, info.File.Name), info.File.File);
                }
                xmlWriter.WriteAttributeString("LeafName", info.Title);
                xmlWriter.WriteAttributeString("FolderUrl", info.FolderUrl);
               
                xmlWriter.WriteStartElement("Fields");
                foreach (string key in info.FieldData.Keys)
                {
                    xmlWriter.WriteStartElement("Field");
                    xmlWriter.WriteAttributeString("Name", key);
                    xmlWriter.WriteString(info.FieldData[key]);
                    xmlWriter.WriteEndElement(); // Field
                }
                xmlWriter.WriteEndElement(); // Fields

                xmlWriter.WriteStartElement("Attachments");
                foreach (FileDetails file in info.Attachments)
                {
                    string folder = Path.Combine(Path.Combine(dataPath, info.FolderUrl.Trim('\\', '/')).Replace("/", "\\"), "item_" + info.ID);
                    if (!Directory.Exists(folder))
                        Directory.CreateDirectory(folder);
                    
                    xmlWriter.WriteElementString("Attachment", Path.Combine(folder, file.Name));

                    File.WriteAllBytes(Path.Combine(folder, file.Name), file.File);
                }
                xmlWriter.WriteEndElement(); // Attachments

                xmlWriter.WriteEndElement(); // Item
            }

            xmlWriter.WriteEndElement();
            xmlWriter.Flush();

            File.WriteAllText(Path.Combine(manifestPath, "Manifest.xml"), sb.ToString());
        }

        #region Private Classes

        private class FileDetails
        {
            public byte[] File = null;
            public string Name = null;
            public SPUser Author = null;
            public DateTime CreatedDate = DateTime.Now;
            public FileDetails(byte[] file, string name, SPUser author, DateTime createdDate)
            {
                File = file;
                Name = name;
                Author = author;
                CreatedDate = createdDate;
            }
        }
        private class ItemInfo
        {
            public FileDetails File = null;
            public string FolderUrl = null;
            public List<FileDetails> Attachments = new List<FileDetails>();
            public Dictionary<string, string> FieldData = new Dictionary<string, string>();
            public int ID = -1;
            public string Title = null;
        }

        #endregion




        /// <summary>
        /// Saves the files.  This method is not used by this class but was created simply to demonstrate how to save files
        /// from a list (without the clutter of the manifest details).
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="list">The list.</param>
        /// <param name="savePath">The save path.</param>
        internal static void SaveFiles(SPWeb web, SPList list, string savePath)
        {
            if (string.IsNullOrEmpty(savePath))
                throw new ArgumentNullException("savePath", "No directory was specified for the files.");

            if (!Directory.Exists(savePath))
                Directory.CreateDirectory(savePath);


            foreach (SPListItem item in list.Items)
            {
                string folderUrl = item.Url.Substring(list.RootFolder.Url.ToString().Length, item.Url.LastIndexOf("/") - list.RootFolder.Url.ToString().Length);
                if (item.File != null)
                {

                    string folder = Path.Combine(savePath, folderUrl.Trim('\\', '/')).Replace("/", "\\");
                    if (!Directory.Exists(folder))
                        Directory.CreateDirectory(folder);

                    File.WriteAllBytes(Path.Combine(folder, item.File.Name), item.File.OpenBinary());
                }


                try
                {
                    foreach (string fileName in item.Attachments)
                    {
                        SPFile file = web.GetFile(item.Attachments.UrlPrefix + fileName);

                        string folder = Path.Combine(Path.Combine(savePath, folderUrl.Trim('\\', '/')).Replace("/", "\\"), "item_" + item.ID);
                        if (!Directory.Exists(folder))
                            Directory.CreateDirectory(folder);


                        File.WriteAllBytes(Path.Combine(folder, file.Name), file.OpenBinary());

                    }
                }
                catch (ArgumentException)
                { }

            }

        }

    }

}
