using System;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Deployment;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class ImportList : SPOperation
    {
        protected string m_targetUrl;
        protected string m_sourceUrl;
        protected bool m_retargetLinks = false;
        protected SPList m_targetList = null;
        protected SPSite m_targetSite = null;
        protected SPWeb m_targetWeb = null;

        /// <summary>
        /// Initializes a new instance of the <see cref="ImportList"/> class.
        /// </summary>
        public ImportList()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the URL to import the list to."));
            parameters.Add(new SPParam("filename", "f", true, null, new SPNonEmptyValidator(), "Please specify the filename."));
            parameters.Add(new SPParam("quiet", "quiet"));
            parameters.Add(new SPParam("includeusersecurity", "security"));
            parameters.Add(new SPParam("haltonwarning", "warning"));
            parameters.Add(new SPParam("haltonfatalerror", "error"));
            parameters.Add(new SPParam("nologfile", "nolog"));
            parameters.Add(new SPParam("updateversions", "updatev", false, SPUpdateVersions.Append.ToString(), new SPIntRangeValidator(1, 3), "Please specify the updateversions setting."));
            parameters.Add(new SPParam("nofilecompression", "nofilecompression"));
            parameters.Add(new SPParam("retargetlinks", "retargetlinks"));
            parameters.Add(new SPParam("sourceurl", "source", false, null, new SPUrlValidator(), "Please specify the URL of the source site (must be on the same farm)."));
            parameters.Add(new SPParam("retainobjectidentity", "retainid"));
            parameters.Add(new SPParam("copysecuritysettings", "copysecurity"));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nImports a list.\r\n\r\nParameters:\r\n\t");
            sb.Append("-url <url of a web site to import to>\r\n\t");
            sb.Append("-filename <import file name>\r\n\t");
            sb.Append("[-includeusersecurity]\r\n\t");
            sb.Append("[-haltonwarning]\r\n\t");
            sb.Append("[-haltonfatalerror]\r\n\t");
            sb.Append("[-nologfile]\r\n\t");
            sb.Append("[-updateversions <1-3>\r\n");
            sb.Append("\t\t1 - Add new versions to the current file (default)\r\n");
            sb.Append("\t\t2 - Overwrite the file and all its versions (delete then insert)\r\n");
            sb.Append("\t\t3 - Ignore the file if it exists on the destination]\r\n\t");
            sb.Append("[-nofilecompression]\r\n\t");
            sb.Append("[-quiet]\r\n\t");
            sb.Append("[-retargetlinks (resets links pointing to the source to now point to the target)]\r\n\t");
            sb.Append("[-sourceurl <url to a view of the original list> (use if retargetlinks)]\r\n\t");
            sb.Append("[-retainobjectidentity]\r\n\t");
            sb.Append("[-copysecuritysettings (must provide sourceurl and includeusersecurity)]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string sourceUrl = Params["url"].Value;
            bool compressFile = !Params["nofilecompression"].UserTypedIn;
            string filename = Params["filename"].Value;
            bool quiet = Params["quiet"].UserTypedIn;
            bool haltOnWarning = Params["haltonwarning"].UserTypedIn;
            bool haltOnFatalError = Params["haltonfatalerror"].UserTypedIn;
            bool includeusersecurity = Params["includeusersecurity"].UserTypedIn;
            bool logFile = !Params["nologfile"].UserTypedIn;
            bool retainObjectIdentity = Params["retainobjectidentity"].UserTypedIn;
            bool copySecurity = Params["copysecuritysettings"].UserTypedIn;

            m_retargetLinks = Params["retargetlinks"].UserTypedIn;
            m_sourceUrl = Params["sourceurl"].Value;
            m_targetUrl = Params["url"].Value;

            SPUpdateVersions updateVersions = SPUpdateVersions.Append;

            SPImportSettings settings = new SPImportSettings();
            SPImport import = new SPImport(settings);
            
            if (Params["updateversions"].UserTypedIn)
                updateVersions = (SPUpdateVersions)Enum.Parse(typeof(SPUpdateVersions), Params["updateversions"].Value);


            ImportHelper.SetupImportObject(settings, compressFile, filename, haltOnFatalError, haltOnWarning, includeusersecurity, logFile, quiet, updateVersions, retainObjectIdentity);

            try
            {
                m_targetSite = new SPSite(m_targetUrl);
                m_targetWeb = m_targetSite.AllWebs[Utilities.GetServerRelUrlFromFullUrl(m_targetUrl)];

                PerformImport(import, settings, logFile, sourceUrl);

                // If the list is a discussion list then attempt to resolve flattened threads.
                if (m_targetList != null)
                    SiteCollectionSettings.RepairSiteCollectionImportedFromSubSite.RepairDiscussionList(m_targetSite, m_targetList);

                if (includeusersecurity && !string.IsNullOrEmpty(m_sourceUrl) && copySecurity)
                {
                    using (SPSite sourceSite = new SPSite(m_sourceUrl))
                    using (SPWeb sourceWeb = sourceSite.OpenWeb())
                    {
                        SPList sourceList = Utilities.GetListFromViewUrl(sourceWeb, m_sourceUrl);

                        if (sourceList != null)
                        {
                            if (m_targetList != null)
                                CopyListSecurity.CopySecurity(sourceList, m_targetList, m_targetWeb, true, quiet);
                        }
                    }
                }
            }
            finally
            {
                if (m_targetSite != null)
                    m_targetSite.Dispose();
                if (m_targetWeb != null)
                    m_targetWeb.Dispose();
            }
            return 1;
        }

        #endregion

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(System.Collections.Specialized.StringDictionary keyValues)
        {
            if (Params["retargetlinks"].UserTypedIn)
                Params["sourceurl"].IsRequired = true;

            if (Params["copysecuritysettings"].UserTypedIn)
            {
                Params["sourceurl"].IsRequired = true;
                Params["includeusersecurity"].IsRequired = true;
            }

            base.Validate(keyValues);
        }

        /// <summary>
        /// Performs the import.
        /// </summary>
        /// <param name="import">The import.</param>
        /// <param name="settings">The settings.</param>
        /// <param name="logFile">if set to <c>true</c> [log file].</param>
        /// <param name="sourceUrl">The Source URL.</param>
        internal void PerformImport(SPImport import, SPImportSettings settings, bool logFile, string sourceUrl)
        {
            using (SPSite site = new SPSite(sourceUrl))
            {
                ExportList.ValidateUser(site);

                using (SPWeb web = site.OpenWeb())
                {
                    settings.SiteUrl = site.Url;
                    settings.WebUrl = web.Url;
                }
            }
            import.ObjectImported += new EventHandler<SPObjectImportedEventArgs>(OnImported);

            try
            {
                import.Run();
            }
            finally
            {
                if (logFile)
                {
                    Console.WriteLine();
                    Console.WriteLine("Log file generated: ");
                    Console.WriteLine("\t{0}", settings.LogFilePath);
                    Console.WriteLine();
                }
            }
        }


        /// <summary>
        /// Called when [imported].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="eventArgs">The <see cref="Microsoft.SharePoint.Deployment.SPObjectImportedEventArgs"/> instance containing the event data.</param>
        public void OnImported(object sender, SPObjectImportedEventArgs eventArgs)
        {
            if (m_targetList == null && eventArgs.Type == SPDeploymentObjectType.List)
            {
                // Get the list for later processing.
                m_targetList = m_targetWeb.GetList(eventArgs.TargetUrl);
            }

            if (!m_retargetLinks)
                return;

            if (eventArgs.Type != SPDeploymentObjectType.ListItem)
                return;

            SPImport import = sender as SPImport;
            if (import == null)
                return;

            try
            {
                string url = eventArgs.SourceUrl; // This is not fully qualified so we need the user specified url for the site.
                using (SPSite site = new SPSite(m_sourceUrl))
                using (SPWeb web = site.OpenWeb(url, false))
                {
                    string targetUrl = m_targetSite.MakeFullUrl(eventArgs.TargetUrl);
                    SPListItem li = web.GetListItem(url);
                    int count = li.BackwardLinks.Count;
                    for (int i = count - 1; i >= 0; i--)
                    {
                        SPLink link = li.BackwardLinks[i];
                        using (SPWeb rweb = site.OpenWeb(link.ServerRelativeUrl, false))
                        {
                            object o = rweb.GetObject(link.ServerRelativeUrl);
                            if (o is SPFile)
                            {
                                SPFile f = o as SPFile;
                                f.ReplaceLink(eventArgs.SourceUrl, targetUrl);
                            }
                            if (o is SPListItem)
                            {
                                SPListItem l = o as SPListItem;
                                l.ReplaceLink(eventArgs.SourceUrl, targetUrl);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Links could not be retargeted for " + eventArgs.SourceUrl + "\r\n" + ex.Message);
            }
        } 


    }
}
