using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class ImportListField : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ImportListField"/> class.
        /// </summary>
        public ImportListField()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the list view URL."));
            parameters.Add(new SPParam("inputfile", "input", false, null, new SPDirectoryExistsAndValidFileNameValidator(), "Make sure a valid filename is provided."));
            parameters.Add(new SPParam("addtodefaultview", "adv"));
            SPEnumValidator fieldOptionsValidator = new SPEnumValidator(typeof(SPAddFieldOptions));
            parameters.Add(new SPParam("addfieldoptions", "fo", false, null, fieldOptionsValidator));
            parameters.Add(new SPParam("retainobjectidentity", "roi"));
            parameters.Add(new SPParam("sortindex", "index", false, null, new SPIntRangeValidator(0, int.MaxValue)));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nImports a field (column) into a list.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <list view URL>");
            sb.Append("\r\n\t-inputfile <input file containing field schema information>");
            sb.AppendFormat("\r\n\t[-addfieldoptions <{0}>", fieldOptionsValidator.DisplayValue);
            sb.Append("\r\n\t[-addtodefaultview]");
            sb.Append("\r\n\t[-retainobjectidentity]");
            sb.Append("\r\n\t[-sortindex <field order index>]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            string xml = File.ReadAllText(Params["inputfile"].Value);
            SPAddFieldOptions fieldOptions = SPAddFieldOptions.Default;
            if (Params["addfieldoptions"].UserTypedIn)
                fieldOptions = (SPAddFieldOptions) Enum.Parse(typeof (SPAddFieldOptions), Params["addfieldoptions"].Value, true);

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(xml);
            Guid id = new Guid(xmlDoc.DocumentElement.GetAttribute("ID"));
            if (!Params["retainobjectidentity"].UserTypedIn)
            {
                id = Guid.NewGuid();
                xmlDoc.DocumentElement.SetAttribute("ID", id.ToString());
                xml = xmlDoc.OuterXml;
            }

            SPList list = Utilities.GetListFromViewUrl(url);
            list.Fields.AddFieldAsXml(xml, Params["addtodefaultview"].UserTypedIn, fieldOptions);

            SPField field = list.Fields[id];
            if (Params["sortindex"].UserTypedIn)
            {
                int sortIndex = int.Parse(Params["sortindex"].Value);
                ReorderField(list, field, sortIndex);
            }

            return 1;
        }

        #endregion

        /// <summary>
        /// Reorders the field.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="field">The field.</param>
        /// <param name="sortIndex">The sort index.</param>
        internal static void ReorderField(SPList list, SPField field, int sortIndex)
        {
            if (field.Reorderable)
            {
                List<SPField> fields = new List<SPField>();
                int count = 0;
                bool added = false;
                // First add the reorderable fields
                for (int i = 0; i < list.Fields.Count; i++)
                {
                    if (list.Fields[i].Reorderable)
                    {
                        if (count == sortIndex)
                        {
                            added = true;
                            fields.Add(field);
                            count++;
                        }

                        if (list.Fields[i].Id == field.Id)
                            continue;

                        fields.Add(list.Fields[i]);
                        count++;
                    }
                }
                if (!added)
                    fields.Add(field);

                // Now add the non-reorderable fields
                for (int i = 0; i < list.Fields.Count; i++)
                {
                    if (!list.Fields[i].Reorderable)
                    {
                        fields.Add(list.Fields[i]);
                    }
                }

                StringBuilder sb = new StringBuilder();

                XmlTextWriter xmlWriter = new XmlTextWriter(new StringWriter(sb));
                xmlWriter.Formatting = Formatting.Indented;

                xmlWriter.WriteStartElement("Fields");

                for (int i = 0; i < fields.Count; i++)
                {
                    xmlWriter.WriteStartElement("Field");
                    xmlWriter.WriteAttributeString("Name", fields[i].InternalName);
                    xmlWriter.WriteEndElement();
                }

                xmlWriter.WriteEndElement();
                xmlWriter.Flush();

                using (SPWeb web = list.ParentWeb)
                {
                    ReorderFields(web, list, sb.ToString());
                }
            }
        }

        /// <summary>
        /// This function reorders the fields in the specified list programmatically as specified by the xmlFieldsOrdered parameter
        /// </summary>
        /// <param name="web">The SPWeb object containing the list</param>
        /// <param name="list">The SPList object to update</param>
        /// <param name="xmlFieldsOrdered">A string in XML-format specifying the field order by the location within a xml-tree</param>
        private static void ReorderFields(SPWeb web, SPList list, string xmlFieldsOrdered)
        {
            try
            {
                string fpRPCMethod = @"<?xml version=""1.0"" encoding=""UTF-8""?>
                 <Method ID=""0,REORDERFIELDS"">
                 <SetList Scope=""Request"">{0}</SetList>
                 <SetVar Name=""Cmd"">REORDERFIELDS</SetVar>
                 <SetVar Name=""ReorderedFields"">{1}</SetVar>
                 <SetVar Name=""owshiddenversion"">{2}</SetVar>
                 </Method>";

                // relookup list version in order to be able to update it
                list = web.Lists[list.ID];

                int currentVersion = list.Version;

                string version = currentVersion.ToString();
                string RpcCall = string.Format(fpRPCMethod, list.ID, SPHttpUtility.HtmlEncode(xmlFieldsOrdered), version);

                web.AllowUnsafeUpdates = true;

                Utilities.ProcessRpcResults(web.ProcessBatchData(RpcCall));
            }
            catch (System.Net.WebException err)
            {
                Console.WriteLine("WARNING:" + err.Message);
            }
        }

    }
}
