using System;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Workflow;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class PublishItems : SPOperation
    {
        private static Counts TaskCounts;

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PublishItems"/> class.
        /// </summary>
        public PublishItems()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", false, null, new SPNonEmptyValidator(), "Please specify the url to search."));
            parameters.Add(new SPParam("scope", "scope", true, null, new SPRegexValidator("(?i:^Farm$|^WebApplication$|^Site$|^Web$|^List$)")));
            parameters.Add(new SPParam("quiet", "q", false, null, null));
            parameters.Add(new SPParam("test", "t", false, null, null));
            parameters.Add(new SPParam("logfile", "log", false, null, new SPDirectoryExistsAndValidFileNameValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nPublishes all items at a given scope.  Use -test to verify what will be published before executing.\r\n\r\nParameters:");
            sb.Append("\r\n\t[-url <url to publish>]");
            sb.Append("\r\n\t-scope <Farm | WebApplication | Site | Web | List>");
            sb.Append("\r\n\t[-quiet]");
            sb.Append("\r\n\t[-test]");
            sb.Append("\r\n\t[-logfile <log file>]");

            Init(parameters, sb.ToString());
        }

        #endregion
        
        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            if (url != null)
                url = url.TrimEnd('/');

            Settings settings = new Settings();
            settings.LogFile = Params["logfile"].Value;
            settings.Quiet = Params["quiet"].UserTypedIn;
            settings.Test = Params["test"].UserTypedIn;
            TaskCounts = new Counts();

            string scope = Params["scope"].Value.ToLowerInvariant();

            if (scope == "farm")
            {
                foreach (SPService svc in SPFarm.Local.Services)
                {
                    if (!(svc is SPWebService))
                        continue;

                    foreach (SPWebApplication webApp in ((SPWebService)svc).WebApplications)
                    {
                        Publish(webApp, settings);
                    }
                }
            }
            else if (scope == "webapplication")
            {
                SPWebApplication webApp = SPWebApplication.Lookup(new Uri(url));
                Publish(webApp, settings);
            }
            else if (scope == "site")
            {
                using (SPSite site = new SPSite(url))
                {
                    Publish(site, settings);
                }
            }
            else if (scope == "web")
            {
                using (SPSite site = new SPSite(url))
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    Publish(web, settings);
                }
            }
            else if (scope == "list")
            {
                SPList list = Utilities.GetListFromViewUrl(url);
                Publish(list, settings);
            }

            Log(settings, string.Format("\r\n\r\nFinished Process: {0} Errors, {1} Items(s) Checked In, {2} Item(s) Published, {3} Item(s) Approved",
                TaskCounts.Errors, TaskCounts.Checkin, TaskCounts.Publish, TaskCounts.Approve));

            return 1;
        }

        #endregion

        #region SPOperation Misc Overrides

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["quiet"].UserTypedIn && Params["test"].UserTypedIn && !Params["logfile"].UserTypedIn)
                throw new SPSyntaxException("The quiet parameter and the test parameter are incompatible if no log file is specified.");

            if (Params["scope"].Value.ToLowerInvariant() == "farm" && Params["url"].UserTypedIn)
                throw new SPSyntaxException("The url parameter is not compatible with a scope of Farm.");
            
            if (Params["scope"].Value.ToLowerInvariant() != "farm" && !Params["url"].UserTypedIn)
                throw new SPSyntaxException("The url parameter is required if the scope is not Farm.");

            base.Validate(keyValues);
        }

        #endregion


        #region Publish Methods

        /// <summary>
        /// Publishes all list items for a given web application.
        /// </summary>
        /// <param name="webApp">The web app.</param>
        /// <param name="settings">The settings.</param>
        private static void Publish(SPWebApplication webApp, Settings settings)
        {
            Log(settings, "Processing Web Application: " + webApp.DisplayName);
            
            foreach (SPSite site in webApp.Sites)
            {
                try
                {
                    Publish(site, settings);
                }
                finally
                {
                    site.Dispose();
                }
            }

            Log(settings, "Finished Processing Web Application: " + webApp.DisplayName + "\r\n");
        }

        /// <summary>
        /// Publishes all list items for a given site collection.
        /// </summary>
        /// <param name="site">The site.</param>
        /// <param name="settings">The settings.</param>
        private static void Publish(SPSite site, Settings settings)
        {
            Log(settings, "Processing Site: " + site.ServerRelativeUrl);
            
            foreach (SPWeb web in site.AllWebs)
            {
                try
                {
                    Publish(web, settings);
                }
                finally
                {
                    web.Dispose();
                }
            }

            Log(settings, "Finished Processing Site: " + site.ServerRelativeUrl + "\r\n");
        }

        /// <summary>
        /// Publishes all list items for a given web.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="settings">The settings.</param>
        private static void Publish(SPWeb web, Settings settings)
        {
            Log(settings, "Processing Web: " + web.ServerRelativeUrl);

            foreach (SPList list in web.Lists)
            {
                Publish(list, settings);
            }

            Log(settings, "Finished Processing Web: " + web.ServerRelativeUrl + "\r\n");
        }

        /// <summary>
        /// Publishes all list items for a given list.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="settings">The settings.</param>
        private static void Publish(SPList list, Settings settings)
        {
            Log(settings, "Processing List: " + list.DefaultViewUrl);

            string source = "\"stsadm.exe -o publishitems\"";

            foreach (SPListItem item in list.Items)
            {
                PublishListItem(item, list, settings, source);
            }

            Log(settings, "Finished Processing List: " + list.DefaultViewUrl + "\r\n");
        }

        #endregion

        #region Primary Worker Methods

        /// <summary>
        /// Publishes the list item.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <param name="list">The list.</param>
        /// <param name="settings">The settings.</param>
        /// <param name="source">The source.</param>
        internal static void PublishListItem(SPListItem item, SPList list, Settings settings, string source)
        {
            if (TaskCounts == null)
                TaskCounts = new Counts();

            try
            {
                if (item.File != null)
                {
                    // We first need to handle the case in which we have a file which means that
                    // we have to deal with the possibility that the file may be checked out.
                    if (item.Level == SPFileLevel.Checkout)
                    {
                        // The file is checked out so we now need to check it in - we'll do a major
                        // checkin which will result in it being published.
                        if (!settings.Test)
                        {
                            item.File.CheckIn("Checked in by " + source, SPCheckinType.MajorCheckIn);
                            // We need to get the File's version of the SPListItem so that we get the changes.
                            // Calling item.Update() will fail because the file is no longer checked out.
                            item = item.File.Item; // If workflow is supported this should now be in a pending state.
                        }
                        TaskCounts.Checkin++;
                        TaskCounts.Publish++; // The major checkin causes it to be published so we'll track that as well.
                        Log(settings, string.Format("Checked in item: {0} ({1})", item.Title, item.Url));
                    }
                    else if (item.Level == SPFileLevel.Draft && item.ModerationInformation == null)
                    {
                        // The file isn't checked out but it is in a draft state so we need to publish it.
                        if (!settings.Test)
                        {
                            item.File.Publish("Published by " + source);
                            // We need to get the File's version of the SPListItem so that we get the changes.
                            // Calling item.Update() will fail because the file is no longer checked out.
                            item = item.File.Item; // If workflow is supported this should now be in a pending state.
                        }
                        TaskCounts.Publish++;
                        Log(settings, string.Format("Published item: {0} ({1})", item.Title, item.Url));
                    }
                }
            }
            catch (Exception ex)
            {
                TaskCounts.Errors++;
                Log(settings, string.Format("An error occured checking in an item:\r\n{0}", ex.Message));
            }

            if (item.ModerationInformation != null)
            {
                // If ModerationInformation is not null then the item supports content approval.
                if (item.File == null &&
                    (item.ModerationInformation.Status == SPModerationStatusType.Draft ||
                    item.ModerationInformation.Status == SPModerationStatusType.Pending))
                {
                    // If content approval is supported but no file is associated with the item then we have
                    // to treat it differently.  We simply set the status information directly.
                    try
                    {
                        if (!settings.Test)
                        {
                            // Because the SPListItem object has no direct approval method we have to 
                            // set the information directly (there's no SPFile object to use).
                            CancelWorkflows(settings, list, item);
                            item.ModerationInformation.Status = SPModerationStatusType.Approved;
                            item.ModerationInformation.Comment = "Approved by " + source;
                            item.Update(); // Because there's no SPFile object we don't have to worry about the item being checkedout for this to succeed as you can't check it out.
                        }
                        TaskCounts.Approve++;
                        Log(settings, string.Format("Approved item: {0} ({1})", item.Title, item.Url));
                    }
                    catch (Exception ex)
                    {
                        TaskCounts.Errors++;
                        Log(settings, string.Format("An error occured approving an item:\r\n{0}", ex.Message));
                    }
                }
                else
                {
                    // The item supports content approval and we have an SPFile object to work with.
                    try
                    {
                        if (item.ModerationInformation.Status == SPModerationStatusType.Pending)
                        {
                            // The item is pending so it's already been published - we just need to approve.
                            if (!settings.Test)
                            {
                                // Cancel any workflows.
                                CancelWorkflows(settings, list, item);
                                item.File.Approve("Approved by " + source);
                                // We don't need to re-retrieve the item as we're now done with it.
                            }
                            TaskCounts.Approve++;
                            Log(settings, string.Format("Approved item: {0} ({1})", item.Title, item.Url));
                        }
                    }
                    catch (Exception ex)
                    {
                        TaskCounts.Errors++;
                        Log(settings, string.Format("An error occured approving an item:\r\n{0}", ex.Message));
                    }

                    try
                    {
                        if (item.ModerationInformation.Status == SPModerationStatusType.Draft)
                        {
                            // The item is in a draft state so we have to first publish it and then approve it.
                            if (!settings.Test)
                            {
                                item.File.Publish("Published by " + source);
                                // Cancel any workflows.
                                CancelWorkflows(settings, list, item);
                                item.File.Approve("Approved by " + source);
                                // We don't need to re-retrieve the item as we're now done with it.
                            }
                            TaskCounts.Publish++;
                            TaskCounts.Approve++;
                            Log(settings, string.Format("Published item: {0} ({1})", item.Title, item.Url));
                        }
                    }
                    catch (Exception ex)
                    {
                        TaskCounts.Errors++;
                        Log(settings, string.Format("An error occured approving an item:\r\n{0}", ex.Message));
                    }
                }
            }
        }

        /// <summary>
        /// Cancels the workflows.  This code is a re-engineering of the code that Microsoft uses
        /// when approving an item via the browser.  That code is in Microsoft.SharePoint.ApplicationPages.ApprovePage.
        /// </summary>
        /// <param name="settings">The settings.</param>
        /// <param name="list">The list.</param>
        /// <param name="item">The item.</param>
        private static void CancelWorkflows(Settings settings, SPList list, SPListItem item)
        {
            if (list.DefaultContentApprovalWorkflowId != Guid.Empty &&
                item.DoesUserHavePermissions((SPBasePermissions.ApproveItems |
                                              SPBasePermissions.EditListItems)))
            {
                // If the user has rights to do so then we need to cancel any workflows that
                // are associated with the item.  This is based on how the 
                SPSecurity.RunWithElevatedPrivileges(
                    delegate
                        {
                            foreach (SPWorkflow workflow in item.Workflows)
                            {
                                if (workflow.ParentAssociation.Id !=
                                    list.DefaultContentApprovalWorkflowId)
                                {
                                    continue;
                                }
                                SPWorkflowManager.CancelWorkflow(workflow);
                                Log(settings,
                                    string.Format("Cancelling workflow {0} for item: {1} ({2})",
                                                  workflow.WebId, item.Title, item.Url));
                            }
                        });
            }
        }

        #endregion

        #region Utility Methods

        /// <summary>
        /// Logs the specified message using the provided settings.
        /// </summary>
        /// <param name="settings">The settings.</param>
        /// <param name="message">The message.</param>
        private static void Log(Settings settings, string message)
        {
            if (settings == null)
                return;

            if (!settings.Quiet)
                Console.WriteLine(message);

            if (settings.LogFile != null)
            {
                File.AppendAllText(settings.LogFile, DateTime.Now + ": " + message + "\r\n");
            }
        }

        #endregion
 
        #region Internal Classes

        /// <summary>
        /// Data class used for passing user provided settings into the various worker methods.
        /// </summary>
        internal class Settings
        {
            public bool Quiet;
            public bool Test;
            public string LogFile;

            /// <summary>
            /// Initializes a new instance of the <see cref="Settings"/> class.
            /// </summary>
            public Settings() {}
            
            /// <summary>
            /// Initializes a new instance of the <see cref="Settings"/> class.
            /// </summary>
            /// <param name="quiet">if set to <c>true</c> [quiet].</param>
            /// <param name="test">if set to <c>true</c> [test].</param>
            /// <param name="logFile">The log file.</param>
            public Settings(bool quiet, bool test, string logFile)
            {
                Quiet = quiet;
                Test = test;
                LogFile = logFile;
            }
        }

        /// <summary>
        /// Data class for storing counts of various actions performed.
        /// </summary>
        private class Counts
        {
            public int Checkin = 0;
            public int Approve = 0;
            public int Publish = 0;
            public int Errors = 0;
        }

        #endregion
    }
}
