using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using System.Text.RegularExpressions;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class ReplaceFieldValues : SPOperation
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ReplaceFieldValues"/> class.
        /// </summary>
        public ReplaceFieldValues()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", false, null, new SPNonEmptyValidator(), "Please specify the url to search."));
            parameters.Add(new SPParam("searchstring", "search", false, null, new SPNonEmptyValidator(), "Please specify the search string."));
            parameters.Add(new SPParam("replacestring", "replace", false, null, new SPNullOrNonEmptyValidator(), "Please specify the replace string."));
            parameters.Add(new SPParam("scope", "scope", true, null, new SPRegexValidator("(?i:^Farm$|^WebApplication$|^Site$|^Web$|^List$)")));
            parameters.Add(new SPParam("field", "f", false, null, new SPNonEmptyValidator(), "Please enter the field name."));
            parameters.Add(new SPParam("useinternalfieldname", "userinternal", false, null, null));
            parameters.Add(new SPParam("inputfile", "input", false, null, new SPDirectoryExistsAndValidFileNameValidator()));
            parameters.Add(new SPParam("inputfiledelimiter", "delimiter", false, "|", new SPNonEmptyValidator()));
            parameters.Add(new SPParam("inputfileisxml", "isxml"));
            parameters.Add(new SPParam("quiet", "q"));
            parameters.Add(new SPParam("test", "t"));
            parameters.Add(new SPParam("publish", "p"));
            parameters.Add(new SPParam("logfile", "log", false, null, new SPDirectoryExistsAndValidFileNameValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nReplaces all occurances of the search string with the replacement string.  Supports use of regular expressions.  Use -test to verify your replacements before executing.\r\n\r\nParameters:");
            sb.Append("\r\n\t[-url <url to search>]");
            sb.Append("\r\n\t{-inputfile <input file> |");
            sb.Append("\r\n\t -searchstring <regular expression string to search for>");
            sb.Append("\r\n\t -replacestring <replacement string>}");
            sb.Append("\r\n\t-scope <Farm | WebApplication | Site | Web | List>");
            sb.Append("\r\n\t[-field <field name>]");
            sb.Append("\r\n\t[-useinternalfieldname (if not present then the display name will be used)]");
            sb.Append("\r\n\t[-inputfiledelimiter <delimiter character to use in the input file (default is \"|\")>]");
            sb.Append("\r\n\t[-inputfileisxml (input is XML in the following format: <Replacements><Replacement><SearchString>string</SearchString><ReplaceString>string</ReplaceString></Replacement><Replacements>)");
            sb.Append("\r\n\t[-quiet]");
            sb.Append("\r\n\t[-test]");
            sb.Append("\r\n\t[-logfile <log file>]");
            sb.Append("\r\n\t[-publish]");

            Init(parameters, sb.ToString());
        }

        #endregion

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            if (url != null)
                url = url.TrimEnd('/');

            Settings settings = new Settings();
            settings.LogFile = Params["logfile"].Value;
            settings.FieldName = Params["field"].Value;
            settings.Publish = Params["publish"].UserTypedIn;
            settings.Quiet = Params["quiet"].UserTypedIn;
            settings.Test = Params["test"].UserTypedIn;
            settings.UseInternalFieldName = Params["useinternalfieldname"].UserTypedIn;
            
            if (Params["inputfile"].UserTypedIn)
            {
                ParseInputFile(settings, Params["inputfile"].Value, Params["inputfiledelimiter"].Value, Params["inputfileisxml"].UserTypedIn);
            }
            else
            {
                settings.SearchStrings.Add(new SearchReplaceData(Params["searchstring"].Value, Params["replacestring"].Value));
            }
            if (settings.SearchStrings.Count == 0)
                throw new SPException("No search strings were specified.");

            string scope = Params["scope"].Value.ToLowerInvariant();

            Log(settings, string.Format("Executing Command: {0}\r\n", command));

            if (scope == "farm")
            {
                foreach (SPService svc in SPFarm.Local.Services)
                {
                    if (!(svc is SPWebService))
                        continue;

                    foreach (SPWebApplication webApp in ((SPWebService)svc).WebApplications)
                    {
                        ReplaceValues(webApp, settings);
                    }
                }
            }
            else if (scope == "webapplication")
            {
                SPWebApplication webApp = SPWebApplication.Lookup(new Uri(url));
                ReplaceValues(webApp, settings);
            }
            else if (scope == "site")
            {
                using (SPSite site = new SPSite(url))
                {
                    ReplaceValues(site, settings);
                }
            }
            else if (scope == "web")
            {
                using (SPSite site = new SPSite(url))
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    ReplaceValues(web, settings);
                }
            }
            else if (scope == "list")
            {
                SPList list = Utilities.GetListFromViewUrl(url);
                ReplaceValues(list, settings);
            }

            return 1;
        }

        #endregion

        #region SPOperation Misc Overrides

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["quiet"].UserTypedIn && Params["test"].UserTypedIn && !Params["logfile"].UserTypedIn)
                throw new SPSyntaxException("The quiet parameter and the test parameter are incompatible if no log file is specified.");

            if (Params["scope"].Value.ToLowerInvariant() == "farm" && Params["url"].UserTypedIn)
                throw new SPSyntaxException("The url parameter is not compatible with a scope of Farm.");
            if (Params["scope"].Value.ToLowerInvariant() != "farm" && !Params["url"].UserTypedIn)
                throw new SPSyntaxException("The url parameter is required if the scope is not Farm.");

            if (Params["inputfile"].UserTypedIn && !File.Exists(Params["inputfile"].Value))
                throw new SPSyntaxException("The inputfile was not found.");

            if ((Params["searchstring"].UserTypedIn || Params["replacestring"].UserTypedIn) && Params["inputfile"].UserTypedIn)
                throw new SPSyntaxException("The searchstring and replacestring parameters are incompatible with the inputfile parameter.");

            if (!(Params["searchstring"].UserTypedIn || Params["replacestring"].UserTypedIn) && !Params["inputfile"].UserTypedIn)
                throw new SPSyntaxException("You must specify at least the searchstring and replacestring or the inputfile parameter.");

            if (!Params["inputfile"].UserTypedIn)
            {
                Params["searchstring"].IsRequired = true;
                Params["replacestring"].IsRequired = true;
            }

            if (Params["searchstring"].UserTypedIn)
            {
                Params["inputfile"].IsRequired = false;
                Params["replacestring"].IsRequired = true;
            }

            if (Params["inputfiledelimiter"].UserTypedIn && Params["inputfileisxml"].UserTypedIn)
                throw new SPSyntaxException("The parameters inputfileisxml and inputfiledelimiter are incompatible.");

            if (!Params["inputfile"].UserTypedIn && (Params["inputfiledelimiter"].UserTypedIn || Params["inputfileisxml"].UserTypedIn))
                throw new SPSyntaxException(
                    "The paramerts inputfiledelimiter and inputfileisxml are only valid when inputfile is used.");

            base.Validate(keyValues);
        }

        #endregion

        #region ReplaceValues Methods

        /// <summary>
        /// Replaces the values.
        /// </summary>
        /// <param name="webApp">The web app.</param>
        /// <param name="settings">The settings.</param>
        private static void ReplaceValues(SPWebApplication webApp, Settings settings)
        {
            Log(settings, "Processing Web Application: " + webApp.DisplayName);
            
            foreach (SPSite site in webApp.Sites)
            {
                try
                {
                    ReplaceValues(site, settings);
                }
                finally
                {
                    site.Dispose();
                }
            }

            Log(settings, "Finished Processing Web Application: " + webApp.DisplayName + "\r\n");
        }

        /// <summary>
        /// Replaces the values.
        /// </summary>
        /// <param name="site">The site.</param>
        /// <param name="settings">The settings.</param>
        private static void ReplaceValues(SPSite site, Settings settings)
        {
            Log(settings, "Processing Site: " + site.ServerRelativeUrl);
            
            foreach (SPWeb web in site.AllWebs)
            {
                try
                {
                    ReplaceValues(web, settings);
                }
                finally
                {
                    web.Dispose();
                }
            }

            Log(settings, "Finished Processing Site: " + site.ServerRelativeUrl + "\r\n");
        }

        /// <summary>
        /// Replaces the values.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="settings">The settings.</param>
        private static void ReplaceValues(SPWeb web, Settings settings)
        {
            Log(settings, "Processing Web: " + web.ServerRelativeUrl);

            foreach (SPList list in web.Lists)
            {
                ReplaceValues(list, settings);
            }

            Log(settings, "Finished Processing Web: " + web.ServerRelativeUrl + "\r\n");
        }

        /// <summary>
        /// Replaces the values.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="settings">The settings.</param>
        private static void ReplaceValues(SPList list, Settings settings)
        {
            Log(settings, "Processing List: " + list.DefaultViewUrl);

            foreach (SPListItem item in list.Items)
            {
                if (item.File != null && Utilities.IsCheckedOut(item) && !Utilities.IsCheckedOutByCurrentUser(item))
                {
                    continue;
                }
                bool wasCheckedOut = true;
                bool modified = false;

                foreach (SPField field in list.Fields)
                {
                    if (item[field.Id] == null || field.ReadOnlyField)
                        continue;

                    if ((list.Title.ToLowerInvariant() == "upgrade area url mapping" ||
                         list.DefaultViewUrl.ToLowerInvariant().IndexOf(UPGRADE_AREA_URL_LIST) >= 0)
                        && field.Title == "V2ServerRelativeUrl")
                        continue; // We don't want to change this url because then external links will break.

                    Type fieldType = item[field.Id].GetType();

                    if (fieldType != typeof (string))
                        continue; // We're only going to work with strings.

                    string fieldName = field.Title.ToLowerInvariant();
                    if (settings.UseInternalFieldName)
                        fieldName = field.InternalName.ToLowerInvariant();

                    if (settings.FieldName == null || settings.FieldName.ToLowerInvariant() == fieldName)
                    {
                        string result = (string)item[field.Id];
                        bool fieldMatchFound = false;
                        foreach (SearchReplaceData searchReplaceData in settings.SearchStrings)
                        {
                            bool isMatch = searchReplaceData.SearchString.IsMatch(result);

                            if (!isMatch)
                                continue;

                            fieldMatchFound = true;
                            result = searchReplaceData.SearchString.Replace(result, searchReplaceData.ReplaceString);
                        }
                        if (result == (string)item[field.Id])
                            fieldMatchFound = false;

                        if (fieldMatchFound)
                        {
                            Log(settings,
                                string.Format("Match found: List={0}, Field={1}, Replacement={2} => {3}", item.Url,
                                              field.Title, item[field.Id], result));
                            if (!settings.Test)
                            {
                                if (item.File != null && item.File.CheckOutStatus == SPFile.SPCheckOutStatus.None)
                                {
                                    item.File.CheckOut();
                                    wasCheckedOut = false;
                                }
                                try
                                {
                                    item[field.Id] = result;
                                }
                                catch (Exception ex)
                                {
                                    string msg = string.Format("\r\nWARNING: Unable to set field value.\r\nList={0}, Field={1}, Replacement={2} => {3}",
                                        item.Url, field.Title, item[field.Id], result);
                                    if (settings.LogFile == null)
                                        msg += string.Format("\r\n{0}\r\n{1}", ex.Message, ex.StackTrace);

                                    if (settings.Quiet)
                                        Console.WriteLine(msg + "\r\nSee log file for me details.");

                                    if (settings.LogFile != null && settings.Quiet)
                                        msg += string.Format("\r\n{0}\r\n{1}", ex.Message, ex.StackTrace);
                                     
                                    Log(settings, msg);
                                }
                                modified = true;
                            }
                        }
                    }
                }
                if (!settings.Test)
                {
                    if (modified)
                    {
                        try
                        {
                            item.SystemUpdate();
                        }
                        catch (Exception ex)
                        {
                            string msg = string.Format("\r\nWARNING: Unable to set item values.\r\nList={0}", item.Url);

                            if (settings.LogFile == null)
                                msg += string.Format("\r\n{0}\r\n{1}", ex.Message, ex.StackTrace);

                            if (settings.Quiet)
                                Console.WriteLine(msg + "\r\nSee log file for me details.");

                            if (settings.LogFile != null)
                                msg += string.Format("\r\n{0}\r\n{1}", ex.Message, ex.StackTrace);

                            Log(settings, msg);
                        }
                    }

                    if (Utilities.IsCheckedOut(item))
                    {
                        if (modified && item.File != null)
                            item.File.CheckIn(
                                "Checking in changes to list item due to automated search and replace", SPCheckinType.MajorCheckIn);
                        else if (!wasCheckedOut && item.File != null)
                            item.File.UndoCheckOut();
                    }

                    if (modified && settings.Publish && !wasCheckedOut)
                        PublishItems.PublishListItem(item, list,
                                                     new PublishItems.Settings(settings.Quiet, settings.Test,
                                                                               settings.LogFile),
                                                     "\"stsadm.exe -o replacefieldvalues\"");
                }
            }
            Log(settings, "Finished Processing List: " + list.DefaultViewUrl + "\r\n");
        }

        #endregion

        #region Utility Methods

        /// <summary>
        /// Parses the input file.
        /// </summary>
        /// <param name="settings">The settings.</param>
        /// <param name="inputFile">The input file.</param>
        /// <param name="delimiter">The delimiter.</param>
        /// <param name="isXml">if set to <c>true</c> [is XML].</param>
        private static void ParseInputFile(Settings settings, string inputFile, string delimiter, bool isXml)
        {
            if (!isXml)
            {
                if (string.IsNullOrEmpty(inputFile))
                    throw new ArgumentNullException("inputFile");

                string[] inputLines = File.ReadAllLines(inputFile);

                if (inputLines.Length > 0)
                {
                    foreach (string line in inputLines)
                    {
                        string[] arguments = line.Split(delimiter.ToCharArray(), 2);
                        if (arguments.Length == 2)
                        {
                            Log(settings,
                                string.Format("Processing inputfile line: SearchString=[{0}], ReplaceString=[{1}]",
                                              arguments[0], arguments[1]));
                            settings.SearchStrings.Add(new SearchReplaceData(arguments[0], arguments[1]));
                        }
                        else
                            throw new ArgumentException(
                                string.Format("The search and replace string contains too many or too few delimiters: {0}", line));
                    }
                }
            }
            else
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(inputFile);
                foreach (XmlElement replacement in xmlDoc.SelectNodes("//Replacement"))
                {
                    XmlElement search = (XmlElement)replacement.SelectSingleNode("./SearchString");
                    XmlElement replace = (XmlElement)replacement.SelectSingleNode("./ReplaceString");

                    if (search == null)
                        throw new SPException(
                            string.Format("The Replacement node did not contain a SearchString child node: {0}",
                                          replacement.InnerXml));

                    if (replace == null)
                        throw new SPException(
                            string.Format("The Replacement node did not contain a ReplaceString child node: {0}",
                                          replacement.InnerXml));

                    Log(settings, string.Format("Processing inputfile node: SearchString=[{0}], ReplaceString=[{1}]",
                          search.InnerText, replace.InnerText));

                    settings.SearchStrings.Add(new SearchReplaceData(search.InnerText, replace.InnerText));
                }
            }
        }

        /// <summary>
        /// Logs the specified message using the provided settings.
        /// </summary>
        /// <param name="settings">The settings.</param>
        /// <param name="message">The message.</param>
        private static void Log(Settings settings, string message)
        {
            if (!settings.Quiet)
                Console.WriteLine(message);

            if (settings.LogFile != null)
            {
                File.AppendAllText(settings.LogFile, DateTime.Now + ": " + message + "\r\n");
            }
        }

        #endregion

        #region Internal Classes

        /// <summary>
        /// Data class used for passing user provided settings into the various worker methods.
        /// </summary>
        private class Settings
        {
            public List<SearchReplaceData> SearchStrings = new List<SearchReplaceData>();
            public string FieldName;
            public bool Publish;
            public bool Quiet;
            public bool Test;
            public string LogFile;
            public bool UseInternalFieldName;
        }

        private class SearchReplaceData
        {
            public Regex SearchString;
            public string ReplaceString;

            public SearchReplaceData(string searchString, string replaceString)
            {
                SearchString = new Regex(searchString);
                ReplaceString = replaceString;
            }
        }
        #endregion
    }
}
