using System;
using System.Collections.Specialized;
using System.Reflection;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class UpdateListView : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateListView"/> class.
        /// </summary>
        public UpdateListView()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the list view URL."));
            parameters.Add(new SPParam("visiblefieldorder", "order", false, null, new SPNonEmptyValidator(), "Provide a comma separated list of internal field names."));
            parameters.Add(new SPParam("viewname", "name", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("mobileview", "mv", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("defaultmobileview", "dmv", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("rowlimit", "rows", false, null, new SPIntRangeValidator(0, int.MaxValue)));
            parameters.Add(new SPParam("paged", "p", false, null, new SPTrueFalseValidator()));
            SPEnumValidator scopeValidator = new SPEnumValidator(typeof (SPViewScope));
            parameters.Add(new SPParam("scope", "scope", false, null, scopeValidator));
            parameters.Add(new SPParam("style", "style", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("sort", "sort", false, null, new SPNullOrNonEmptyValidator()));
            parameters.Add(new SPParam("filter", "filter", false, null, new SPNullOrNonEmptyValidator()));
            parameters.Add(new SPParam("group", "group", false, null, new SPNullOrNonEmptyValidator()));
            parameters.Add(new SPParam("collapsegroups", "collapse", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("grouplimit", "gl", false, null, new SPIntRangeValidator(0, int.MaxValue)));
            parameters.Add(new SPParam("makedefault", "md"));
            parameters.Add(new SPParam("hidden", "h", false, null, new SPTrueFalseValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nUpdates a list view.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <list view URL>");
            sb.Append("\r\n\t[-visiblefieldorder <comma separated list of internal field names>]");
            sb.Append("\r\n\t[-viewname <view name>]");
            sb.Append("\r\n\t[-mobileview <true | false>]");
            sb.Append("\r\n\t[-defaultmobileview <true | false>]");
            sb.Append("\r\n\t[-rowlimit <number of items to display>]");
            sb.Append("\r\n\t[-paged <true | false>]");
            sb.AppendFormat("\r\n\t[-scope <{0}>]", scopeValidator.DisplayValue);
            sb.Append("\r\n\t[-style <view style>]");
            sb.Append("\r\n\t[-sort <use the following format where the field name is the internal field name (leave blank to clear): \"field1=asc|desc,field2=asc|desc\">]");
            sb.Append("\r\n\t[-filter <CAML XML (leave blank to clear)>");
            sb.Append("\r\n\t[-group <use the following format where the field name is the internal field name (leave blank to clear): \"field1=asc|desc,field2=asc|desc\">]");
            sb.Append("\r\n\t[-collapsegroups <true | false>]");
            sb.Append("\r\n\t[-grouplimit <number of groups to display per page>]");
            sb.Append("\r\n\t[-makedefault]");
            sb.Append("\r\n\t[-hidden <true | false>]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;

            using (SPSite site = new SPSite(url))
            using (SPWeb web = site.OpenWeb())
            {
                SPList list = Utilities.GetListFromViewUrl(web, url);
                if (list == null)
                        throw new Exception("List not found.");

                SPView view = null;
                url = SPEncode.UrlDecodeAsUrl(url);
                foreach (SPView v in list.Views)
                {
                    if (url.ToLower() == SPEncode.UrlDecodeAsUrl(web.Site.MakeFullUrl(v.ServerRelativeUrl)).ToLower())
                    {
                        view = v;
                        break;
                    }
                }
                if (view == null)
                    throw new Exception("View not found.");

                if (Params["visiblefieldorder"].UserTypedIn)
                {
                    view.ViewFields.DeleteAll();
                    foreach (string s in Params["visiblefieldorder"].Value.Split(','))
                    {
                        SPField field = null;
                        try
                        {
                            field = view.ParentList.Fields.GetField(s);
                        }
                        catch(ArgumentException)
                        {
                        }
                        if (field == null)
                            throw new SPException(
                                string.Format("Field '{0}' specified for visiblefieldorder does not exist.", s));
                        view.ViewFields.Add(field);
                    }
                }

                if (Params["viewname"].UserTypedIn)
                    view.Title = Params["viewname"].Value;

                if (Params["mobileview"].UserTypedIn)
                    view.MobileView = bool.Parse(Params["mobileview"].Value);

                if (Params["defaultmobileview"].UserTypedIn)
                    view.MobileDefaultView = bool.Parse(Params["defaultmobileview"].Value);

                if (Params["rowlimit"].UserTypedIn)
                    view.RowLimit = uint.Parse(Params["rowlimit"].Value);

                if (Params["paged"].UserTypedIn)
                    view.Paged = bool.Parse(Params["paged"].Value);

                if (Params["scope"].UserTypedIn)
                    view.Scope = (SPViewScope)Enum.Parse(typeof(SPViewScope), Params["scope"].Value, true);

                if (Params["style"].UserTypedIn)
                {
                    string viewStyle = Params["style"].Value;
                    SetStyle(web, view, viewStyle);
                }

                if (Params["sort"].UserTypedIn)
                {
                    string sortString = Params["sort"].Value;
                    SetSort(view, sortString);
                }

                if (Params["filter"].UserTypedIn)
                {
                    string caml = Params["filter"].Value;
                    SetFilter(view, caml);
                }

                if (Params["group"].UserTypedIn)
                {
                    string groupString = Params["group"].Value;
                    SetGroups(view, groupString);
                }

                bool? collapseGroups = null;
                int? groupLimit = null;
                if (Params["collapsegroups"].UserTypedIn)
                    collapseGroups = bool.Parse(Params["collapsegroups"].Value);
                if (Params["grouplimit"].UserTypedIn)
                    groupLimit = int.Parse(Params["grouplimit"].Value);

                SetGroupAttributes(view, collapseGroups, groupLimit);

                if (Params["makedefault"].UserTypedIn)
                    view.DefaultView = bool.Parse(Params["makedefault"].Value);

                if (Params["hidden"].UserTypedIn)
                    view.Hidden = bool.Parse(Params["hidden"].Value);

                
                view.Update();
            }

            return 1;
        }

       
        #endregion

        /// <summary>
        /// Sets the filter.
        /// </summary>
        /// <param name="view">The view.</param>
        /// <param name="caml">The caml.</param>
        private static void SetFilter(SPView view, string caml)
        {
            XmlDocument sortDoc = new XmlDocument();
            sortDoc.LoadXml(string.Format("<Query>{0}</Query>", view.Query));

            XmlElement where = (XmlElement)sortDoc.SelectSingleNode("//Where");
            if (string.IsNullOrEmpty(caml))
            {
                if (where != null)
                {
                    sortDoc.DocumentElement.RemoveChild(where);
                    view.Query = sortDoc.DocumentElement.InnerXml;
                }
            }
            else
            {
                if (where != null)
                    sortDoc.DocumentElement.RemoveChild(where);

                view.Query = sortDoc.DocumentElement.InnerXml + caml;
            }
        }

        /// <summary>
        /// Sets the sort.
        /// </summary>
        /// <param name="view">The view.</param>
        /// <param name="sortString">The sort string.</param>
        private static void SetSort(SPView view, string sortString)
        {
            XmlDocument sortDoc = new XmlDocument();
            sortDoc.LoadXml(string.Format("<Query>{0}</Query>", view.Query));

            XmlElement orderBy = (XmlElement)sortDoc.SelectSingleNode("//OrderBy");
            if (string.IsNullOrEmpty(sortString))
            {
                if (orderBy != null)
                    sortDoc.DocumentElement.RemoveChild(orderBy);
            }
            else
            {
                if (orderBy != null)
                    sortDoc.DocumentElement.RemoveChild(orderBy);

                orderBy = sortDoc.CreateElement("OrderBy");
                sortDoc.DocumentElement.AppendChild(orderBy);

                foreach (string s in sortString.Split(','))
                {
                    string[] s2 = s.Split('=');
                    string fieldName = s2[0];
                    string order = s2[1];

                    SPField field = null;
                    try
                    {
                        field = view.ParentList.Fields.GetFieldByInternalName(fieldName);
                        
                    }
                    catch (ArgumentException)
                    {
                    }

                    if (field == null)
                        throw new SPException(string.Format("The field '{0}' specified in the sort parameter was not found.  Make sure the case is correct.", fieldName));

                    XmlElement fieldRef = sortDoc.CreateElement("FieldRef");
                    fieldRef.SetAttribute("Name", field.InternalName);
                    if (order.ToLowerInvariant() != "asc")
                        fieldRef.SetAttribute("Ascending", "FALSE");

                    orderBy.AppendChild(fieldRef);
                }
            }
            view.Query = sortDoc.DocumentElement.InnerXml;
        }

        /// <summary>
        /// Sets the group attributes.
        /// </summary>
        /// <param name="view">The view.</param>
        /// <param name="collapseGroups">The collapse groups.</param>
        /// <param name="groupLimit">The group limit.</param>
        private static void SetGroupAttributes(SPView view, bool? collapseGroups, int? groupLimit)
        {
            if (!collapseGroups.HasValue && !groupLimit.HasValue)
                return;

            XmlDocument groupDoc = new XmlDocument();
            groupDoc.LoadXml(string.Format("<Query>{0}</Query>", view.Query));

            XmlElement groupBy = (XmlElement)groupDoc.SelectSingleNode("//GroupBy");

            if (groupBy == null)
                throw new SPException("There are no group by columns set.  Cannot set collapsegroups or grouplimit.");

            if (collapseGroups.HasValue)
                groupBy.SetAttribute("Collapse", collapseGroups.Value.ToString().ToUpper());
            if (groupLimit.HasValue)
                groupBy.SetAttribute("GroupLimit", groupLimit.Value.ToString());

            view.Query = groupDoc.DocumentElement.InnerXml;
        }

        /// <summary>
        /// Sets the sort.
        /// </summary>
        /// <param name="view">The view.</param>
        /// <param name="groupString">The sort string.</param>
        private static void SetGroups(SPView view, string groupString)
        {
            XmlDocument groupDoc = new XmlDocument();
            groupDoc.LoadXml(string.Format("<Query>{0}</Query>", view.Query));

            XmlElement groupBy = (XmlElement)groupDoc.SelectSingleNode("//GroupBy");
            if (string.IsNullOrEmpty(groupString))
            {
                if (groupBy != null)
                    groupDoc.DocumentElement.RemoveChild(groupBy);
            }
            else
            {
                bool collapse = true;
                int groupLimit = 100;

                if (groupBy != null)
                {
                    collapse = bool.Parse(groupBy.GetAttribute("Collapse"));
                    groupLimit = int.Parse(groupBy.GetAttribute("GroupLimit"));

                    groupDoc.DocumentElement.RemoveChild(groupBy);
                }

                groupBy = groupDoc.CreateElement("GroupBy");
                groupDoc.DocumentElement.AppendChild(groupBy);

                groupBy.SetAttribute("Collapse", collapse.ToString().ToUpper());
                groupBy.SetAttribute("GroupLimit", groupLimit.ToString());

                foreach (string s in groupString.Split(','))
                {
                    string[] s2 = s.Split('=');
                    string fieldName = s2[0];
                    string order = s2[1];

                    SPField field = null;
                    try
                    {
                        field = view.ParentList.Fields.GetFieldByInternalName(fieldName);
                    }
                    catch (ArgumentException)
                    {
                    }

                    if (field == null)
                        throw new SPException(string.Format("The field '{0}' specified in the group parameter was not found.  Make sure the case is correct.", fieldName));

                    XmlElement fieldRef = groupDoc.CreateElement("FieldRef");
                    fieldRef.SetAttribute("Name", field.InternalName);
                    if (order.ToLowerInvariant() != "asc")
                        fieldRef.SetAttribute("Ascending", "FALSE");

                    groupBy.AppendChild(fieldRef);
                }
            }
            view.Query = groupDoc.DocumentElement.InnerXml;
        }

        /// <summary>
        /// Sets the view style.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="view">The view.</param>
        /// <param name="viewStyle">The view style.</param>
        private static void SetStyle(SPWeb web, SPView view, string viewStyle)
        {
            bool styleFound = false;
            foreach (SPViewStyle style in web.ViewStyles)
            {
                // Locate the style from the list of styles so that we can get the style ID.
                if (!string.IsNullOrEmpty(style.Title) && style.Title.ToLowerInvariant() == viewStyle.ToLowerInvariant())
                {
                    styleFound = true;
                    view.ApplyStyle(style);
                }
                        
            }
            // If we didn't find the style and the user specified "default" then we need to clear the existing style setting.
            // Problem is that there's no way to do this using public API calls so we have to manipulate the XML directly.
            if (!styleFound && viewStyle.ToLowerInvariant() == "default")
            {
                // XmlDocument m_xdView = view.m_xdView;
                FieldInfo m_xdViewProp = view.GetType().GetField("m_xdView",
                                                                 BindingFlags.NonPublic |
                                                                 BindingFlags.Instance |
                                                                 BindingFlags.InvokeMethod |
                                                                 BindingFlags.GetField);
                XmlDocument m_xdView = (XmlDocument)m_xdViewProp.GetValue(view);
                XmlElement viewStyleNode = (XmlElement)m_xdView.SelectSingleNode("//ViewStyle");
                if (viewStyleNode != null)
                {
                    viewStyleNode.ParentNode.RemoveChild(viewStyleNode);

                    view.ViewHeader = null;
                    view.ViewBody = null;
                    view.ViewFooter = null;
                    view.ViewEmpty = null;
                    view.GroupByHeader = null;
                    view.GroupByFooter = null;
                }
            }
            else if (!styleFound)
                throw new SPException(string.Format("The View Style '{0}' was not found.", viewStyle));
        }


        

        
    }
}
