using System;
using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint.Administration;

namespace Edfinancial.SharePoint.STSADM.Commands.Logging
{
    public class SetUsageAnalysis : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetUsageAnalysis"/> class.
        /// </summary>
        public SetUsageAnalysis()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("enablelogging", "el", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("enableusageprocessing", "eup", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("logfilelocation", "log", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("numberoflogfiles", "num", false, null, new SPIntRangeValidator(0, 30)));
            parameters.Add(new SPParam("processingstarttime", "start", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("processingendtime", "end", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("sspname", "ssp", false, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("enableadvancedprocessing", "eap", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("enablequerylogging", "eql", false, null, new SPTrueFalseValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the usage analysis settings.\r\n\r\nParameters:");
            sb.Append("\r\n\t[-enablelogging <true | false>]");
            sb.Append("\r\n\t[-enableusageprocessing <true | false>]");
            sb.Append("\r\n\t[-logfilelocation <log file location>]");
            sb.Append("\r\n\t[-numberoflogfiles <number of log files to create (0-30)>]");
            sb.Append("\r\n\t[-processingstarttime <usage analysis start time>]");
            sb.Append("\r\n\t[-processingendtime <usage analysis end time>]");
            sb.Append("\r\n\t[-sspname <SSP name>]");
            sb.Append("\r\n\t[-enableadvancedprocessing <true | false>]");
            sb.Append("\r\n\t[-enablequerylogging <true | false>]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            // Get the usage settings from the administration web service (basically the Central Administration web application)
            SPUsageSettings usageSettings = SPWebService.AdministrationService.UsageSettings;

            if (Params["enablelogging"].UserTypedIn)
                usageSettings.LoggingEnabled = bool.Parse(Params["enablelogging"].Value);

            if (Params["enableusageprocessing"].UserTypedIn)
                usageSettings.UsageProcessingEnabled = bool.Parse(Params["enableusageprocessing"].Value);

            if (Params["logfilelocation"].UserTypedIn)
                usageSettings.LogFilesDirectory = Params["logfilelocation"].Value;

            if (Params["numberoflogfiles"].UserTypedIn)
                usageSettings.NumberLogFiles = uint.Parse(Params["numberoflogfiles"].Value);

            if (Params["processingstarttime"].UserTypedIn) // Validation makes sure that end time is also provided
            {
                DateTime startTime = DateTime.Parse(Params["processingstarttime"].Value);
                DateTime endTime = DateTime.Parse(Params["processingendtime"].Value);
                usageSettings.SetProcessingInterval(startTime.Hour, startTime.Minute, endTime.Hour, endTime.Minute);
            }
            usageSettings.Update();


            if (Params["sspname"].UserTypedIn)
            {
                string sspname = Params["sspname"].Value;

                // The SSP is locked down so we need to use reflection to get at it.
                object sharedResourceProvider = Utilities.GetSharedResourceProvider(sspname);

                if (Params["enableadvancedprocessing"].UserTypedIn)
                {
                    Type configurationType = Type.GetType(
                            "Microsoft.SharePoint.Portal.Analytics.Configuration, Microsoft.SharePoint.Portal, Version=12.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c");
                    
                    bool enable = bool.Parse(Params["enableadvancedprocessing"].Value);
                    //Microsoft.SharePoint.Portal.Analytics.Configuration.SetAnalyticsEnabledOnSrp(sharedResourceProvider, enable);
                    Utilities.ExecuteMethod(configurationType, "SetAnalyticsEnabledOnSrp",
                                            new Type[] {sharedResourceProvider.GetType(), typeof (bool)},
                                            new object[] {sharedResourceProvider, enable});
                }

                if (Params["enablequerylogging"].UserTypedIn)
                {
                    Type queryLoggerType = Type.GetType(
                            "Microsoft.Office.Server.Search.Query.QueryLogger, Microsoft.Office.Server.Search, Version=12.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c");
                    
                    bool enable = bool.Parse(Params["enablequerylogging"].Value);
                    //Microsoft.Office.Server.Search.Query.QueryLogger.SetQueryLoggingEnabledOnSsp(sharedResourceProvider, enable);
                    Utilities.ExecuteMethod(queryLoggerType, "SetQueryLoggingEnabledOnSsp",
                                            new Type[] {sharedResourceProvider.GetType(), typeof (bool)},
                                            new object[] {sharedResourceProvider, enable});
                }

                Console.WriteLine("The settings updated may require an iisreset before the changes are visible.");
            }

            return 1;
        }

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["processingstarttime"].UserTypedIn != Params["processingendtime"].UserTypedIn)
            {
                throw new SPSyntaxException("The processingstarttime and processingendtime parameters must both be provided.");
            }

            if (Params["enableadvancedprocessing"].UserTypedIn || Params["enablequerylogging"].UserTypedIn)
                Params["sspname"].IsRequired = true;

            if (Params["sspname"].UserTypedIn && !Params["enableadvancedprocessing"].UserTypedIn && !Params["enablequerylogging"].UserTypedIn)
                throw new SPSyntaxException("Specify enableadvancedprocessing and/or enablequerylogging.");

            base.Validate(keyValues);
        }
        #endregion
    }
}
