using System;
using System.Collections.Specialized;
using System.Reflection;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server;
using Microsoft.Office.Server.UserProfiles;

namespace Edfinancial.SharePoint.STSADM.Commands.MySites
{
    public class MySiteSettings : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="MySiteSettings"/> class.
        /// </summary>
        public MySiteSettings()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("nameformat", "f", false, null, new SPRegexValidator("(?i:^Username_CollisionError$|^Username_CollisionDomain$|^Domain_Username$)")));
            parameters.Add(new SPParam("location", "l", false, null, new SPNonEmptyValidator(), "Please specify the location at which to create personal sites."));
            parameters.Add(new SPParam("chooselanguage", "cl", false, null, new SPEnableDisableValidator(), "Please choose whether users can choose the language of their personal site."));
            parameters.Add(new SPParam("readers", "r", false, null, new SPNonEmptyValidator(), "Please enter the comma separated list of site group readers."));
            parameters.Add(new SPParam("searchcenter", "search", false, null, new SPNonEmptyValidator(), "Please enter the preferred search center URL."));
            parameters.Add(new SPParam("remotemysites", "remote", false, null, new SPEnableDisableValidator(), "Please specify whether you wish to enable or disable my sites to support global deployments."));
            parameters.Add(new SPParam("provider", "p", false, null, new SPUrlValidator(), "Please specify the personal site provider."));
            
            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the my site settings for the My Sites web application.\r\n\r\nParameters:");
            sb.Append("\r\n\t-sspname <SSP name>");
            sb.Append("\r\n\t[-nameformat <Username_CollisionError | Username_CollisionDomain | Domain_Username>]");
            sb.Append("\r\n\t[-location <personal site location>]");
            sb.Append("\r\n\t[-chooselanguage <enable | disable>]");
            sb.Append("\r\n\t[-readers <comma separated list of site group readers>]");
            sb.Append("\r\n\t[-searchcenter <preferred search center URL>]");
            sb.Append("\r\n\t[-remotemysites <enable | disable>]");
            sb.Append("\r\n\t[-provider <personal site provider URL (ex: \"http://mysites\")>]");
            
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string sspname = Params["sspname"].Value;

            ServerContext current = ServerContext.GetContext(sspname);
            UserProfileManager upm = new UserProfileManager(current);
            
            SiteNameFormat nameFormat = upm.PersonalSiteFormat;
            if (Params["nameformat"].UserTypedIn)
                nameFormat = (SiteNameFormat)Enum.Parse(typeof(SiteNameFormat), Params["nameformat"].Value, true);

            string location = upm.PersonalSiteInclusion;
            if (Params["location"].UserTypedIn)
                location = Params["location"].Value;

            bool chooseLanguage = upm.IsPersonalSiteMultipleLanguage;
            if (Params["chooselanguage"].UserTypedIn)
                chooseLanguage = (Params["chooselanguage"].Value == "enable" ? true : false);

            string readers = upm.PersonalSiteReaders;
            if (Params["readers"].UserTypedIn)
                readers = Params["readers"].Value;

            CommitPersonalSiteSettings(upm, chooseLanguage, location, nameFormat, readers);

            bool modified = false;

            // UserProfileApplication userProfileApplication = current.UserProfileApplication;
            System.Reflection.PropertyInfo userProfileApplicationProp = current.GetType().GetProperty("UserProfileApplication",
                                                     BindingFlags.NonPublic |
                                                     BindingFlags.Instance |
                                                     BindingFlags.InvokeMethod |
                                                     BindingFlags.GetProperty);
            object userProfileApplication = userProfileApplicationProp.GetValue(current, null);
           
            // Set the search center url
            if (Params["searchcenter"].UserTypedIn)
            {
                // userProfileApplication.SearchCenterUrl = Params["searchcenter"].Value;
                System.Reflection.PropertyInfo searchCenterProp = userProfileApplication.GetType().GetProperty("SearchCenterUrl",
                                                                                            BindingFlags.NonPublic |
                                                                                            BindingFlags.Public |
                                                                                            BindingFlags.Instance |
                                                                                            BindingFlags.SetProperty |
                                                                                            BindingFlags.FlattenHierarchy);
                searchCenterProp.SetValue(userProfileApplication, Params["searchcenter"].Value, null);
                modified = true;
            }
            if (Params["remotemysites"].UserTypedIn)
            {
                //userProfileApplication.EnablePersonalFeaturesForMultipleDeployments = Params["remotemysites"].Value;
                System.Reflection.PropertyInfo remoteProp = userProfileApplication.GetType().GetProperty("EnablePersonalFeaturesForMultipleDeployments",
                                                                                            BindingFlags.NonPublic |
                                                                                            BindingFlags.Public |
                                                                                            BindingFlags.Instance |
                                                                                            BindingFlags.SetProperty |
                                                                                            BindingFlags.FlattenHierarchy);
                bool remoteMySites = (Params["remotemysites"].Value == "enable" ? true : false);
                remoteProp.SetValue(userProfileApplication, remoteMySites, null);
                modified = true;
            }

            if (Params["provider"].UserTypedIn)
            {
                Uri uri = new Uri(Params["provider"].Value);
                //userProfileApplication.MySitePortalUrl = uri.AbsoluteUri;
                System.Reflection.PropertyInfo searchCenterProp = userProfileApplication.GetType().GetProperty("MySitePortalUrl",
                                                                                 BindingFlags.NonPublic |
                                                                                 BindingFlags.Public |
                                                                                 BindingFlags.Instance |
                                                                                 BindingFlags.SetProperty |
                                                                                 BindingFlags.FlattenHierarchy);
                searchCenterProp.SetValue(userProfileApplication, uri.AbsoluteUri, null);
                modified = true;
            }

            if (modified)
            {
                MethodInfo update =
                    userProfileApplication.GetType().GetMethod("Update",
                                                      BindingFlags.NonPublic |
                                                      BindingFlags.Public |
                                                      BindingFlags.Instance |
                                                      BindingFlags.InvokeMethod |
                                                      BindingFlags.FlattenHierarchy,
                                                      null,
                                                      new Type[] { }, null);
                // userProfileApplication.Update(true);
                update.Invoke(userProfileApplication, null);

                Console.WriteLine("The settings updated may require an iisreset before the changes are visible.");
            }

            return 1;
        }


        #endregion

        /// <summary>
        /// Commits the personal site settings.
        /// </summary>
        /// <param name="upm">The user profile manager.</param>
        /// <param name="chooseLanguage">if set to <c>true</c> the user may choose a language for their personal site.</param>
        /// <param name="location">The personal site location.</param>
        /// <param name="nameFormat">The site name format.</param>
        /// <param name="readers">The readers site groups (comma separated).</param>
        internal static void CommitPersonalSiteSettings(UserProfileManager upm, bool chooseLanguage, string location, SiteNameFormat nameFormat, string readers)
        {
            MethodInfo commitPersonalSiteSettings =
                upm.GetType().GetMethod("CommitPersonalSiteSettings",
                                        BindingFlags.NonPublic | BindingFlags.Public |
                                        BindingFlags.Instance | BindingFlags.InvokeMethod);

            commitPersonalSiteSettings.Invoke(upm,
                                              new object[]
                                                  {
                                                      location, nameFormat, readers, chooseLanguage
                                                  });
        }
    }
}
