using System.Collections.Generic;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Pages
{
    public class EnumUnGhostedFiles : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EnumUnGhostedFiles"/> class.
        /// </summary>
        public EnumUnGhostedFiles()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site url."));
            parameters.Add(new SPParam("recursesubwebs", "recurse", false, null, null));
            Init(parameters, "\r\n\r\nReturns a list of all unghosted (customized) files for a web.\r\n\r\nParameters:\r\n\t-url <web site url>\r\n\t[-recursesubwebs]");
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');
            bool recurse = Params["recursesubwebs"].UserTypedIn;
            List<string> unghostedFiles = new List<string>();

            using (SPSite site = new SPSite(url))
            {
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    if (recurse)
                    {
                        RecurseSubWebs(web, ref unghostedFiles);
                    }
                    else 
                        CheckFoldersForUnghostedFiles(web.RootFolder, ref unghostedFiles);
                }
            }

            if (unghostedFiles.Count == 0)
            {
                output += "There are no unghosted (customized) files on the current web.\r\n";
            }
            else
            {
                output += "The following files are unghosted:";

                foreach (string fileName in unghostedFiles)
                {
                    output += "\r\n\t" + fileName;
                }
            }

            return 1;
        }

        #endregion

        /// <summary>
        /// Recurses the sub webs.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="unghostedFiles">The unghosted files.</param>
        private static void RecurseSubWebs(SPWeb web, ref List<string>unghostedFiles)
        {
            foreach (SPWeb subweb in web.Webs)
            {
                try
                {
                    RecurseSubWebs(subweb, ref unghostedFiles);
                }
                finally
                {
                    subweb.Dispose();
                }
            }
            CheckFoldersForUnghostedFiles(web.RootFolder, ref unghostedFiles);
        }

        /// <summary>
        /// Checks the folders for unghosted files.
        /// </summary>
        /// <param name="folder">The folder.</param>
        /// <param name="unghostedFiles">The unghosted files.</param>
        private static void CheckFoldersForUnghostedFiles(SPFolder folder, ref List<string> unghostedFiles)
        {
            foreach (SPFolder sub in folder.SubFolders)
            {
                CheckFoldersForUnghostedFiles(sub, ref unghostedFiles);
            }

            foreach (SPFile file in folder.Files)
            {
                if (file.CustomizedPageStatus == SPCustomizedPageStatus.Customized)
                {
                    if (!unghostedFiles.Contains(file.ServerRelativeUrl))
                        unghostedFiles.Add(file.ServerRelativeUrl);
                }
            }
        }

    }
}
