using System;
using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Publishing;

namespace Edfinancial.SharePoint.STSADM.Commands.Pages
{
    public class EnumWelcomePages : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EnumWelcomePages"/> class.
        /// </summary>
        public EnumWelcomePages()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", false, null, new SPNonEmptyValidator(), "Please specify the url to search."));
            parameters.Add(new SPParam("scope", "scope", true, null, new SPRegexValidator("(?i:^Farm$|^WebApplication$|^Site$|^Web$)")));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nLists all the welcome page(s) for a farm, web application, site collection, or web.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <url to search>");
            sb.Append("\r\n\t-scope <Farm | WebApplication | Site | Web>");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            if (url != null)
                url = url.TrimEnd('/');

            string scope = Params["scope"].Value.ToLowerInvariant();

            if (scope == "farm")
            {
                foreach (SPService svc in SPFarm.Local.Services)
                {
                    if (!(svc is SPWebService))
                        continue;

                    foreach (SPWebApplication webApp in ((SPWebService)svc).WebApplications)
                    {
                        DisplayWelcomePageUrl(webApp);
                    }
                }
            }
            else if (scope == "webapplication")
            {
                SPWebApplication webApp = SPWebApplication.Lookup(new Uri(url));
                DisplayWelcomePageUrl(webApp);
            }
            else if (scope == "site")
            {
                using (SPSite site = new SPSite(url))
                {
                    DisplayWelcomePageUrl(site);
                }
            }
            else if (scope == "web")
            {
                using (SPSite site = new SPSite(url))
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    DisplayWelcomePageUrl(site, web, true);
                }
            }

            return 1;
        }


        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["scope"].UserTypedIn)
            {
                if (Params["scope"].Value.ToLowerInvariant() == "farm" && Params["url"].UserTypedIn)
                    throw new SPSyntaxException("The url parameter is not compatible with a scope of Farm.");
                if (Params["scope"].Value.ToLowerInvariant() != "farm" && !Params["url"].UserTypedIn)
                    throw new SPSyntaxException("The url parameter is required if the scope is not Farm.");
            }
            base.Validate(keyValues);
        }        

        #endregion


        /// <summary>
        /// Displays the welcome page URL.
        /// </summary>
        /// <param name="webApp">The web app.</param>
        private static void DisplayWelcomePageUrl(SPWebApplication webApp)
        {
            foreach (SPSite site in webApp.Sites)
            {
                try
                {
                    DisplayWelcomePageUrl(site);
                }
                finally
                {
                    site.Dispose();
                }
            }
        }

        /// <summary>
        /// Displays the welcome page URL.
        /// </summary>
        /// <param name="site">The site.</param>
        private static void DisplayWelcomePageUrl(SPSite site)
        {
            foreach (SPWeb web in site.AllWebs)
            {
                try
                {
                    DisplayWelcomePageUrl(site, web, false);
                }
                finally
                {
                    web.Dispose();
                }
            }
        }

        /// <summary>
        /// Displays the welcome page URL.
        /// </summary>
        /// <param name="site">The site.</param>
        /// <param name="web">The web.</param>
        /// <param name="recurseSubWebs">if set to <c>true</c> [recurse sub webs].</param>
        internal static void DisplayWelcomePageUrl(SPSite site, SPWeb web, bool recurseSubWebs)
        {
            if (recurseSubWebs)
            {
                foreach (SPWeb subweb in web.Webs)
                {
                    DisplayWelcomePageUrl(site, subweb, recurseSubWebs);
                }
            }

            if (!PublishingWeb.IsPublishingWeb(web))
                return;

            PublishingWeb pubWeb = PublishingWeb.GetPublishingWeb(web);

            Console.WriteLine(site.MakeFullUrl(web.ServerRelativeUrl) + " = " + pubWeb.DefaultPage);
        }

    }
}
