using System;
using System.Text;
using System.Threading;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Publishing;

namespace Edfinancial.SharePoint.STSADM.Commands.Pages
{
    public class FixPublishingPagesPageLayoutUrl : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="FixPublishingPagesPageLayoutUrl"/> class.
        /// </summary>
        public FixPublishingPagesPageLayoutUrl()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("website", "web", false, null, new SPUrlValidator()));
            parameters.Add(new SPParam("sitecollection", "site", false, null, new SPUrlValidator()));
            parameters.Add(new SPParam("webapplication", "app", false, null, new SPUrlValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append(
                "\r\n\r\nFixes the Page Layout URL property of publishing pages which can get messed up during an upgrade or from importing into a new farm.\r\n\r\nParameters:");
            sb.Append("\r\n\t-website <web site url> | -sitecollection <site collection url> | -webapplication <web application url>");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            SPTriParameterValidator.Validate("website", Params["website"].Value,
                                             "sitecollection", Params["sitecollection"].Value, 
                                             "webapplication", Params["webapplication"].Value);

            if (Params["webapplication"].UserTypedIn)
            {
                string webapplication = Params["webapplication"].Value.TrimEnd('/');
                SPWebApplication webapp = SPWebApplication.Lookup(new Uri(webapplication));
                foreach (SPSite site in webapp.Sites)
                {
                    try
                    {
                        foreach (SPWeb web in site.AllWebs)
                        {
                            try
                            {
                                PublishingWeb pubweb = PublishingWeb.GetPublishingWeb(web);

                                FixPages(pubweb);
                            }
                            finally
                            {
                                web.Dispose();
                            }
                        }
                    }
                    finally
                    {
                        site.Dispose();
                    }
                }
            }
            else if (Params["sitecollection"].UserTypedIn)
            {
                string sitecollection = Params["sitecollection"].Value.TrimEnd('/');
                using (SPSite site = new SPSite(sitecollection))
                {
                    foreach (SPWeb web in site.AllWebs)
                    {
                        try
                        {
                            PublishingWeb pubweb = PublishingWeb.GetPublishingWeb(web);

                            FixPages(pubweb);
                        }
                        finally
                        {
                            web.Dispose();
                        }
                    }
                }
            }
            else if (Params["website"].UserTypedIn)
            {
                string website = Params["website"].Value.TrimEnd('/');
                using (SPSite site = new SPSite(website))
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(website)])
                {
                    PublishingWeb pubweb = PublishingWeb.GetPublishingWeb(web);

                    FixPages(pubweb);
                }
            }
            return 1;
        }

        #endregion

        /// <summary>
        /// Fixes the pages page layout url so that it points to the page layout in the container site collections master page gallery.
        /// </summary>
        /// <param name="publishingWeb">The target publishing web.</param>
        public static void FixPages(PublishingWeb publishingWeb)
        {
            if (!PublishingWeb.IsPublishingWeb(publishingWeb.Web))
                return;

            PublishingPageCollection pages;
            int tryCount = 0;
            while (true)
            {
                try
                {
                    tryCount++;
                    pages = publishingWeb.GetPublishingPages();
                    break;
                }
                catch (InvalidPublishingWebException)
                {
                    if (tryCount > 4)
                        throw;
                    Thread.Sleep(10000);
                    SPWeb web = publishingWeb.Web;
                    SPSite site = web.Site;
                    string url = site.MakeFullUrl(web.ServerRelativeUrl);
                    site.Close();
                    site.Dispose();
                    web.Close();
                    web.Dispose();
                    publishingWeb.Close();
                    site = new SPSite(url);
                    web = site.OpenWeb(Utilities.GetServerRelUrlFromFullUrl(url));
                    publishingWeb = PublishingWeb.GetPublishingWeb(web);
                }
            }

            foreach (PublishingPage page in pages)
            {
                string pageLayoutUrl = (string) page.ListItem[FieldId.PageLayout];
                if (string.IsNullOrEmpty(pageLayoutUrl) || pageLayoutUrl.IndexOf("/_catalogs/") < 0)
                    continue;

                SPFieldUrlValue url = new SPFieldUrlValue(pageLayoutUrl);


                string temp = publishingWeb.Web.Site.ServerRelativeUrl.TrimEnd('/') +
                              url.Url.Substring(url.Url.IndexOf("/_catalogs/"));

                temp = publishingWeb.Web.Site.MakeFullUrl(temp);

                if (url.Url.ToLowerInvariant() == temp.ToLowerInvariant())
                    continue;

                // Can't edit items that are checked out.
                if (Utilities.IsCheckedOut(page.ListItem))
                    continue;

                url.Url = temp;
                page.CheckOut();
                page.ListItem[FieldId.PageLayout] = url;
                page.ListItem.UpdateOverwriteVersion();
                page.ListItem.File.CheckIn("Fixed URL to page layout.", SPCheckinType.MajorCheckIn);
                if (page.ListItem.ModerationInformation != null)
                    page.ListItem.File.Approve("Publishing changes to page layout.");
            }
        }


    }
}
