using System.IO;
using System.Reflection;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;

namespace Edfinancial.SharePoint.STSADM.Commands.Pages
{
    public class ReGhostFile : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ReGhostFile"/> class.
        /// </summary>
        public ReGhostFile()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the file url."));
            parameters.Add(new SPParam("force", "force", false, null, null));
            Init(parameters, "\r\n\r\nReghosts a file (use force to override CustomizedPageStatus check).\r\n\r\nParameters:\r\n\t-url <file url>\r\n\t[-force]");
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            bool force = Params["force"].UserTypedIn;

            using (SPSite site = new SPSite(url))
            {
                using (SPWeb web = site.OpenWeb())
                {
                    SPFile file = web.GetFile(url);
                    if (file == null)
                    {
                        output += "File not found.";
                        return 0;
                    }

                    if (file.CustomizedPageStatus != SPCustomizedPageStatus.Customized && !force)
                    {
                        output += file.ServerRelativeUrl + " was not unghosted.";
                        return 1;
                    }
                    else
                    {
                        if (file.CustomizedPageStatus != SPCustomizedPageStatus.Customized && force)
                        {
                            if (!string.IsNullOrEmpty((string)file.Properties["vti_setuppath"]))
                            {
                                file.Properties["vti_hasdefaultcontent"] = "false";

                                string setupPath = (string) file.Properties["vti_setuppath"];
                                string rootPath = SPUtility.GetGenericSetupPath("Template");

                                if (!File.Exists(Path.Combine(rootPath, setupPath)))
                                {
                                    string message = "The template file (" + Path.Combine(rootPath, setupPath) + 
                                        ") does not exist so re-ghosting will not be possible.";

                                    // something's wrong with the setup path - lets see if we can fix it
                                    // Try and remove a leading locale if present
                                    setupPath = "SiteTemplates\\" + setupPath.Substring(5);
                                    if (File.Exists(Path.Combine(rootPath, setupPath)))
                                    {
                                        message += "  It appears that a possible template match does exist at \"" +
                                                   Path.Combine(rootPath, setupPath) +
                                                   "\" however this tool currently is not able to handle pointing the file to the correct template path.  This scenario is most likely due to an upgrade from SPS 2003.";

                                        // We found a matching file so reset the property and update the file.
                                        // ---  I wish this would work but it simply doesn't - something is preventing the
                                        //      update from occuring.  Manipulating the database directly results in a 404
                                        //      when attempting to load the "fixed" page so there's gotta be something beyond
                                        //      just updating the setuppath property.
                                        //file.Properties["vti_setuppath"] = setupPath;
                                        //file.Update();
                                    }
                                    output += message;
                                    return 0;
                                }
                            }
                        }
                        file.RevertContentStream();

                        file = web.GetFile(url);
                        if (file.CustomizedPageStatus == SPCustomizedPageStatus.Customized)
                        {
                            // Still unsuccessful so take measures further
                            if (force)
                            {
                                object request = Utilities.GetSPRequestObject(web);

                                MethodInfo revertContentStreams = request.GetType().GetMethod("RevertContentStreams", Utilities.AllBindings);

                                // I found some cases where calling this directly was the only way to force the re-ghosting of the file.
                                // I think the trick is that it's not updating the file properties after doing the revert (the
                                // RevertContentStream method will call SPRequest.UpdateFileOrFolderProperties() immediately after the 
                                // RevertContentStreams call but ommitting the update call seems to make a difference.
                                revertContentStreams.Invoke(request,
                                                            new object[]
                                                                {
                                                                    web.Url, file.Url,
                                                                    file.CheckOutStatus != SPFile.SPCheckOutStatus.None
                                                                });

                                MethodInfo dirtyThisFileObject = file.GetType().GetMethod("DirtyThisFileObject", Utilities.AllBindings);
                                dirtyThisFileObject.Invoke(file, new object[] { });

                                file = web.GetFile(url);

                                if (file.CustomizedPageStatus == SPCustomizedPageStatus.Customized)
                                {
                                    output += "Unable to re-ghost file " + file.ServerRelativeUrl;
                                    return 0;
                                }
                                else
                                {
                                    output += file.ServerRelativeUrl + " was re-ghosted!";
                                    return 1;
                                }

                            }
                            else
                            {
                                output += "Unable to re-ghost file " + file.ServerRelativeUrl;
                                return 0;
                            }
                        }
                        else
                        {
                            output += file.ServerRelativeUrl + " was re-ghosted!";
                            return 1;
                        }
                    }
                }
            }
        }

        #endregion


    }
}
