using System;
using System.Collections.Specialized;
using System.Reflection;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server;
using Microsoft.Office.Server.Infrastructure;

namespace Edfinancial.SharePoint.STSADM.Commands.Permissions
{
    public class SetSspAcl : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetSspAcl"/> class.
        /// </summary>
        public SetSspAcl()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("rights", "r", false, null, new SPNonEmptyValidator(), "Please specify the desired rights."));
            parameters.Add(new SPParam("user", "user", true, null, new SPNonEmptyValidator(), "Please specify the username."));
            string rights = "All | None | CreatePersonalSite, ManageAnalytics, ManageAudiences, ManageUserProfiles, SetPermissions, UsePersonalFeatures";

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSet the personalization services permissions for an SSP.  Specify 'None' for rights to remove an existing user.\r\n\r\nParameters:");
            sb.Append("\r\n\t-sspname <SSP name>");
            sb.AppendFormat("\r\n\t-rights <comma separated: {0}>", rights);
            sb.Append("\r\n\t-user <DOMAIN\\name>");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string sspname = Params["sspname"].Value;
            string username = Params["user"].Value;
            SharedServiceRights rights = GetUserRights(Params["rights"].Value.Split(','));

            ServerContext current = ServerContext.GetContext(sspname);
            
            //SharedServiceAccessControlList acl = SharedServiceAccessControlList.GetInstance(current);
            Type sharedServiceAccessControlListType = Type.GetType("Microsoft.Office.Server.Infrastructure.SharedServiceAccessControlList, Microsoft.Office.Server, Version=12.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c");

            MethodInfo sharedServiceAccessControlListMethodInfo =
                sharedServiceAccessControlListType.GetMethod("GetInstance",
                                                  Utilities.AllBindings, 
                                                  null,
                                                  new Type[] { typeof(ServerContext) }, null);
            object acl = sharedServiceAccessControlListMethodInfo.Invoke(null, new object[] { current });

            //SharedServiceAccessControlEntry aclEntry = acl[username];
            PropertyInfo itemProp = acl.GetType().GetProperty("Item", Utilities.AllBindings);
            SharedServiceAccessControlEntry aclEntry = (SharedServiceAccessControlEntry)itemProp.GetValue(acl, new object[] { username });

            if (aclEntry == null)
            {
                if (rights == SharedServiceRights.None)
                {
                    output += "User does not currently have any rights assiged - nothing to do.";
                    return 0;
                }

                // Adding a new user
                aclEntry = new SharedServiceAccessControlEntry(username, rights);

                MethodInfo add =
                    acl.GetType().GetMethod("Add",
                                            Utilities.AllBindings,
                                            null,
                                            new Type[] { typeof(SharedServiceAccessControlEntry) }, null);

                //acl.Add(aclEntry);
                add.Invoke(acl, new object[] { aclEntry });
            }
            else
            {
                if (rights == SharedServiceRights.None)
                {
                    // Remove an existing user
                    MethodInfo remove =
                        acl.GetType().GetMethod("Remove",
                                                Utilities.AllBindings,
                                                null,
                                                new Type[] {typeof (string)}, null);

                    //acl.Remove(username);
                    remove.Invoke(acl, new object[] { username });
                }
                else
                {
                    // Modifying an existing user
                    aclEntry = new SharedServiceAccessControlEntry(username, rights);

                    MethodInfo updateAccessControlEntry =
                        acl.GetType().GetMethod("UpdateAccessControlEntry",
                                                Utilities.AllBindings,
                                                null,
                                                new Type[] {typeof (SharedServiceAccessControlEntry)},
                                                null);

                    //acl.UpdateAccessControlEntry(username, aclEntry);
                    updateAccessControlEntry.Invoke(acl, new object[] {aclEntry});
                }
            }

            MethodInfo update =
                acl.GetType().GetMethod("Update",
                                        Utilities.AllBindings,
                                        null,
                                        new Type[] {}, null);
            // acl.Update();
            update.Invoke(acl, null);


            return 1;
        }
        
        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            string[] rights = Params["rights"].Value.Split(',');
            bool all = false;
            bool none = false;
            bool specific = false;
            foreach (string s in rights)
            {
                try
                {
                    SharedServiceRights right = (SharedServiceRights)Enum.Parse(typeof(SharedServiceRights), s, true);
                    if (right == SharedServiceRights.All)
                        all = true;
                    else if (right == SharedServiceRights.None)
                        none = true;
                    else
                        specific = true;
                }
                catch
                {
                    throw new SPSyntaxException(string.Format("The permission '{0}' is not a valid permission.", s.Trim()));
                }
            }
            if (all && none)
                throw new SPSyntaxException("The 'All' and 'None' permissions cannot be used together.");
            if (all && none && specific)
                throw new SPSyntaxException("The 'All' and 'None' permissions cannot be used together or with other permissions.");
            if (all && specific)
                throw new SPSyntaxException("The 'All' permission cannot be used with other permissions.");
            if (none && specific)
                throw new SPSyntaxException("The 'None' permission cannot be used with other permissions.");

            base.Validate(keyValues);
        }
        #endregion

        /// <summary>
        /// Gets the user rights.
        /// </summary>
        /// <param name="rights">The rights.</param>
        /// <returns></returns>
        private static SharedServiceRights GetUserRights(string[] rights)
        {
            SharedServiceRights right = SharedServiceRights.None;
            foreach (string s in rights)
            {
                right |= (SharedServiceRights)Enum.Parse(typeof(SharedServiceRights), s, true);
            }

            return right;
        }
    }
}
