using System;
using System.Collections.Generic;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server.Search.Administration;
using Microsoft.Office.Server.Search.Query;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Search
{
    public class AddSearchRule : SPOperation
    {
        private enum PageType
        {
            WebAddress,
            PropertyQuery,
            ContentSource,
            AllContent
        }
 
        /// <summary>
        /// Initializes a new instance of the <see cref="AddSearchRule"/> class.
        /// </summary>
        public AddSearchRule()
        {
            SPEnumValidator scopeFilterValidator = new SPEnumValidator(typeof(ScopeRuleFilterBehavior));
            SPEnumValidator scopeTypeValidator = new SPEnumValidator(typeof(PageType));
            SPEnumValidator webTypeValidator = new SPEnumValidator(typeof(UrlScopeRuleType));
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site collection."));
            parameters.Add(new SPParam("scope", "s", true, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("behavior", "b", true, null, scopeFilterValidator));
            parameters.Add(new SPParam("type", "t", true, null, scopeTypeValidator));
            parameters.Add(new SPParam("webtype", "wt", false, null, webTypeValidator));
            parameters.Add(new SPParam("webvalue", "wv", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("property", "p", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("propertyvalue", "pv", false, null, new SPNonEmptyValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nAdds a search scope rule to the specified scope for a given site collection.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <site collection url>");
            sb.Append("\r\n\t-scope <scope name>");
            sb.AppendFormat("\r\n\t-behavior <{0}>", scopeFilterValidator.DisplayValue);
            sb.AppendFormat("\r\n\t-type <{0}>", scopeTypeValidator.DisplayValue);
            sb.AppendFormat("\r\n\t[-webtype <{0}>]", webTypeValidator.DisplayValue);
            sb.Append("\r\n\t[-webvalue <value associated with the specified web type>]");
            sb.Append("\r\n\t[-property <managed property name>]");
            sb.Append("\r\n\t[-propertyvalue <value associated with the specified property or content source>]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members
        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');
            string scopeName = Params["scope"].Value;
            PageType type = (PageType)Enum.Parse(typeof(PageType), Params["type"].Value, true);

            ScopeRuleFilterBehavior behavior = ScopeRuleFilterBehavior.Include;
            if (Params["behavior"].UserTypedIn)
                behavior = (ScopeRuleFilterBehavior)Enum.Parse(typeof(ScopeRuleFilterBehavior), Params["behavior"].Value, true);

            
            using (SPSite site = new SPSite(url))
            using (SPWeb web = site.RootWeb)
            {
                if (!web.CurrentUser.IsSiteAdmin)
                    throw new UnauthorizedAccessException();

                SearchContext context = SearchContext.GetContext(site);
                Scopes scopeManager = new Scopes(context);

                Scope scope;
                try
                {
                    scope = scopeManager.GetScope(new Uri(site.Url), scopeName);
                }
                catch (ScopeNotFoundException)
                {
                    scope = scopeManager.GetScope(null, scopeName);
                }
                Schema schema = new Schema(context);

                switch(type)
                {
                    case PageType.AllContent:
                        scope.Rules.CreateAllContentRule();
                        break;
                    case PageType.ContentSource:
                        scope.Rules.CreatePropertyQueryRule(behavior, schema.AllManagedProperties["ContentSource"], Params["propertyvalue"].Value);
                        break;
                    case PageType.PropertyQuery:
                        ManagedProperty prop;
                        try
                        {
                            prop = schema.AllManagedProperties[Params["property"].Value];
                        }
                        catch (KeyNotFoundException)
                        {
                            throw new SPException(
                                string.Format("Property '{0}' was not found.", Params["property"].Value));
                        }
                        scope.Rules.CreatePropertyQueryRule(behavior, prop, Params["propertyvalue"].Value);

                        break;
                    case PageType.WebAddress:
                        UrlScopeRuleType webType =
                            (UrlScopeRuleType) Enum.Parse(typeof (UrlScopeRuleType), Params["webtype"].Value, true);
                        scope.Rules.CreateUrlRule(behavior, webType, Params["webvalue"].Value);
                        break;
                }
            }

            return 1;
        }

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(System.Collections.Specialized.StringDictionary keyValues)
        {
            PageType type = (PageType)Enum.Parse(typeof(PageType), Params["type"].Value, true);
            switch (type)
            {
                case PageType.AllContent:
                    Params["behavior"].IsRequired = false;
                    break;
                case PageType.ContentSource:
                    Params["propertyvalue"].IsRequired = true;
                    break;
                case PageType.PropertyQuery:
                    Params["property"].IsRequired = true;
                    Params["propertyvalue"].IsRequired = true;
                    break;
                case PageType.WebAddress:
                    Params["webtype"].IsRequired = true;
                    Params["webvalue"].IsRequired = true;
                    break;
            }

            base.Validate(keyValues);
        }
        #endregion

    }
}
