using System;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server.Search.Administration;
using Microsoft.Office.Server.Search.Query;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Search
{
    public class UpdateSearchScope : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="UpdateSearchScope"/> class.
        /// </summary>
        public UpdateSearchScope()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site collection."));
            parameters.Add(new SPParam("name", "n", true, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("description", "d", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("groups", "g", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("searchpage", "sp", false, null, new SPNonEmptyValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nUpdates the specified search scope for a given site collection.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <site collection url>");
            sb.Append("\r\n\t-name <scope name>");
            sb.Append("\r\n\t[-description <scope description>]");
            sb.Append("\r\n\t[-groups <display groups (comma separate multiple groups)>]");
            sb.Append("\r\n\t[-searchpage <specific search results page to send users to for results when they search in this scope>]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members
        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');
            string name = Params["name"].Value;
            string description = Params["description"].Value + string.Empty;
            string searchPage = null;
            if (Params["searchpage"].UserTypedIn)
                searchPage = Params["searchpage"].Value;

            using (SPSite site = new SPSite(url))
            using (SPWeb web = site.RootWeb)
            {
                if (!web.CurrentUser.IsSiteAdmin)
                    throw new UnauthorizedAccessException();

                Scopes scopeManager = new Scopes(SearchContext.GetContext(site));

                Scope scope;
                try
                {
                    scope = scopeManager.GetScope(new Uri(site.Url), name);
                }
                catch (ScopeNotFoundException)
                {
                    scope = scopeManager.GetScope(null, name);
                }
                if (Params["description"].UserTypedIn)
                    scope.Description = description;
                if (Params["searchpage"].UserTypedIn)
                    scope.AlternateResultsPage = searchPage;

                scope.Update();

                // If the user passed in any groups then add the scope to those groups.
                if (Params["groups"].UserTypedIn)
                {
                    // Clear out any group settings.
                    foreach (ScopeDisplayGroup g in scopeManager.AllDisplayGroups)
                    {
                        if (g.Contains(scope))
                        {
                            g.Remove(scope);
                            g.Update();
                        }
                    }

                    // Add back the specified groups.
                    foreach (string g in Params["groups"].Value.Split(','))
                    {
                        ScopeDisplayGroup group;
                        try
                        {
                            group = scopeManager.GetDisplayGroup(new Uri(site.Url), g.Trim());
                        }
                        catch (Exception)
                        {
                            group = null;
                        }
                        if (group == null)
                        {
                            scope.Delete(); // We don't want the scope created if there was an error with the groups so undo what we've done.
                            throw new SPException(string.Format("Display group '{0}' not found.", g));
                        }
                        group.Add(scope);
                        group.Update();
                    }
                }
            }

            return 1;
        }

        #endregion

    }
}
