using System;
using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Navigation;
using Microsoft.SharePoint.Publishing;
using Microsoft.SharePoint.Publishing.Navigation;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class AddNavigationNode : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="AddNavigationNode"/> class.
        /// </summary>
        public AddNavigationNode()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site collection."));
            parameters.Add(new SPParam("name", "n", true, null, new SPNonEmptyValidator(), "Please specify the name."));
            parameters.Add(new SPParam("linkurl", "l", true, null, new SPNonEmptyValidator(), "Please specify the link url."));
            parameters.Add(new SPParam("description", "d", false, null, new SPNonEmptyValidator()));
            SPEnumValidator enumValidator = new SPEnumValidator(typeof (NodeTypes));
            parameters.Add(new SPParam("type", "t", true, null, enumValidator, "Please specify the type."));
            parameters.Add(new SPParam("global", "g", false, null, null));
            parameters.Add(new SPParam("current", "c", false, null, null));
            parameters.Add(new SPParam("addasfirst", "first", false, null, null));
            parameters.Add(new SPParam("addaslast", "last", false, null, null));
            parameters.Add(new SPParam("addafter", "after", false, null, null));
            parameters.Add(new SPParam("newwindow", "nw", false, null, null));
            
            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nAdds a new node to the site hierarchy.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <site collection url>");
            sb.Append("\r\n\t-name <navigation name>");
            sb.Append("\r\n\t-linkurl <link url>");
            sb.Append("\r\n\t-type <link type: " + enumValidator.DisplayValue + ">");
            sb.Append("\r\n\t-global / -current");
            sb.Append("\r\n\t-addasfirst / -addaslast / -addafter <previous node id>");
            sb.Append("\r\n\t[-newwindow]");
            sb.Append("\r\n\t[-description <description text>]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            SPBinaryParameterValidator.Validate("global", (Params["global"].UserTypedIn ? "true" : null),
                                                "current", (Params["current"].UserTypedIn ? "true" : null));
            
            SPTriParameterValidator.Validate("addasfirst", (Params["addasfirst"].UserTypedIn ? "true" : null),
                                             "addaslast", (Params["addaslast"].UserTypedIn ? "true" : null),
                                             "addafter", Params["addafter"].Value);

            string url = Params["url"].Value.TrimEnd('/');
            string linkUrl = Params["linkurl"].Value;
            string name = Params["name"].Value;
            string description = Params["description"].Value;
            NodeTypes type = (NodeTypes)Enum.Parse(typeof(NodeTypes), Params["type"].Value, true);
            bool global = Params["global"].UserTypedIn;
            bool current = Params["current"].UserTypedIn;
            bool addAsFirst = Params["addasfirst"].UserTypedIn;
            bool addAsLast = Params["addaslast"].UserTypedIn;
            bool addAfter = Params["addafter"].UserTypedIn;
            bool newWindow = Params["newwindow"].UserTypedIn;
            string previousNodeID = string.Empty;
            if (addAfter)
                previousNodeID = Params["addafter"].Value;

            using (SPSite site = new SPSite(url))
            {
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    PublishingWeb pubweb = PublishingWeb.GetPublishingWeb(web);

                    SPNavigationNodeCollection nodes;
                    if (global)
                        nodes = pubweb.GlobalNavigationNodes;
                    else if (current)
                        nodes = pubweb.CurrentNavigationNodes;
                    else
                        throw new ApplicationException("Unknown error occured.");

                    SPNavigationNode node = SPNavigationSiteMapNode.CreateSPNavigationNode(
                        name, linkUrl, type, nodes);

                    node.Properties["CreatedDate"] = DateTime.Now;
                    node.Properties["LastModifiedDate"] = DateTime.Now;
                    node.Properties["Description"] = description;
                    if (newWindow)
                        node.Properties["Target"] = "_blank";
                    else
                        node.Properties["Target"] = string.Empty;

                    node.Update();

                    if (addAsFirst)
                        node.MoveToFirst(nodes);
                    else if (addAsLast)
                        node.MoveToLast(nodes);
                    else if (addAfter)
                    {
                        SPNavigationNode previousNode = web.Navigation.GetNodeById(int.Parse(previousNodeID));
                        if (previousNode == null)
                        {
                            output = "Previous node was not found.";
                            return 0;
                        }
                        node.Move(nodes, previousNode);
                    }
                }
            }

            return 1;
        }

        #endregion

    }
}
