using System;
using System.Collections.Specialized;
using System.Reflection;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class AddSiteAdmin : SPOperation
    {
         /// <summary>
        /// Initializes a new instance of the <see cref="AddSiteAdmin"/> class.
        /// </summary>
        public AddSiteAdmin()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site collection url."));
            parameters.Add(new SPParam("userlogin", "u", true, null, new SPNonEmptyValidator(), "Please specify the users login."));
            parameters.Add(new SPParam("useremail", "email", true, null, new SPRegexValidator(@"^[^ \r\t\n\f@]+@[^ \r\t\n\f@]+$"), "Please specify the users email address."));
            parameters.Add(new SPParam("username", "username", true, null, new SPNonEmptyValidator(), "Please specify the users name."));
            parameters.Add(new SPParam("role", "r", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("group", "g", false, null, new SPNonEmptyValidator()));
            Init(parameters, "\r\n\r\nAdds a user as a site admin (must be a farm administrator or a current site admin).\r\n\r\nParameters:\r\n\t-url <url of site collection>\r\n\t-userlogin <DOMAIN\\user>\r\n\t-useremail <someone@example.com>\r\n\t-username <display name>\r\n\t[-role <role name> / -group <group name>]");
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            if (Params["role"].UserTypedIn && Params["group"].UserTypedIn)
                throw new SPException(SPResource.GetString("ExclusiveArgs", new object[] { "role, group" }));

            string url = Params["url"].Value.TrimEnd('/');
            string login = Params["userlogin"].Value;
            string email = Params["useremail"].Value;
            string username = Params["username"].Value;

            SPWeb web = null;

            using (SPSite site = new SPSite(url))
            using (SPSiteAdministration adminSite = new SPSiteAdministration(url))
            try
            {
                web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)];

                login = Utilities.TryGetNT4StyleAccountName(login, web.Site.WebApplication);

                if (SPFarm.Local.CurrentUserIsAdministrator() || web.CurrentUser.IsSiteAdmin)
                {
                    // First lets get our user object.
                    SPUser user = null;
                    try
                    {
                        user = web.AllUsers[login];
                    }
                    catch (SPException) { }

                    if (user == null)
                    {
                        web.SiteUsers.Add(login, email, username, string.Empty);
                        user = web.AllUsers[login];
                    }
                    if (user == null)
                        throw new SPException("User cannot be found.");


                    if (web.CurrentUser.IsSiteAdmin)
                    {
                        // This is the easy part - the calling user is a site admin so we can simply add the user
                        user.IsSiteAdmin = true;
                        user.Update();
                    }
                    else
                    {
                        // This is the hard part - we need to trick the system to allow us to add the user
                        // (not sure why Microsoft didn't allow farm administrators to add site admins when 
                        // they can change site owners which produces the same affect but what if you don't
                        // want to change the site owner?).

                        try
                        {
                            // Give it a try first (shouldn't work but what the heck)
                            user.IsSiteAdmin = true;
                            user.Update();
                        }
                        catch (SPException)
                        {
                            if (web.CurrentUser.ID != user.ID)
                            {
                                // If we are adding a user that is not ourselves then we can simply temporarily set
                                // ourselves as a site owner, add the user, and then remove ourselves as the site owner.
                                // The reason why we can't use this approach if we are setting ourselves as a site admin
                                // is that as soon as we attempt to remove ourselves as an owner it will also make it so
                                // we are not longer a site admin.

                                // Note that we only do this to avoid the reflection call to the internal method at all costs (see below).

                                string originalSiteOwner = adminSite.OwnerLoginName;
                                adminSite.OwnerLoginName = web.CurrentUser.LoginName;

                                // We've changed the properties of the site object so we need to close and re-open our SPWeb object.
                                web.Close();
                                web.Dispose();

                                web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)];

                                // Now that we have a new SPWeb object we can re-retrieve the SPUser object which we now know will exist
                                // so no need for null checks as the above already certified that it exists.
                                user = web.AllUsers[login];

                                user.IsSiteAdmin = true;
                                user.Update();

                                // Now we need to reset the original owner.
                                adminSite.OwnerLoginName = originalSiteOwner;
                            }
                            else
                            {
                                // It didn't work because we're not a site admin and we are trying to set ourselves
                                // as a site admin which means we can't use the site owner trick from above.
                                // Fortunately, Microsoft provides a method that you can call to put the site object
                                // into an Admin mode (AdministratorOperationMode).  This is what is called by the
                                // SPSiteAdministration object's constructor so that farm administrators can set
                                // the owner and other information.  Unfortunately Microsoft doesn't expose the
                                // site object via the SPSiteAdministration object nor do they make the AdministratorOperationMode
                                // method public (it's marked internal).  So, we have to use reflection to set this flag.
                                // In my opinion this is a safe call to make as the property re-checks all the security
                                // to make sure that you have appropriate rights to administer the site.  If Microsoft
                                // would have made the SPSite object available from the SPSiteAdministration object
                                // this would not be necessary.
                                
                                PropertyInfo adminOpModeProp = site.GetType().GetProperty("AdministratorOperationMode",
                                                                                     BindingFlags.NonPublic |
                                                                                     BindingFlags.Instance |
                                                                                     BindingFlags.InvokeMethod |
                                                                                     BindingFlags.GetProperty);
                                adminOpModeProp.SetValue(site, true, null);

                                // We've changed the properties of the site object so we need to close and re-open our SPWeb object.
                                web.Close();
                                web.Dispose();

                                web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)];

                                // Now that we have a new SPWeb object we can re-retrieve the SPUser object which we now know will exist
                                // so no need for null checks as the above already certified that it exists.
                                user = web.AllUsers[login];

                                user.IsSiteAdmin = true;
                                user.Update();

                            }
                        }
                    }

                    if (Params["role"].UserTypedIn)
                    {
                        SPRoleDefinition roleDefinition = null;
                        try
                        {
                            roleDefinition = web.RoleDefinitions[Params["role"].Value];
                        }
                        catch (ArgumentException) {}

                        if (roleDefinition == null)
                            throw new SPException("The specified role does not exist.");

                        SPRoleDefinitionBindingCollection roleDefinitionBindings = new SPRoleDefinitionBindingCollection();
                        roleDefinitionBindings.Add(roleDefinition);
                        SPRoleAssignment roleAssignment = new SPRoleAssignment(user);
                        roleAssignment.ImportRoleDefinitionBindings(roleDefinitionBindings);
                        web.RoleAssignments.Add(roleAssignment);
                    }
                    else if (Params["group"].UserTypedIn)
                    {
                        SPGroup group = null;
                        try
                        {
                            group = web.SiteGroups[Params["group"].Value];
                        }
                        catch (ArgumentException) {}

                        if (group == null)
                            throw new SPException("The specified group does not exist.");

                        group.AddUser(user);
                    }
                }
                else
                {
                    throw new Exception("You need to be a site collection administrator or farm administrator to set this user as a site administrator.");
                }
            }
            finally
            {
                if (web != null)
                    web.Dispose();
            }

            return 1;
        }

        #endregion
    }
}
