using System;
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Navigation;
using Microsoft.SharePoint.Publishing;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class EnumNavigation : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EnumNavigation"/> class.
        /// </summary>
        public EnumNavigation()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site collection."));
            parameters.Add(new SPParam("outputfile", "output", false, null, new SPDirectoryExistsAndValidFileNameValidator(), "Make sure the output directory exists and a valid filename is provided."));
            parameters.Add(new SPParam("xml", "xml", false, null, null));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nReturns the site navigation hierarchy.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <site collection url>");
            sb.Append("\r\n\t[-xml]");
            sb.Append("\r\n\t[-outputfile <file to output results to>]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');
            bool xml = Params["xml"].UserTypedIn;

            using (SPSite site = new SPSite(url))
            {
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    PublishingWeb pubweb = PublishingWeb.GetPublishingWeb(web);

                    if (!xml)
                    {
                        StringBuilder sb = new StringBuilder();

                        sb.Append("Global Navigation:\r\n");
                        EnumerateCollection(pubweb.GlobalNavigationNodes, ref sb, 1);

                        sb.Append("Current Navigation:\r\n");
                        EnumerateCollection(pubweb.CurrentNavigationNodes, ref sb, 1);

                        output += sb.ToString();
                    }
                    else
                    {
                        XmlDocument xmlDoc = GetNavigationAsXml(pubweb);

                        output += Utilities.GetFormattedXml(xmlDoc);
                    }
                }
            }
            if (Params["outputfile"].UserTypedIn)
            {
                File.WriteAllText(Params["outputfile"].Value, output);
                output = string.Empty;
            }

            return 1;
        }

        #endregion

        /// <summary>
        /// Gets the navigation as XML.
        /// </summary>
        /// <param name="pubweb">The pubweb.</param>
        /// <returns></returns>
        internal static XmlDocument GetNavigationAsXml(PublishingWeb pubweb)
        {
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.AppendChild(xmlDoc.CreateElement("Navigation"));
            xmlDoc.DocumentElement.AppendChild(xmlDoc.CreateElement("Global"));
            xmlDoc.DocumentElement.AppendChild(xmlDoc.CreateElement("Current"));
            EnumerateCollection(pubweb, true, pubweb.GlobalNavigationNodes, (XmlElement)xmlDoc.DocumentElement.ChildNodes[0]);
            EnumerateCollection(pubweb, false, pubweb.CurrentNavigationNodes, (XmlElement)xmlDoc.DocumentElement.ChildNodes[1]);
            return xmlDoc;
        }

        /// <summary>
        /// Enumerates the collection.
        /// </summary>
        /// <param name="pubWeb">The publishing web.</param>
        /// <param name="isGlobal">if set to <c>true</c> [is global].</param>
        /// <param name="nodes">The nodes.</param>
        /// <param name="xmlParentNode">The XML parent node.</param>
        private static void EnumerateCollection(PublishingWeb pubWeb, bool isGlobal, SPNavigationNodeCollection nodes, XmlElement xmlParentNode)
        {
            if (nodes == null || nodes.Count == 0)
                return;

            foreach (SPNavigationNode node in nodes)
            {
                NodeTypes type = NodeTypes.None;
                if (node.Properties["NodeType"] != null && !string.IsNullOrEmpty(node.Properties["NodeType"].ToString()))
                    type = (NodeTypes)Enum.Parse(typeof(NodeTypes), node.Properties["NodeType"].ToString());

                if ((type == NodeTypes.Area && !pubWeb.IncludeSubSitesInNavigation) || (type == NodeTypes.Page && !pubWeb.IncludePagesInNavigation))
                    continue;

                XmlElement xmlChildNode = xmlParentNode.OwnerDocument.CreateElement("Node");
                xmlParentNode.AppendChild(xmlChildNode);

                xmlChildNode.SetAttribute("Id", node.Id.ToString());
                xmlChildNode.SetAttribute("Title", node.Title);


                // Set the default visibility to true.
                xmlChildNode.SetAttribute("IsVisible", "True");

                #region Determine Visibility

                if (type == NodeTypes.Area)
                {
                    SPWeb web = null;
                    string name = node.Url.Trim('/');
                    if (name.Length != 0 && name.IndexOf("/") > 0)
                    {
                        name = name.Substring(name.LastIndexOf('/') + 1);
                    }
                    try
                    {
                        web = pubWeb.Web.Webs[name];
                    }
                    catch (ArgumentException)
                    {
                    }

                    if (web != null && web.Exists && web.ServerRelativeUrl.ToLower() == node.Url.ToLower() && PublishingWeb.IsPublishingWeb(web))
                    {
                        PublishingWeb tempPubWeb = PublishingWeb.GetPublishingWeb(web);
                        if (isGlobal)
                            xmlChildNode.SetAttribute("IsVisible", tempPubWeb.IncludeInGlobalNavigation.ToString());
                        else
                            xmlChildNode.SetAttribute("IsVisible", tempPubWeb.IncludeInCurrentNavigation.ToString());
                    }
                }
                else if (type == NodeTypes.Page)
                {
                    PublishingPage page = null;
                    try
                    {
                        page = pubWeb.GetPublishingPages()[node.Url];
                    }
                    catch (ArgumentException)
                    {
                    }
                    if (page != null)
                    {
                        if (isGlobal)
                            xmlChildNode.SetAttribute("IsVisible", page.IncludeInGlobalNavigation.ToString());
                        else
                            xmlChildNode.SetAttribute("IsVisible", page.IncludeInCurrentNavigation.ToString());
                    }
                }
                #endregion

                XmlElement xmlProp = xmlParentNode.OwnerDocument.CreateElement("Url");
                xmlProp.InnerText = node.Url;
                xmlChildNode.AppendChild(xmlProp);

                foreach (DictionaryEntry d in node.Properties)
                {
                    xmlProp = xmlParentNode.OwnerDocument.CreateElement(d.Key.ToString());
                    xmlProp.InnerText = d.Value.ToString();
                    xmlChildNode.AppendChild(xmlProp);
                }
                EnumerateCollection(pubWeb, isGlobal, node.Children, xmlChildNode);
            }
        }

        /// <summary>
        /// Enumerates the collection.
        /// </summary>
        /// <param name="nodes">The nodes.</param>
        /// <param name="sb">The sb.</param>
        /// <param name="level">The level.</param>
        private static void EnumerateCollection(SPNavigationNodeCollection nodes, ref StringBuilder sb, int level)
        {
            if (nodes == null || nodes.Count == 0)
                return;

            string indent = "";

            for (int i = 0; i < level; i++)
            {
                indent += "  ";
            }
            if (level > 0)
                indent += "- ";

            foreach (SPNavigationNode node in nodes)
            {
                sb.AppendFormat("{0}{1}: {2}\r\n", indent, node.Id, node.Title);
                sb.Append(indent.Replace('-', ' ') + "    :Url=" + node.Url + "\r\n");
                foreach (DictionaryEntry d in node.Properties)
                    sb.Append(indent.Replace('-', ' ') + "    :" + d.Key + "=" + d.Value + "\r\n");
                EnumerateCollection(node.Children, ref sb, level + 1);
            }
        }
    }
}
