using System;
using System.Collections.Specialized;
using System.IO;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Deployment;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class MoveWeb : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="MoveWeb"/> class.
        /// </summary>
        public MoveWeb()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the URL of the web to move."));
            parameters.Add(new SPParam("parenturl", "parent", true, null, new SPUrlValidator(), "Please specify the parent url."));
            parameters.Add(new SPParam("haltonwarning", "warning", false, null, null));
            parameters.Add(new SPParam("haltonfatalerror", "error", false, null, null));
            parameters.Add(new SPParam("includeusersecurity", "security", false, null, null));
            parameters.Add(new SPParam("retainobjectidentity", "retainid", false, null, null));
            Init(parameters, "\r\n\r\nMoves a web.\r\n\r\nParameters:\r\n\t-url <url of web to move>\r\n\t-parenturl <url of parent web>\r\n\t[-haltonwarning (only considered if moving to a new site collection)]\r\n\t[-haltonfatalerror (only considered if moving to a new site collection)]\r\n\t[-includeusersecurity (only considered if moving to a new site collection)]\r\n\t[-retainobjectidentity (only considered if moving to a new site collection)]\r\n");
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');
            string parentUrl = Params["parenturl"].Value.TrimEnd('/');
            bool retainObjectIdentity = Params["retainobjectidentity"].UserTypedIn;

            using (SPSite sourceSite = new SPSite(url))
            using (SPWeb sourceWeb = sourceSite.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
            using (SPSite parentSite = new SPSite(parentUrl))
            using (SPWeb parentWeb = parentSite.AllWebs[Utilities.GetServerRelUrlFromFullUrl(parentUrl)])
            {
                if (sourceWeb.ID == parentWeb.ID)
                {
                    throw new Exception("Source web and parent web cannot be the same.");
                }
                if (sourceWeb.ParentWeb != null && sourceWeb.ParentWeb.ID == parentWeb.ID)
                {
                    throw new Exception(
                        "Parent web specified matches the source web's current parent - move is not necessary.");
                }
                if (sourceWeb.IsRootWeb && sourceSite.ID == parentSite.ID)
                {
                    throw new Exception("Cannot move root web within the same site collection.");
                }
                if (sourceWeb.IsRootWeb && retainObjectIdentity)
                {
                    // If we allow retainobjectidentity when moving a root web the import will attempt to import over
                    // the parent web's site collection which would be really bad.
                    throw new Exception("Cannot move a root web when \"-retainobjectidentity\" is used.");
                }

                if (sourceSite.ID == parentSite.ID)
                {
                    // This ones the easy one - just need to set the property and update the web.
                    string oldUrl = sourceWeb.ServerRelativeUrl;

                    sourceWeb.ServerRelativeUrl = Utilities.ConcatServerRelativeUrls(parentWeb.ServerRelativeUrl, sourceWeb.Name);
                    sourceWeb.Update();

                    // Upgrade any redirect links if present.
                    UpdateV2ToV3UpgradeAreaUrlMappings.FixUpgradeAreaUrlMappings(sourceSite.WebApplication, oldUrl, sourceWeb.ServerRelativeUrl);

                }
                else
                {
                    // Now for the hard one - we need to move to another site collection which requires using the export/import commands.
                    bool haltOnWarning = Params["haltonwarning"].UserTypedIn;
                    bool haltOnFatalError = Params["haltonfatalerror"].UserTypedIn;
                    bool includeUserSecurity = Params["includeusersecurity"].UserTypedIn;


                    string webName = sourceWeb.Name;
                    if (string.IsNullOrEmpty(webName))
                        webName = sourceWeb.ServerRelativeUrl.Substring(sourceWeb.ServerRelativeUrl.LastIndexOf('/') + 1);
                    if (string.IsNullOrEmpty(webName))
                        webName = sourceWeb.ID.ToString();

                    string originalWebName = webName;
                    string tempWebName = Guid.NewGuid().ToString();
                    bool wasMoved = false;
                    if (retainObjectIdentity && sourceWeb.ParentWeb != null && !sourceWeb.ParentWeb.IsRootWeb)
                    {
                        Console.WriteLine("Moving source web to child of root web...");

                        sourceWeb.ServerRelativeUrl = Utilities.ConcatServerRelativeUrls(sourceSite.ServerRelativeUrl, tempWebName);
                        sourceWeb.Update();
                        wasMoved = true;

                        Console.WriteLine("Source web moved to: " + sourceWeb.ServerRelativeUrl + "\r\n");
                    }

                    Console.WriteLine("Exporting Web...");
                    string fileName = ExportHelper.ExportSite(sourceSite.MakeFullUrl(sourceWeb.ServerRelativeUrl), 
                        haltOnWarning, haltOnFatalError, true, includeUserSecurity, true, SPIncludeVersions.All, 0, false, true);
                    Console.WriteLine("Web exported.\r\n");

                    string newWebUrl = Utilities.ConcatServerRelativeUrls(parentUrl, webName);
                    if (!retainObjectIdentity)
                    {
                        Console.WriteLine("Creating web for import...");
                        CreateWeb(newWebUrl);
                        Console.WriteLine("Web created.\r\n");
                    }

                    // Store for later use as the source site may not be available due to the delete.
                    SPWebApplication sourceWebApp = sourceSite.WebApplication;

                    // Need to delete the web before importing if retaining the object identity.
                    if (retainObjectIdentity)
                        DeleteWeb(sourceWeb);

                    Console.WriteLine("Importing web...");
                    ImportHelper import = new ImportHelper();
                    import.ImportSite(fileName, newWebUrl, haltOnWarning, haltOnFatalError, true, 
                        includeUserSecurity, true, true, retainObjectIdentity, SPUpdateVersions.Append);
                    Console.WriteLine("Web imported.\r\n");

                    if (retainObjectIdentity && wasMoved)
                    {
                        Console.WriteLine("Renaming imported web to original name...");

                        // We first need to make sure that a web with the same name doesn't already exist.
                        // If it does then simply add an index to the name of the web.
                        int i = 0;
                        string suffix = "";
                        while (true)
                        {
                            i++;
                            try
                            {
                                using (SPWeb tempWeb = parentSite.AllWebs[Utilities.ConcatServerRelativeUrls(parentWeb.ServerRelativeUrl, originalWebName + suffix)])
                                {
                                    if (tempWeb.Exists)
                                    {
                                        suffix = i.ToString();
                                        continue;
                                    }
                                    else
                                        break;
                                }
                                    
                            }
                            catch(Exception)
                            {
                                suffix = i.ToString();
                            }
                            break;
                        }
                        using (SPWeb targetWeb = parentSite.AllWebs[Utilities.ConcatServerRelativeUrls(parentWeb.ServerRelativeUrl, tempWebName)])
                        {
                            originalWebName = originalWebName + suffix;
                            targetWeb.ServerRelativeUrl = Utilities.ConcatServerRelativeUrls(parentWeb.ServerRelativeUrl, originalWebName);
                            targetWeb.Update();

                            Console.WriteLine("Imported web renamed: " + targetWeb.ServerRelativeUrl + "\r\n");
                        }

                    }

                    Console.WriteLine("Repairing Area upgrade URLs...");
                    // Upgrade any redirect links if present.
                    string newWebServerRelativeUrl =
                        Utilities.ConcatServerRelativeUrls(parentWeb.ServerRelativeUrl, originalWebName);

                    // The spsredirect.aspx page which uses the upgrade list only supports server relative urls so
                    // if we change the link to be absolute the redirect won't work so don't bother changing it.
                    // Note that we could get around this if we had a simple redirect page that could take in
                    // a target url - then we could have spsredirect.aspx load the server relative redirect
                    // page which would have the actual url passed into it (unfortunately redirect.aspx does
                    // some funky stuff so we can't use it).
                    if (sourceWebApp.Id == parentSite.WebApplication.Id)
                    {
                        UpdateV2ToV3UpgradeAreaUrlMappings.FixUpgradeAreaUrlMappings(sourceWebApp,
                                                                                     sourceWeb.ServerRelativeUrl,
                                                                                     newWebServerRelativeUrl);
                    }
                    Console.WriteLine("Area upgrade URLs repaired.\r\n");

                    if (!retainObjectIdentity)
                    {
                        Console.WriteLine("Retargetting web parts...");
                        using (SPWeb targetWeb = parentSite.AllWebs[newWebServerRelativeUrl])
                        {
                            RepairSiteCollectionImportedFromSubSite.RetargetMiscWebParts(sourceSite, sourceWeb, targetWeb);

                            WebParts.RetargetGroupedListingsWebPart.RetargetGroupedListings(targetWeb, "grouped listings");
                        }
                        Console.WriteLine("Retargetting of web parts complete.");
                    }

                    // Need to delete the web after repairing if not retaining the object identity.
                    if (!retainObjectIdentity)
                        DeleteWeb(sourceWeb);

                    Directory.Delete(fileName, true);
                }
            }

            return 1;
        }

        #endregion

        /// <summary>
        /// Deletes the web.
        /// </summary>
        /// <param name="sourceWeb">The source web.</param>
        private static void DeleteWeb(SPWeb sourceWeb)
        {
            Console.WriteLine("Deleting source web...");
            if (!sourceWeb.IsRootWeb)
            {
                DeleteSubWebs(sourceWeb.Webs);

                sourceWeb.Delete();
                Console.WriteLine("Source web deleted.\r\n");
            }
            else
            {
                sourceWeb.Site.Delete();
            }
        }

        /// <summary>
        /// Deletes the sub webs.
        /// </summary>
        /// <param name="webs">The webs.</param>
        private static void DeleteSubWebs(SPWebCollection webs)
        {
            foreach (SPWeb web in webs)
            {
                if (web.Webs.Count > 0)
                    DeleteSubWebs(web.Webs);

                web.Delete();
            }
        }

        /// <summary>
        /// Creates the web.
        /// </summary>
        /// <param name="fullWebUrl">The full url of the web to create.</param>
        private static void CreateWeb(string fullWebUrl)
        {
            string strWebTemplate = null;
            string strTitle = null;
            string strDescription = null;
            uint nLCID = 0;
            bool convert = false;
            bool useUniquePermissions = false;

            using (SPSiteAdministration administration = new SPSiteAdministration(fullWebUrl))
                administration.AddWeb(Utilities.GetServerRelUrlFromFullUrl(fullWebUrl), strTitle, strDescription, nLCID, strWebTemplate, useUniquePermissions, convert);

        }

        

    }
}
