using System;
using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class SetAuditSettings : SPOperation
    {
        private enum ModeEnum
        {
            Replace, Add, Remove
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="SetAuditSettings"/> class.
        /// </summary>
        public SetAuditSettings()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site collection url."));
            SPEnumValidator modeValidator = new SPEnumValidator(typeof (ModeEnum));
            parameters.Add(new SPParam("mode", "m", true, null, modeValidator));
            foreach (string name in Enum.GetNames(typeof(SPAuditMaskType)))
            {
                parameters.Add(new SPParam(name, name.ToLowerInvariant()));
            }
            
            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSet the events that should be audited for documents, items, lists, libraries, and sites within the site collection.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <site collection url>");
            sb.AppendFormat("\r\n\t-mode <{0}>", modeValidator.DisplayValue);
            foreach (string name in Enum.GetNames(typeof(SPAuditMaskType)))
            {
                sb.Append("\r\n\t[-");
                sb.Append(name.ToLowerInvariant());
                sb.Append("]");
            }

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');
            ModeEnum mode = (ModeEnum) Enum.Parse(typeof (ModeEnum), Params["mode"].Value, true);

            using (SPSite site = new SPSite(url))
            {
                // Initialize the mask to it's default.
                SPAuditMaskType auditMask = SPAuditMaskType.None;
                if (mode != ModeEnum.Replace)
                    auditMask = site.Audit.AuditFlags; // We're not replace the mask so we need to store the curren settings.

                if (mode == ModeEnum.Remove)
                {
                    // Remove settings
                    foreach (SPAuditMaskType mask in Enum.GetValues(typeof(SPAuditMaskType)))
                    {
                        if (Params[mask.ToString()].UserTypedIn)
                            auditMask = auditMask & ~mask;
                    }
                }
                else
                {
                    // Add settings (replace is just an add but starts with a blank slate)
                    foreach (SPAuditMaskType mask in Enum.GetValues(typeof(SPAuditMaskType)))
                    {
                        if (Params[mask.ToString()].UserTypedIn)
                            auditMask = auditMask | mask;
                    }
                }
                // Update the Audit object with the new flags
                site.Audit.AuditFlags = auditMask;
                site.Audit.Update();
            }

            return 1;
        }

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            bool none = false;
            bool all = false;
            bool specific = false;
            foreach (SPAuditMaskType mask in Enum.GetValues(typeof(SPAuditMaskType)))
            {
                if (Params[mask.ToString()].UserTypedIn)
                {
                    if (mask == SPAuditMaskType.All)
                        all = true;
                    else if (mask == SPAuditMaskType.None)
                        none = true;
                    else
                        specific = true;
                }
            }
            if (all && none)
                throw new SPSyntaxException("The 'All' and 'None' flags cannot be used together.");
            if (all && none && specific)
                throw new SPSyntaxException("The 'All' and 'None' flags cannot be used together or with other flags.");
            if (all && specific)
                throw new SPSyntaxException("The 'All' flag cannot be used with other flags.");
            if (none && specific)
                throw new SPSyntaxException("The 'None' flag cannot be used with other flags.");

            base.Validate(keyValues);
        }

        #endregion

    }
}
