using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Navigation;
using Microsoft.SharePoint.Publishing;
using Microsoft.SharePoint.Publishing.Navigation;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class SetNavigationNodes : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetNavigationNodes"/> class.
        /// </summary>
        public SetNavigationNodes()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site collection."));
            parameters.Add(new SPParam("inputfile", "input", true, null, new SPFileExistsValidator(), "Please specify the input file name."));
            parameters.Add(new SPParam("showsubsites", "subsites", false, "true", new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            parameters.Add(new SPParam("showpages", "pages", false, "false", new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            parameters.Add(new SPParam("deleteexistingglobal", "deleteglobal", false, "true", new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            parameters.Add(new SPParam("deleteexistingcurrent", "deletecurrent", false, "true", new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            parameters.Add(new SPParam("backuptarget", "backup", false, null, new SPDirectoryExistsAndValidFileNameValidator(), "Make sure the backup file directory exists and a valid filename is provided."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nRebuilds the Global and/or Current navigation based on a passed in XML file which can be generated by the enumnavigation command and then modified (note that the Id attribute is ignored).\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <site collection url>");
            sb.Append("\r\n\t-inputfile <xml file to use as input>");
            sb.Append("\r\n\t[-showsubsites <true (default) | false>]");
            sb.Append("\r\n\t[-showpages <true | false (default)>]");
            sb.Append("\r\n\t[-deleteexistingglobal <true (default) | false>]");
            sb.Append("\r\n\t[-deleteexistingcurrent <true (default) | false>]");
            sb.Append("\r\n\t[-backuptarget <filename>]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');
            string filename = Params["inputfile"].Value;

            bool showSubSites = bool.Parse(Params["showsubsites"].Value);
            bool showPages = bool.Parse(Params["showpages"].Value);
            bool deleteExistingGlobal = bool.Parse(Params["deleteexistingglobal"].Value);
            bool deleteExistingCurrent = bool.Parse(Params["deleteexistingcurrent"].Value);
            

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(filename);


            using (SPSite site = new SPSite(url))
            {
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    if (Params["backuptarget"].UserTypedIn)
                    {
                        PublishingWeb targetPubWeb = PublishingWeb.GetPublishingWeb(web);
                        XmlDocument xmlBackupDoc = EnumNavigation.GetNavigationAsXml(targetPubWeb);
                        xmlBackupDoc.Save(Params["backuptarget"].Value);
                    }

                    SetNavigation(site, web, xmlDoc, showSubSites, showPages, deleteExistingGlobal, deleteExistingCurrent);
                }
            }

            return 1;
        }

        /// <summary>
        /// Replaces any occurance of "SiteCollectionUrl" nodes with text corresponding to the 
        /// server relative url of the site collection.
        /// </summary>
        /// <param name="site">The site.</param>
        /// <param name="xmlDoc">The XML doc.</param>
        private static void AutoAddSiteCollectionUrl(SPSite site, XmlDocument xmlDoc)
        {
            XmlNodeList autoAddList = xmlDoc.SelectNodes("//SiteCollectionUrl");
            if (autoAddList.Count == 0)
                return;

            foreach (XmlElement autoAddElement in autoAddList)
            {
                XmlText textNode = xmlDoc.CreateTextNode(site.ServerRelativeUrl);
                autoAddElement.ParentNode.ReplaceChild(textNode, autoAddElement);
            }
        }

        /// <summary>
        /// Replaces any occurance of "SiteCollectionUrl" nodes with text corresponding to the 
        /// server relative url of the site collection.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="xmlDoc">The XML doc.</param>
        private static void AutoAddWebUrl(SPWeb web, XmlDocument xmlDoc)
        {
            XmlNodeList autoAddList = xmlDoc.SelectNodes("//WebUrl");
            if (autoAddList.Count == 0)
                return;

            foreach (XmlElement autoAddElement in autoAddList)
            {
                XmlText textNode = xmlDoc.CreateTextNode(web.ServerRelativeUrl);
                autoAddElement.ParentNode.ReplaceChild(textNode, autoAddElement);
            }
        }

        /// <summary>
        /// Replaces any occurance of "AutoAddSubSites" nodes with nodes corresponding to each of the sub-sites for the web.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="xmlDoc">The XML doc.</param>
        private static void AutoAddSubSites(SPWeb web, XmlDocument xmlDoc)
        {
            XmlNodeList autoAddList = xmlDoc.SelectNodes("//AutoAddSubSites");
            if (autoAddList.Count == 0)
                return;

            foreach (XmlElement autoAddElement in autoAddList)
            {
                foreach (SPWeb childWeb in web.Webs)
                {
                    try
                    {
                        XmlElement navNode = xmlDoc.CreateElement("Node");
                        navNode.SetAttribute("Title", childWeb.Title);
                        navNode.SetAttribute("IsVisible", "True");
                        
                        XmlElement prop = xmlDoc.CreateElement("Url");
                        prop.InnerText = childWeb.ServerRelativeUrl;
                        navNode.AppendChild(prop);

                        prop = xmlDoc.CreateElement("vti_navsequencechild");
                        prop.InnerText = "true";
                        navNode.AppendChild(prop);

                        prop = xmlDoc.CreateElement("NodeType");
                        prop.InnerText = "Area";
                        navNode.AppendChild(prop);

                        prop = xmlDoc.CreateElement("Description");
                        prop.InnerText = childWeb.Description;
                        navNode.AppendChild(prop);

                        autoAddElement.ParentNode.AppendChild(navNode);
                    }
                    finally
                    {
                        childWeb.Dispose();
                    }
                }
                autoAddElement.ParentNode.RemoveChild(autoAddElement);
            }
        }

        /// <summary>
        /// Sets the navigation.
        /// </summary>
        /// <param name="site">The site.</param>
        /// <param name="web">The web site.</param>
        /// <param name="xmlDoc">The XML doc containing the navigation nodes.</param>
        /// <param name="showSubSites">if set to <c>true</c> [show sub sites].</param>
        /// <param name="showPages">if set to <c>true</c> [show pages].</param>
        /// <param name="deleteExistingGlobal">if set to <c>true</c> [delete existing global nodes].</param>
        /// <param name="deleteExistingCurrent">if set to <c>true</c> [delete existing current nodes].</param>
        public static void SetNavigation(SPSite site, SPWeb web, XmlDocument xmlDoc, bool showSubSites, bool showPages, bool deleteExistingGlobal, bool deleteExistingCurrent)
        {
            PublishingWeb pubweb = PublishingWeb.GetPublishingWeb(web);

            AutoAddSubSites(web, xmlDoc);
            AutoAddSiteCollectionUrl(site, xmlDoc);
            AutoAddWebUrl(web, xmlDoc);

            // First need to set whether or not we show sub-sites and pages
            pubweb.IncludeSubSitesInNavigation = showSubSites;
            pubweb.IncludePagesInNavigation = showPages;
            pubweb.Update();

            List<SPNavigationNode> existingGlobalNodes = new List<SPNavigationNode>();
            List<SPNavigationNode> existingCurrentNodes = new List<SPNavigationNode>();
            // We can't delete the navigation items until we've added the new ones so store the existing 
            // ones for later deletion (note that we don't have to store all of them - just the top level).
            // I have no idea why this is the case - but when I tried to clear everything out first I got
            // all kinds of funky errors that just made no sense to me - this works so....
            foreach (SPNavigationNode node in pubweb.GlobalNavigationNodes)
                existingGlobalNodes.Add(node);
            foreach (SPNavigationNode node in pubweb.CurrentNavigationNodes)
                existingCurrentNodes.Add(node);

            XmlNodeList newGlobalNodes = xmlDoc.SelectNodes("//Navigation/Global/Node");
            XmlNodeList newCurrentNodes = xmlDoc.SelectNodes("//Navigation/Current/Node");


            // If we've got global or current nodes in the xml then the intent is to reset those elements.
            // If we've also specified to delete any existing elements then we need to first hide all the
            // sub-sites and pages (you can't delete them because they don't exist as a node).  Note that
            // we are only doing this if showSubSites is true - if it's false we don't see them so no point
            // in hiding them.  Any non-sub-site or non-page will be deleted after we've added the new nodes.
            if (newGlobalNodes.Count > 0 && deleteExistingGlobal && showSubSites)
            {
                // Initialize the sub-sites (forces the provided XML to specify whether any should be visible)
                foreach (SPWeb tempWeb in pubweb.Web.Webs)
                {
                    pubweb.ExcludeFromNavigation(true, tempWeb.ID);
                }
                pubweb.Update();
            }
            if (newCurrentNodes.Count > 0 && deleteExistingCurrent && showSubSites)
            {
                foreach (SPWeb tempWeb in pubweb.Web.Webs)
                {
                    pubweb.ExcludeFromNavigation(false, tempWeb.ID);
                }
                pubweb.Update();
            }

            // Now we need to add all the global nodes (if any - if the collection is empty the following will just return and do nothing)
            AddNodes(pubweb, true, pubweb.GlobalNavigationNodes, newGlobalNodes);
            // Update the web as the above may have made modifications
            pubweb.Update();

            // Now delete all the previously existing global nodes.
            if (newGlobalNodes.Count > 0 && deleteExistingGlobal)
            {
                foreach (SPNavigationNode node in existingGlobalNodes)
                {
                    node.Delete();
                }
            }

            // Now we need to add all the current nodes (if any)
            AddNodes(pubweb, false, pubweb.CurrentNavigationNodes, newCurrentNodes);
            // Update the web as the above may have made modifications
            pubweb.Update();

            // Now delete all the previously existing current nodes.
            if (newCurrentNodes.Count > 0 && deleteExistingCurrent)
            {
                foreach (SPNavigationNode node in existingCurrentNodes)
                {
                    node.Delete();
                }
            }
        }


        /// <summary>
        /// Adds the nodes.
        /// </summary>
        /// <param name="pubWeb">The publishing web.</param>
        /// <param name="isGlobal">if set to <c>true</c> [is global].</param>
        /// <param name="existingNodes">The existing nodes.</param>
        /// <param name="newNodes">The new nodes.</param>
        private static void AddNodes(PublishingWeb pubWeb, bool isGlobal, SPNavigationNodeCollection existingNodes, XmlNodeList newNodes)
        {
            if (newNodes.Count == 0)
                return;

            for (int i = 0; i < newNodes.Count; i++)
            {
                XmlElement newNodeXml = (XmlElement)newNodes[i];
                string url = newNodeXml.SelectSingleNode("Url").InnerText;
                string title = newNodeXml.GetAttribute("Title");
                NodeTypes type = NodeTypes.None;
                if (newNodeXml.SelectSingleNode("NodeType") != null && !string.IsNullOrEmpty(newNodeXml.SelectSingleNode("NodeType").InnerText))
                    type = (NodeTypes)Enum.Parse(typeof(NodeTypes), newNodeXml.SelectSingleNode("NodeType").InnerText);

                bool isVisible = true;
                if (!string.IsNullOrEmpty(newNodeXml.GetAttribute("IsVisible")))
                    isVisible = bool.Parse(newNodeXml.GetAttribute("IsVisible"));

                if (type == NodeTypes.Area)
                {
                    // You can't just add an "Area" node (which represents a sub-site) to the current web if the
                    // url does not correspond with an actual sub-site (the code will appear to work but you won't
                    // see anything when you load the page).  So we need to check and see if the node actually
                    // points to a sub-site - if it does not then change it to "AuthoredLinkToWeb".
                    SPWeb web = null;
                    string name = url.Trim('/');
                    if (name.Length != 0 && name.IndexOf("/") > 0)
                    {
                        name = name.Substring(name.LastIndexOf('/') + 1);
                    }
                    try
                    {
                        // Note that pubWeb.Web.Webs[] does not return null if the item doesn't exist - it simply throws an exception (I hate that!)
                        web = pubWeb.Web.Webs[name];
                    }
                    catch (ArgumentException)
                    {
                    }
                    if (web == null || !web.Exists || web.ServerRelativeUrl.ToLower() != url.ToLower())
                    {
                        // The url doesn't correspond with a sub-site for the current web so change the node type.
                        // This is most likely due to copying navigation elements from another site
                        type = NodeTypes.AuthoredLinkToWeb;
                    }
                    else if (web.Exists && web.ServerRelativeUrl.ToLower() == url.ToLower())
                    {
                        // We did find a matching sub-site so now we need to set the visibility
                        if (isVisible)
                            pubWeb.IncludeInNavigation(isGlobal, web.ID);
                        else
                            pubWeb.ExcludeFromNavigation(isGlobal, web.ID);
                    }

                }
                else if (type == NodeTypes.Page)
                {
                    // Adding links to pages has the same limitation as sub-sites (Area nodes) so we need to make
                    // sure it actually exists and if it doesn't then change the node type.
                    PublishingPage page = null;
                    try
                    {
                        // Note that GetPublishingPages()[] does not return null if the item doesn't exist - it simply throws an exception (I hate that!)
                        page = pubWeb.GetPublishingPages()[url];
                    }
                    catch (ArgumentException)
                    {
                    }
                    if (page == null)
                    {
                        // The url doesn't correspond with a page for the current web so change the node type.
                        // This is most likely due to copying navigation elements from another site
                        type = NodeTypes.AuthoredLinkToPage;
                        url = pubWeb.Web.Site.MakeFullUrl(url);
                    }
                    else
                    {
                        // We did find a matching page so now we need to set the visibility
                        if (isVisible)
                            pubWeb.IncludeInNavigation(isGlobal, page.ListItem.UniqueId);
                        else
                            pubWeb.ExcludeFromNavigation(isGlobal, page.ListItem.UniqueId);
                    }
                }

                // If it's not a sub-site or a page that's part of the current web and it's set to
                // not be visible then just move on to the next (there is no visibility setting for
                // nodes that are not of type Area or Page).
                if (!isVisible && type != NodeTypes.Area && type != NodeTypes.Page)
                    continue;

                // Finally, can add the node to the collection.
                SPNavigationNode node = SPNavigationSiteMapNode.CreateSPNavigationNode(
                    title, url, type, existingNodes);

               
                // Now we need to set all the other properties
                foreach (XmlElement property in newNodeXml.ChildNodes)
                {
                    // We've already set these so don't set them again.
                    if (property.Name == "Url" || property.Name == "Node" || property.Name == "NodeType")
                        continue;

                    // CreatedDate and LastModifiedDate need to be the correct type - all other properties are strings
                    if (property.Name == "CreatedDate" && !string.IsNullOrEmpty(property.InnerText))
                    {
                        node.Properties["CreatedDate"] = DateTime.Parse(property.InnerText);
                        continue;
                    }
                    if (property.Name == "LastModifiedDate" && !string.IsNullOrEmpty(property.InnerText))
                    {
                        node.Properties["LastModifiedDate"] = DateTime.Parse(property.InnerText);
                        continue;
                    }

                    node.Properties[property.Name] = property.InnerText;
                }
                // If we didn't have a CreatedDate or LastModifiedDate then set them to now.
                if (node.Properties["CreatedDate"] == null)
                    node.Properties["CreatedDate"] = DateTime.Now;
                if (node.Properties["LastModifiedDate"] == null)
                    node.Properties["LastModifiedDate"] = DateTime.Now;

                // Save our changes to the node.
                node.Update();
                node.MoveToLast(existingNodes); // Should already be at the end but I prefer to make sure :)
                
                XmlNodeList childNodes = newNodeXml.SelectNodes("Node");

                // If we have child nodes then make a recursive call passing in the current nodes Children property as the collection to add to.
                if (childNodes.Count > 0)
                    AddNodes(pubWeb, isGlobal, node.Children, childNodes);
            }

        }

        #endregion
    }
}
