using System;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Publishing;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class SetNavigationSettings : SPOperation
    {
        internal enum CurrentNavSettingsEnum
        {
            InheritParent,
            CurrentSiteAndSiblings,
            CurrentSiteOnly
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SetNavigationSettings"/> class.
        /// </summary>
        public SetNavigationSettings()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator()));
            parameters.Add(new SPParam("showsubsites", "subsites", false, null, new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            parameters.Add(new SPParam("showpages", "pages", false, null, new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            SPEnumValidator sortMethodValidator = new SPEnumValidator(typeof (OrderingMethod));
            parameters.Add(new SPParam("sortmethod", "sm", false, null, sortMethodValidator));
            SPEnumValidator autoSortMethodValidator = new SPEnumValidator(typeof(AutomaticSortingMethod));
            parameters.Add(new SPParam("autosortmethod", "asm", false, null, autoSortMethodValidator));
            parameters.Add(new SPParam("sortascending", "sa", false, null, new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            parameters.Add(new SPParam("inheritglobalnav", "ig", false, null, new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            SPEnumValidator currentNavValidator = new SPEnumValidator(typeof(CurrentNavSettingsEnum));
            parameters.Add(new SPParam("currentnav", "c", false, null, currentNavValidator));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the navigation settings for a web site (use setnavigationnodes to change the actual nodes that appear).\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <site collection url>");
            sb.Append("\r\n\t[-showsubsites <true | false>]");
            sb.Append("\r\n\t[-showpages <true | false>]");
            sb.AppendFormat("\r\n\t[-sortmethod <{0}>]", sortMethodValidator.DisplayValue);
            sb.AppendFormat("\r\n\t[-autosortmethod <{0}>]", autoSortMethodValidator.DisplayValue);
            sb.Append("\r\n\t[-sortascending <true | false>]");
            sb.Append("\r\n\t[-inheritglobalnav <true | false>]");
            sb.AppendFormat("\r\n\t[-currentnav <{0}>]", currentNavValidator.DisplayValue);

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');

            using (SPSite site = new SPSite(url))
            {
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    PublishingWeb pubweb = PublishingWeb.GetPublishingWeb(web);

                    if (Params["showsubsites"].UserTypedIn)
                        pubweb.IncludeSubSitesInNavigation = bool.Parse(Params["showsubsites"].Value);

                    if (Params["showpages"].UserTypedIn)
                        pubweb.IncludePagesInNavigation = bool.Parse(Params["showpages"].Value);

                    OrderingMethod sortMethod = pubweb.NavigationOrderingMethod;
                    if (Params["sortmethod"].UserTypedIn)
                    {
                        sortMethod = (OrderingMethod)Enum.Parse(typeof (OrderingMethod), Params["sortmethod"].Value, true);
                        pubweb.NavigationOrderingMethod = sortMethod;
                    }

                    if (sortMethod != OrderingMethod.Manual)
                    {
                        if (Params["autosortmethod"].UserTypedIn)
                            pubweb.NavigationAutomaticSortingMethod = (AutomaticSortingMethod)Enum.Parse(typeof(AutomaticSortingMethod), Params["autosortmethod"].Value, true);
                        if (Params["sortascending"].UserTypedIn)
                            pubweb.NavigationSortAscending = bool.Parse(Params["sortascending"].Value);
                    }
                    else
                    {
                        if (Params["autosortmethod"].UserTypedIn)
                            Console.WriteLine("WARNING: parameter autosortmethod is incompatible with sortmethod {0}.  The parameter will be ignored.", sortMethod);
                        if (Params["sortascending"].UserTypedIn)
                            Console.WriteLine("WARNING: parameter sortascending is incompatible with sortmethod {0}.  The parameter will be ignored.", sortMethod);
                    }

                    if (Params["inheritglobalnav"].UserTypedIn)
                        pubweb.InheritGlobalNavigation = bool.Parse(Params["inheritglobalnav"].Value);

                    if (Params["currentnav"].UserTypedIn)
                    {
                        CurrentNavSettingsEnum currentNav = (CurrentNavSettingsEnum)Enum.Parse(typeof (CurrentNavSettingsEnum), Params["currentnav"].Value, true);
                        if (currentNav == CurrentNavSettingsEnum.InheritParent)
                        {
                            pubweb.InheritCurrentNavigation = true;
                            pubweb.NavigationShowSiblings = false;
                        }
                        else if (currentNav == CurrentNavSettingsEnum.CurrentSiteAndSiblings)
                        {
                            pubweb.InheritCurrentNavigation = true;
                            pubweb.NavigationShowSiblings = true;
                        }
                        else if (currentNav == CurrentNavSettingsEnum.CurrentSiteOnly)
                        {
                            pubweb.InheritCurrentNavigation = false;
                            pubweb.NavigationShowSiblings = false;
                        }
                    }

                    pubweb.Update();
                }
            }

            return 1;
        }

        #endregion
    }
}
