using System.Collections.Specialized;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class SetSiteGeneralSettings : SPOperation
    {
        private string m_siteLogoUrl;
        private string m_siteLogoDesc;

        /// <summary>
        /// Initializes a new instance of the <see cref="SetSiteGeneralSettings"/> class.
        /// </summary>
        public SetSiteGeneralSettings()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site url."));
            parameters.Add(new SPParam("title", "title", false, null, new SPNullOrNonEmptyValidator(), "Please specify the site title."));
            parameters.Add(new SPParam("description", "desc", false, null, new SPNullOrNonEmptyValidator(), "Please specify the description for the site."));
            parameters.Add(new SPParam("logourl", "logo", false, null, new SPNullOrNonEmptyValidator(), "Please specify the url to the site logo."));
            parameters.Add(new SPParam("logoalttext", "alt", false, null, new SPNullOrNonEmptyValidator(), "Please specify the alternative text for the logo."));
            parameters.Add(new SPParam("recurselogochanges", "recurselogo", false, null, null));
            Init(parameters, "\r\n\r\nSets the general properties of a site.\r\n\r\nParameters:\r\n\t-url <site url>\r\n\t[-title <site title>]\r\n\t[-description <description for the site>]\r\n\t[-logourl <site logo url>]\r\n\t[-logoalttext <alternative text for logo>]\r\n\t[-recurselogochanges (recurse logo changes to all sub-webs)]");
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;

            using (SPSite site = new SPSite(url))
            {
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    if (Params["title"].UserTypedIn)
                        web.Title = Params["title"].Value;

                    if (Params["description"].UserTypedIn)
                        web.Description = Params["description"].Value;

                    if (!Params["recurselogochanges"].UserTypedIn)
                    {
                        if (Params["logourl"].UserTypedIn)
                            web.SiteLogoUrl = Params["logourl"].Value;

                        if (Params["logoalttext"].UserTypedIn)
                            web.SiteLogoDescription = Params["logoalttext"].Value;
                    }
                    else
                    {
                        if (!Params["logourl"].UserTypedIn && !Params["logoalttext"].UserTypedIn)
                        {
                            // If no logo information was provided but the recurse flag was then the intent is 
                            // to recurse the current value down so we need to save locally the current value so it
                            // can be used in the recursion method.
                            m_siteLogoUrl = web.SiteLogoUrl;
                            m_siteLogoDesc = web.SiteLogoDescription;
                        }
                        RecurseSubWebs(web);
                    }
                    web.Update();
                }
            }

            return 1;
        }

        #endregion


        /// <summary>
        /// Recurses the sub webs.
        /// </summary>
        /// <param name="web">The web.</param>
        private void RecurseSubWebs(SPWeb web)
        {
            foreach (SPWeb subweb in web.Webs)
            {
                try
                {
                    RecurseSubWebs(subweb);
                }
                finally
                {
                    subweb.Dispose();
                }
            }
            bool modified = false;
            if (Params["logourl"].UserTypedIn)
            {
                web.SiteLogoUrl = Params["logourl"].Value;
                modified = true;
            }

            if (Params["logoalttext"].UserTypedIn)
            {
                web.SiteLogoDescription = Params["logoalttext"].Value;
                modified = true;
            }

            if (!Params["logourl"].UserTypedIn && !Params["logoalttext"].UserTypedIn && Params["recurselogochanges"].UserTypedIn)
            {
                web.SiteLogoUrl = m_siteLogoUrl;
                web.SiteLogoDescription = m_siteLogoDesc;
                modified = true;
            }

            if (modified)
                web.Update();
        }

    }
}
