using System;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class Upgrade2 : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Upgrade2"/> class.
        /// </summary>
        public Upgrade2()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", false, null, new SPUrlValidator()));
            parameters.Add(new SPParam("forceupgrade", "forceupgrade"));
            parameters.Add(new SPParam("quiet", "q"));
            parameters.Add(new SPParam("inplace", "inplace"));
            parameters.Add(new SPParam("sidebyside", "sidebyside"));
            parameters.Add(new SPParam("farmuser", "fu", false, null, new SPNullOrNonEmptyValidator()));
            parameters.Add(new SPParam("farmpassword", "fp", false, null, new SPNullOrNonEmptyValidator()));
            parameters.Add(new SPParam("reghostonupgrade", "reghost"));
            parameters.Add(new SPParam("finalizeupgrade", "finalize"));
            parameters.Add(new SPParam("sitelistpath", "sitelistpath", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("targetdb", "targetdb", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("moveto", "move", false, null, new SPUrlValidator()));
            parameters.Add(new SPParam("theme", "theme", false, null, new SPNonEmptyValidator()));


            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSlightly more flexible version of the built-in upgrade command (addresses the fact that the URL parameter does not do anything).\r\n\r\nParameters:");
            sb.Append("\r\n\t{-inplace |\r\n\t -sidebyside}");
            sb.Append("\r\n\t[-url <url>]");
            sb.Append("\r\n\t[-forceupgrade]");
            sb.Append("\r\n\t[-quiet]");
            sb.Append("\r\n\t[-farmuser <farm user>]");
            sb.Append("\r\n\t[-farmpassword <farm user password>]");
            sb.Append("\r\n\t[-reghost]");
            sb.Append("\r\n\t[-sitelistpath <sites xml file>]");
            sb.Append("\r\n\t[-targetdb <target database name>]");
            sb.Append("\r\n\t[-moveto <url of a new target path to move the upgraded site collection to>]");
            sb.AppendFormat("\r\n\t-theme <id of the theme to apply (see {0}\\Layouts\\[LCID]\\SPThemes.xml for template IDs>", SPUtility.GetGenericSetupPath("Template"));

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            /* stsadm.exe -o upgrade
                   {-inplace |
                    -sidebyside}
                   [-url <url>]
                   [-forceupgrade]
                   [-quiet]
                   [-farmuser <farm user>]
                   [-farmpassword <farm user password>]
                   [-reghost]
                   [-sitelistpath <sites xml file>]
            */
            StringBuilder cmd = new StringBuilder();
            cmd.Append(" -o upgrade");

            if (Params["forceupgrade"].UserTypedIn)
                cmd.Append(" -forceupgrade");

            if (Params["quiet"].UserTypedIn)
                cmd.Append(" -quiet");

            if (Params["inplace"].UserTypedIn)
                cmd.Append(" -inplace");
            else if (Params["sidebyside"].UserTypedIn)
                cmd.Append(" -sidebyside");

            if (Params["reghost"].UserTypedIn)
                cmd.Append(" -reghost");

            if (Params["farmuser"].UserTypedIn)
                cmd.AppendFormat(" -farmuser \"{0}\"", Params["farmuser"].Value);

            if (Params["farmpassword"].UserTypedIn)
                cmd.AppendFormat(" -farmpassword \"{0}\"", Params["farmpassword"].Value);

            if (Params["sidebyside"].UserTypedIn && Params["sitelistpath"].UserTypedIn)
                cmd.AppendFormat(" -sitelistpath \"{0}\"", Params["sitelistpath"].Value);

            if (Params["inplace"].UserTypedIn && Params["url"].UserTypedIn)
                cmd.AppendFormat(" -url \"{0}\"", Params["url"].Value);

            string filename = null;
            if (Params["sidebyside"].UserTypedIn && Params["url"].UserTypedIn)
            {
                filename = Path.GetTempFileName();
                string xml = string.Format("<RedirectedSites Count=\"1\"><Site Url=\"{0}\"", Params["url"].Value);
                if (Params["targetdb"].UserTypedIn)
                {
                    xml += string.Format(" TargetDatabase=\"{0}\"", Params["targetdb"].Value);
                }
                xml += " /></RedirectedSites>";
                File.WriteAllText(filename, xml);

                cmd.AppendFormat(" -sitelistpath \"{0}\"", filename);
            }

            Console.WriteLine("Upgrading site...");
            if (Utilities.RunStsAdmOperation(cmd.ToString(), true) != 0)
                throw new SPException("Error occured upgrading.\r\nCOMMAND: stsadm.exe" + cmd);
            Console.WriteLine("Site upgraded.\r\n");

            if (filename != null)
                File.Delete(filename);

            if (Params["moveto"].UserTypedIn)
            {
                Console.WriteLine("Moving site collection to new location...");
                MoveSite.MoveSiteCollection(Params["url"].Value, Params["moveto"].Value);
                Console.WriteLine("Site collection moved.");
            }

            if (Params["theme"].UserTypedIn)
            {
                Console.WriteLine("Applying specified theme to site collection...");
                Themes.ApplyTheme.ApplyThemeToWeb(Params["theme"].Value, Params["moveto"].Value, true);
                Console.WriteLine("Theme applied.\r\n");
            }

            return 1;
        }
        
        #endregion

        

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (!(Params["inplace"].UserTypedIn ^ Params["sidebyside"].UserTypedIn))
            {
                throw new SPSyntaxException(SPResource.GetString("InPlaceOrSideBySide", new object[] { "inplace", "sidebyside" }));
            }
            if (!string.IsNullOrEmpty(Params["url"].Value) && !string.IsNullOrEmpty(Params["sitelistpath"].Value))
            {
                throw new SPSyntaxException(SPResource.GetString("IncompatibleParametersSpecified", new object[] { "url", "sitelistpath" }));
            }
            if (!string.IsNullOrEmpty(Params["targetdb"].Value) && !string.IsNullOrEmpty(Params["sitelistpath"].Value))
            {
                throw new SPSyntaxException(SPResource.GetString("IncompatibleParametersSpecified", new object[] { "targetdb", "sitelistpath" }));
            }
            if (!string.IsNullOrEmpty(Params["targetdb"].Value) && Params["inplace"].UserTypedIn)
            {
                throw new SPSyntaxException(SPResource.GetString("IncompatibleParametersSpecified", new object[] { "targetdb", "inplace" }));
            }
            if (Params["moveto"].UserTypedIn)
            {
                Params["url"].Enabled = true;
                Params["url"].IsRequired = true;
            }
            else
                Params["url"].Enabled = Params["url"].UserTypedIn;

            bool isSideBySide = Params["sidebyside"].UserTypedIn;
            Params["sitelistpath"].IsRequired = isSideBySide && !Params["url"].UserTypedIn;
            Params["sitelistpath"].Enabled = isSideBySide;
            if (Params["sitelistpath"].UserTypedIn && !Params["sidebyside"].UserTypedIn)
            {
                throw new SPSyntaxException(SPResource.GetString("SiteListPathMisuse", new object[] { "sitelistpath" }));
            }
            base.Validate(keyValues);
        }


    }
}
