using System;
using System.Collections.Specialized;
using System.Globalization;
using System.IO;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Portal.SiteAdmin;
using Microsoft.SharePoint.Utilities;
using SiteDirectoryEntryRequirements=Microsoft.SharePoint.Portal.ServerAdmin.SiteDirectoryEntryRequirements;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteDirectory
{
    public class SetLocalSiteDirectory : SPOperation
    {

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SetLocalSiteDirectory"/> class.
        /// </summary>
        public SetLocalSiteDirectory()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("siteurl", "site", true, null, new SPUrlValidator(), "Please specify the site collection url."));
            parameters.Add(new SPParam("url", "url", true, null, new SPNonEmptyValidator(), "Please specify the site directory url (must be site relative, i.e. \"/SiteDirectory\")."));
            parameters.Add(new SPParam("enforcelistinginsitedir", "enforce", false, null, null));
            parameters.Add(new SPParam("sitedirentryrequirement", "entryreq", false, null, new SPIntRangeValidator(0, 2), "Please specify what information is required to be filled in by new site creators (0=none, 1=at least one category, 2=all categories)."));
            parameters.Add(new SPParam("allowsitecollectioncreation", "allowsitecreation", false, null, null));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the local, or site collection, site directory.\r\n\r\nParameters:");
            sb.Append("\r\n\t-siteurl <site collection url>");
            sb.Append("\r\n\t-url <site directory url>");
            sb.Append("\r\n\t[-enforcelistinginsitedir]");
            sb.Append("\r\n\t[-sitedirentryrequirement <0=none, 1=at least one category, 2=all categories>]");
            sb.Append("\r\n\t[-allowsitecollectioncreation]");

            Init(parameters, sb.ToString());
        }

        #endregion


        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            bool enforceNewListing = Params["enforcelistinginsitedir"].UserTypedIn;
            SiteDirectoryEntryRequirements entryRequirements = SiteDirectoryEntryRequirements.SiteDirectoryCatsOptional;

            bool allowSiteCollectionCreation = Params["allowsitecollectioncreation"].UserTypedIn;

            if (enforceNewListing)
            {
    			string sitedirentryrequirement = Params["sitedirentryrequirement"].Value;
                if (sitedirentryrequirement == "1")
                    entryRequirements = SiteDirectoryEntryRequirements.OneSiteDirectoryCatMandatory;
                else if (sitedirentryrequirement == "2")
                    entryRequirements = SiteDirectoryEntryRequirements.AllSiteDirectoryCatMandatory;
                else if (sitedirentryrequirement == "0")
                    entryRequirements = SiteDirectoryEntryRequirements.SiteDirectoryCatsOptional;
            }

			string url = Params["url"].Value.TrimEnd('/');
            string siteUrl = Params["siteurl"].Value;

            using (SPSite site = new SPSite(siteUrl))
            {
                if (!IsValidSiteDirectoryLocation(site, url))
                {
                    throw new SPException("The site directory location entered is not a site collection relative path, or the specified Site Directory does not exist in the specified Site Collection.");
                }

                WriteToWebPropertyBag(site, url, enforceNewListing, entryRequirements, allowSiteCollectionCreation);
            }

            return 1;
        }

        #endregion

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["enforcelistinginsitedir"].UserTypedIn)
            {
                Params["sitedirentryrequirement"].IsRequired = true;
                Params["sitedirentryrequirement"].Enabled = true;
            }
            base.Validate(keyValues);
        }

        /// <summary>
        /// Determines whether the site directory location is valid.
        /// </summary>
        /// <param name="site">The site containing the site directory.</param>
        /// <param name="siteDirectoryLoc">The site directory location.</param>
        /// <returns>
        /// 	<c>true</c> if the site directory location is valid then <c>true</c>; otherwise, <c>false</c>.
        /// </returns>
        private static bool IsValidSiteDirectoryLocation(SPSite site, string siteDirectoryLoc)
        {
            if (!string.IsNullOrEmpty(siteDirectoryLoc))
            {
                Guid webId;
                GetSiteDirectoryId(site, siteDirectoryLoc, out webId);
                if (webId == Guid.Empty)
                {
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Gets the site directory web id.
        /// </summary>
        /// <param name="site">The site collection containing the site directory.</param>
        /// <param name="siteDirectoryLoc">The site directory location.</param>
        /// <param name="WebId">The web id.</param>
        private static void GetSiteDirectoryId(SPSite site, string siteDirectoryLoc, out Guid WebId)
        {
            WebId = Guid.Empty;
            if (!string.IsNullOrEmpty(siteDirectoryLoc))
            {
                try
                {
                    using (SPWeb web = site.OpenWeb(SPHttpUtility.UrlPathDecode(siteDirectoryLoc, false)))
                    {
                        WebId = web.ID;
                    }
                }
                catch (FileNotFoundException)
                {
                }
                catch (ArgumentException)
                {
                }
            }
        }


        /// <summary>
        /// Writes to the web property bag.
        /// </summary>
        /// <param name="site">The site containing the site directory.</param>
        /// <param name="siteDirectoryLoc">The site directory location.</param>
        /// <param name="enforceNewListing">if set to <c>true</c> [enforce new listing].</param>
        /// <param name="entryRequirements">The entry requirements.</param>
        /// <param name="allowSiteCollectionCreation">if set to <c>true</c> [allow site collection creation].</param>
        private static void WriteToWebPropertyBag(SPSite site, string siteDirectoryLoc, bool enforceNewListing, SiteDirectoryEntryRequirements entryRequirements, bool allowSiteCollectionCreation)
        {
            using (SPWeb rootWeb = site.RootWeb)
            {
                // Set the SiteID corresponding to the site housing the site directory.
                if (rootWeb.AllProperties.ContainsKey("DefaultSiteDirectorySiteId"))
                {
                    rootWeb.AllProperties["DefaultSiteDirectorySiteId"] =
                        site.ID.ToString("D", CultureInfo.InvariantCulture);
                }
                else
                {
                    rootWeb.AllProperties.Add("DefaultSiteDirectorySiteId",
                                              site.ID.ToString("D", CultureInfo.InvariantCulture));
                }

                // Set the WebID corresponding to the site directory.
                Guid webId;
                GetSiteDirectoryId(site, siteDirectoryLoc, out webId);

                if (rootWeb.AllProperties.ContainsKey("DefaultSiteDirectoryWebId"))
                {
                    rootWeb.AllProperties["DefaultSiteDirectoryWebId"] =
                        webId.ToString("D", CultureInfo.InvariantCulture);
                }
                else
                {
                    rootWeb.AllProperties.Add("DefaultSiteDirectoryWebId",
                                              webId.ToString("D", CultureInfo.InvariantCulture));
                }

                // Remove the default location if specified.
                if (rootWeb.AllProperties.ContainsKey("DefaultSiteDirectoryLocation"))
                {
                    rootWeb.AllProperties.Remove("DefaultSiteDirectoryLocation");
                }

                // Set the enforce new listings option.
                if (rootWeb.AllProperties.ContainsKey(LocalSiteDirectoryConstants.EnforceNewListingForSites.ToString()))
                {
                    rootWeb.AllProperties[LocalSiteDirectoryConstants.EnforceNewListingForSites.ToString()] =
                        enforceNewListing.ToString();
                }
                else
                {
                    rootWeb.AllProperties.Add(LocalSiteDirectoryConstants.EnforceNewListingForSites.ToString(),
                                              enforceNewListing.ToString());
                }

                // Set the allow site collection creation option if self service site creation is enabled.
                if (site.WebApplication.SelfServiceSiteCreationEnabled)
                {
                    if (rootWeb.AllProperties.ContainsKey(
                            LocalSiteDirectoryConstants.AllowSiteCollectionCreation.ToString()))
                    {
                        rootWeb.AllProperties[LocalSiteDirectoryConstants.AllowSiteCollectionCreation.ToString()] =
                            allowSiteCollectionCreation.ToString();
                    }
                    else
                    {
                        rootWeb.AllProperties.Add(LocalSiteDirectoryConstants.AllowSiteCollectionCreation.ToString(),
                                                  allowSiteCollectionCreation.ToString());
                    }
                }
                else if (allowSiteCollectionCreation)
                {
                    throw new SPException("You must enable self service site creation via the central adminisration tool before you can allow site collection creation.");
                }

                if (enforceNewListing)
                {
                    // If we're enforcing new listings then set the directory entry requirements.
                    if (rootWeb.AllProperties.ContainsKey(LocalSiteDirectoryConstants.SiteDirectoryEntryRequirements.ToString()))
                    {
                        rootWeb.AllProperties[LocalSiteDirectoryConstants.SiteDirectoryEntryRequirements.ToString()] =
                            entryRequirements.ToString();
                    }
                    else
                    {
                        rootWeb.AllProperties.Add(
                            LocalSiteDirectoryConstants.SiteDirectoryEntryRequirements.ToString(),
                            entryRequirements.ToString());
                    }
                }

                // Update the web with the specified properties.
                rootWeb.Update();
            }
        }
    }
}
