using System.Collections.Specialized;
using System.Globalization;
using System.IO;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Portal.Administration;
using Microsoft.SharePoint.Portal.ServerAdmin;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteDirectory
{
    public class SetMasterSiteDirectory : SPOperation
    {
        private PortalService m_portalService;

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SetMasterSiteDirectory"/> class.
        /// </summary>
        public SetMasterSiteDirectory()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site directory url."));
            parameters.Add(new SPParam("enforcelistinginsitedir", "enforce", false, null, null));
            parameters.Add(new SPParam("sitedirentryrequirement", "entryreq", false, null, new SPIntRangeValidator(0, 2), "Please specify what information is required to be filled in by new site creators (0=none, 1=at least one category, 2=all categories)."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the master site directory.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <site directory url>");
            sb.Append("\r\n\t[-enforcelistinginsitedir]");
            sb.Append("\r\n\t[-sitedirentryrequirement <0=none, 1=at least one category, 2=all categories>]");

            Init(parameters, sb.ToString());
        }

        #endregion


        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            bool enforceNewListing = Params["enforcelistinginsitedir"].UserTypedIn;
            SiteDirectoryEntryRequirements entryRequirements = SiteDirectoryEntryRequirements.SiteDirectoryCatsOptional;

            if (enforceNewListing)
            {
    			string sitedirentryrequirement = Params["sitedirentryrequirement"].Value;
                if (sitedirentryrequirement == "1")
                    entryRequirements = SiteDirectoryEntryRequirements.OneSiteDirectoryCatMandatory;
                else if (sitedirentryrequirement == "2")
                    entryRequirements = SiteDirectoryEntryRequirements.AllSiteDirectoryCatMandatory;
                else if (sitedirentryrequirement == "0")
                    entryRequirements = SiteDirectoryEntryRequirements.SiteDirectoryCatsOptional;
            }

			string url = Params["url"].Value;

            m_portalService = SPFarm.Local.Services.GetValue<PortalService>(string.Empty);

            if (m_portalService == null)
            {
                m_portalService = new PortalService(string.Empty, SPFarm.Local);
            }

            WriteToPortalService(url, enforceNewListing, entryRequirements);

            return 1;
        }

        #endregion

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["enforcelistinginsitedir"].UserTypedIn)
            {
                Params["sitedirentryrequirement"].IsRequired = true;
                Params["sitedirentryrequirement"].Enabled = true;
            }
            base.Validate(keyValues);
        }

        #region Worker Methods

        /// <summary>
        /// Writes the master site directory location.
        /// </summary>
        /// <param name="masterSiteDirLoc">The master site dir loc.</param>
        private void WriteMasterSiteDirectoryLocation(string masterSiteDirLoc)
        {
            if (!string.IsNullOrEmpty(masterSiteDirLoc))
            {
                try
                {
                    using (SPSite site = new SPSite(masterSiteDirLoc))
                    {
                        using (SPWeb web = site.OpenWeb())
                        {
                            m_portalService.MasterSiteDirectorySiteId = site.ID.ToString("D", CultureInfo.InvariantCulture);
                            m_portalService.MasterSiteDirectoryWebId = web.ID.ToString("D", CultureInfo.InvariantCulture);
                        }
                    }
                }
                catch (FileNotFoundException)
                {
                }
            }
            else
            {
                m_portalService.MasterSiteDirectorySiteId = string.Empty;
                m_portalService.MasterSiteDirectoryWebId = string.Empty;
            }
        }

        /// <summary>
        /// Writes to portal service.
        /// </summary>
        /// <param name="url">The URL.</param>
        /// <param name="enforceNewListing">if set to <c>true</c> [enforce new listing].</param>
        /// <param name="entryRequirements">The entry requirements.</param>
        private void WriteToPortalService(string url, bool enforceNewListing, SiteDirectoryEntryRequirements entryRequirements)
        {
            WriteMasterSiteDirectoryLocation(url);

            m_portalService.EnforceNewListingForSites = enforceNewListing;

            if (!enforceNewListing)
            {
                m_portalService.SiteDirectoryEntryRequirements = SiteDirectoryEntryRequirements.SiteDirectoryCatsOptional.ToString();
            }
            else
            {
                m_portalService.SiteDirectoryEntryRequirements = entryRequirements.ToString();
            }
            m_portalService.Update();
        }

        #endregion
    }

}
