using System;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint.Administration;

namespace Edfinancial.SharePoint.STSADM.Commands.TimerJob
{
    public class EnumTimerJobDefinitions : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EnumTimerJobDefinitions"/> class.
        /// </summary>
        public EnumTimerJobDefinitions()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("scope", "s", false, "Farm", new SPEnumValidator(typeof(JobSelectionMode)), "Please specify the scope: Farm | Service | WebApplication"));
            parameters.Add(new SPParam("serviceid", "sid", false, null, new SPNonEmptyValidator(), "Please specify the ID of the service."));
            parameters.Add(new SPParam("webappurl", "webapp", false, null, new SPNonEmptyValidator(), "Please specify the web application URL."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nDisplays Information about the all Timer Jobs for the Farm, Service, or Web Application.\r\n\r\nParemeters:");
            sb.Append("\r\n\t[-scope <Farm (default) | Service | WebApplication>]");
            sb.Append("\r\n\t[-serviceid <ID of the service if Service is specified for scope>]");
            sb.Append("\r\n\t[-webappurl <URL of the web application if WebApplication is specified for scope>]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            JobSelectionMode scope = JobSelectionMode.Farm;

            if (Params["scope"].UserTypedIn)
                scope = (JobSelectionMode)Enum.Parse(typeof(JobSelectionMode), Params["scope"].Value, true);

            SPService service = null;
            SPWebApplication webApp = null;
            if (scope == JobSelectionMode.Service)
            {
                string serviceId = Params["serviceid"].Value;
                service = SPFarm.Local.Services[serviceId];
            }
            else if (scope == JobSelectionMode.WebApplication)
            {
                string webAppUrl = Params["webappurl"].Value;
                webApp = SPWebApplication.Lookup(new Uri(webAppUrl));
            }

            XmlDocument xmlDoc = BuildXmlDoc(scope, service, webApp);
            StringBuilder sb = new StringBuilder();
           
            XmlTextWriter xmlWriter = new XmlTextWriter(new StringWriter(sb));
            xmlWriter.Formatting = Formatting.Indented;
            xmlDoc.WriteContentTo(xmlWriter);
            xmlWriter.Flush();
            output += sb.ToString();

            return 1;
        }

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            JobSelectionMode scope = JobSelectionMode.Farm;

            if (Params["scope"].UserTypedIn)
                scope = (JobSelectionMode)Enum.Parse(typeof(JobSelectionMode), Params["scope"].Value, true);

            if (scope == JobSelectionMode.Service)
                Params["serviceid"].IsRequired = true;

            if (scope == JobSelectionMode.WebApplication)
                Params["webappurl"].IsRequired = true;

            base.Validate(keyValues);
        }

        #endregion

        public enum JobSelectionMode
        {
            Farm,
            Service,
            WebApplication
        }


        /// <summary>
        /// Adds the service.
        /// </summary>
        /// <param name="service">The service.</param>
        /// <param name="xmlDoc">The XML doc.</param>
        private static void AddService(SPService service, XmlDocument xmlDoc)
        {
            SPWebService service2;

            foreach (SPJobDefinition job in service.JobDefinitions)
            {
                Helper.GetJobInformation(job, xmlDoc);
            }
            service2 = service as SPWebService;
            if (service2 == null)
            {
                return;
            }
            foreach (SPWebApplication application in service2.WebApplications)
            {
                AddWebApplication(application, xmlDoc);
            }
        }

        /// <summary>
        /// Adds the web application.
        /// </summary>
        /// <param name="webapp">The webapp.</param>
        /// <param name="xmlDoc">The XML doc.</param>
        private static void AddWebApplication(SPWebApplication webapp, XmlDocument xmlDoc)
        {
            foreach (SPJobDefinition job in webapp.JobDefinitions)
            {
                Helper.GetJobInformation(job, xmlDoc);
            }
        }


        /// <summary>
        /// Builds the XML doc.
        /// </summary>
        /// <param name="mode">The mode.</param>
        /// <param name="service">The service.</param>
        /// <param name="webApp">The web app.</param>
        public static XmlDocument BuildXmlDoc(JobSelectionMode mode, SPService service, SPWebApplication webApp)
        {
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.AppendChild(xmlDoc.CreateElement("Jobs"));

            switch (mode)
            {
                case JobSelectionMode.Farm:
                    foreach (SPService temp in SPFarm.Local.Services)
                    {
                        AddService(temp, xmlDoc);
                    }
                    break;
                case JobSelectionMode.Service:
                    if (service == null)
                    {
                        break;
                    }
                    AddService(service, xmlDoc);
                    break;
                case JobSelectionMode.WebApplication:
                    if (webApp != null)
                    {
                        AddWebApplication(webApp, xmlDoc);
                    }
                    break;
            }
            return xmlDoc;
        }

    }
}
