using System;
using System.Collections.Generic;
using System.Xml;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace Edfinancial.SharePoint.STSADM.Commands.TimerJob
{
    internal class Helper
    {
        /// <summary>
        /// Loops through all job definitions in the webapplication of the given site and retuns the job definition
        /// which name is equal to the given name.
        /// </summary>
        /// <param name="site">site collection object</param>
        /// <param name="jobname">name of the job definition to search for</param>
        /// <returns></returns>
        internal static List<SPJobDefinition> GetJobDefinition(SPSite site, string jobname)
        {
            List<SPJobDefinition> jobs = new List<SPJobDefinition>();
            //loop through the job definitions
            foreach (SPJobDefinition job in site.WebApplication.JobDefinitions)
            {
                if (job.Name.Equals(jobname))
                {
                    jobs.Add(job);
                }
            }

            return jobs;
        }

        /// <summary>
        /// Gets the job definition.
        /// </summary>
        /// <param name="jobName">The job name.</param>
        /// <param name="databaseId">The database id.</param>
        /// <returns></returns>
        internal static List<SPJobDefinition> GetJobDefinition(string jobName, out Guid databaseId)
        {
            databaseId = SPServer.Local.Id;
            List<SPJobDefinition> jobs = new List<SPJobDefinition>();

            // Loop through all the services on the farm
            foreach (SPService service in SPFarm.Local.Services)
            {
                foreach (SPJobDefinition job in service.JobDefinitions)
                {
                    if (job.Name.Equals(jobName))
                    {
                        jobs.Add(job);
                    }
                }
                // If the service is a web service then loop through all the web apps otherwise move on to the next service.
                SPWebService service2 = service as SPWebService;
                if (service2 == null)
                {
                    continue;
                }
                foreach (SPWebApplication application in service2.WebApplications)
                {
                    foreach (SPJobDefinition job in application.JobDefinitions)
                    {
                        if (job.Name.Equals(jobName))
                        {
                            jobs.Add(job);
                        }
                    }
                }
            }

            return jobs;
        }

        /// <summary>
        /// Returns all information about the given job definition.
        /// </summary>
        /// <param name="job">Job definition</param>
        /// <returns></returns>
        internal static string GetJobInformation(SPJobDefinition job)
        {
            string returnValue = string.Empty;
            returnValue += "\r\nDisplayname = " + job.DisplayName; //DisplayName
            returnValue += "\r\nId = " + job.Id; //Id
            returnValue += "\r\nIsDisabled = " + job.IsDisabled; //IsDisabled
            returnValue += "\r\nLast Run Time = " + job.LastRunTime; //LastRunTime
            returnValue += "\r\nLockType = " + job.LockType; //LockType
            returnValue += "\r\nName = " + job.Name; //Name
            returnValue += "\r\nRetry = " + job.Retry; //Retry
            returnValue += "\r\nStatus = " + job.Status; //Status
            returnValue += "\r\nTitle = " + job.Title; //Title
            returnValue += "\r\nTypeName = " + job.TypeName; //TypeName
            returnValue += "\r\nVersion = " + job.Version; //Version
            returnValue += "\r\nSchedule Typename = " + job.Schedule.GetType().FullName; //Schedule
            returnValue += "\r\nNext Occurrence based on now = " + job.Schedule.NextOccurrence(DateTime.Now);
                //Schedule Next Occurance
            if (job.Schedule is SPDailySchedule)
            {
                SPDailySchedule dailySchedule = job.Schedule as SPDailySchedule;
                if (dailySchedule != null)
                {
                    returnValue += "\r\nBegin Hour = " + dailySchedule.BeginHour;
                    returnValue += "\r\nBegin Minute = " + dailySchedule.BeginMinute;
                    returnValue += "\r\nBegin Second = " + dailySchedule.BeginSecond;
                    returnValue += "\r\nEnd Hour = " + dailySchedule.EndHour;
                    returnValue += "\r\nEnd Minute = " + dailySchedule.EndMinute;
                    returnValue += "\r\nEnd Second = " + dailySchedule.EndSecond;
                }
            }
            else if (job.Schedule is SPMinuteSchedule)
            {
                SPMinuteSchedule minuteSchedule = job.Schedule as SPMinuteSchedule;
                if (minuteSchedule != null)
                {
                    returnValue += "\r\nBegin Second = " + minuteSchedule.BeginSecond;
                    returnValue += "\r\nEnd Second = " + minuteSchedule.EndSecond;
                    returnValue += "\r\nInterval = " + minuteSchedule.Interval;
                }
            }
            else if (job.Schedule is SPMonthlySchedule)
            {
                SPMonthlySchedule monthlySchedule = job.Schedule as SPMonthlySchedule;
                if (monthlySchedule != null)
                {
                    returnValue += "\r\nBegin Day = " + monthlySchedule.BeginDay;
                    returnValue += "\r\nBegin Hour = " + monthlySchedule.BeginHour;
                    returnValue += "\r\nBegin Minute = " + monthlySchedule.BeginMinute;
                    returnValue += "\r\nBegin Second = " + monthlySchedule.BeginSecond;
                    returnValue += "\r\nEnd Day = " + monthlySchedule.EndDay;
                    returnValue += "\r\nEnd Hour = " + monthlySchedule.EndHour;
                    returnValue += "\r\nEnd Minute = " + monthlySchedule.EndMinute;
                    returnValue += "\r\nEnd Second = " + monthlySchedule.EndSecond;
                }
            }
            else if (job.Schedule is SPOneTimeSchedule)
            {
                SPOneTimeSchedule oneTimeSchedule = job.Schedule as SPOneTimeSchedule;
                if (oneTimeSchedule != null)
                {
                    returnValue += "\r\nTime = " + oneTimeSchedule.Time;
                }
            }
            return returnValue;
        }


        /// <summary>
        /// Returns all information about the given job definition.
        /// </summary>
        /// <param name="job">Job definition</param>
        /// <param name="xmlDoc">The XML doc.</param>
        internal static void GetJobInformation(SPJobDefinition job, XmlDocument xmlDoc)
        {
            XmlElement jobNode = xmlDoc.CreateElement("Job");
            xmlDoc.DocumentElement.AppendChild(jobNode);

            jobNode.SetAttribute("Id", job.Id.ToString());
            jobNode.SetAttribute("Title", job.Title);
            jobNode.SetAttribute("Name", job.Name);

            XmlElement property;
            XmlElement serviceNode = xmlDoc.CreateElement("Service");
            jobNode.AppendChild(serviceNode);

            if (job.Service != null)
            {
                property = xmlDoc.CreateElement("Name");
                property.InnerText = job.Service.Name;
                serviceNode.AppendChild(property);

                property = xmlDoc.CreateElement("Id");
                property.InnerText = job.Service.Id.ToString();
                serviceNode.AppendChild(property);

                property = xmlDoc.CreateElement("DisplayName");
                property.InnerText = job.Service.DisplayName;
                serviceNode.AppendChild(property);

                property = xmlDoc.CreateElement("TypeName");
                property.InnerText = job.Service.TypeName;
                serviceNode.AppendChild(property);
            }

            XmlElement webNode = xmlDoc.CreateElement("WebApplication");
            jobNode.AppendChild(webNode);

            if (job.WebApplication != null)
            {
                property = xmlDoc.CreateElement("Name");
                property.InnerText = job.WebApplication.Name;
                webNode.AppendChild(property);

                property = xmlDoc.CreateElement("Id");
                property.InnerText = job.WebApplication.Id.ToString();
                webNode.AppendChild(property);
            }

            property = xmlDoc.CreateElement("DisplayName");
            property.InnerText = job.DisplayName;
            jobNode.AppendChild(property);

            property = xmlDoc.CreateElement("IsDisabled");
            property.InnerText = job.IsDisabled.ToString();
            jobNode.AppendChild(property);

            property = xmlDoc.CreateElement("LastRunTime");
            property.InnerText = job.LastRunTime.ToString();
            jobNode.AppendChild(property);

            property = xmlDoc.CreateElement("LockType");
            property.InnerText = job.LockType.ToString();
            jobNode.AppendChild(property);

            property = xmlDoc.CreateElement("Retry");
            property.InnerText = job.Retry.ToString();
            jobNode.AppendChild(property);

            property = xmlDoc.CreateElement("Status");
            property.InnerText = job.Status.ToString();
            jobNode.AppendChild(property);

            property = xmlDoc.CreateElement("TypeName");
            property.InnerText = job.TypeName;
            jobNode.AppendChild(property);

            property = xmlDoc.CreateElement("Version");
            property.InnerText = job.Version.ToString();
            jobNode.AppendChild(property);

            property = xmlDoc.CreateElement("ScheduleTypeName");
            property.InnerText = job.Schedule.GetType().FullName;
            jobNode.AppendChild(property);

            property = xmlDoc.CreateElement("NextOccurrenceBasedOnNow");
            try
            {
                property.InnerText = job.Schedule.NextOccurrence(DateTime.Now).ToString();
            }
            catch (NotImplementedException)
            {
                property.InnerText = "";
            }
            jobNode.AppendChild(property);

            XmlElement scheduleNode = xmlDoc.CreateElement("Schedule");
            jobNode.AppendChild(scheduleNode);

            if (job.Schedule is SPDailySchedule)
            {
                SPDailySchedule dailySchedule = job.Schedule as SPDailySchedule;
                if (dailySchedule != null)
                {
                    property = xmlDoc.CreateElement("BeginHour");
                    property.InnerText = dailySchedule.BeginHour.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("BeginMinute");
                    property.InnerText = dailySchedule.BeginMinute.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("BeginSecond");
                    property.InnerText = dailySchedule.BeginSecond.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("EndHour");
                    property.InnerText = dailySchedule.EndHour.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("EndMinute");
                    property.InnerText = dailySchedule.EndMinute.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("EndSecond");
                    property.InnerText = dailySchedule.EndSecond.ToString();
                    scheduleNode.AppendChild(property);
                }
            }
            else if (job.Schedule is SPMinuteSchedule)
            {
                SPMinuteSchedule minuteSchedule = job.Schedule as SPMinuteSchedule;
                if (minuteSchedule != null)
                {
                    property = xmlDoc.CreateElement("BeginSecond");
                    property.InnerText = minuteSchedule.BeginSecond.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("EndSecond");
                    property.InnerText = minuteSchedule.EndSecond.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("Interval");
                    property.InnerText = minuteSchedule.Interval.ToString();
                    scheduleNode.AppendChild(property);
                }
            }
            else if (job.Schedule is SPMonthlySchedule)
            {
                SPMonthlySchedule monthlySchedule = job.Schedule as SPMonthlySchedule;
                if (monthlySchedule != null)
                {
                    property = xmlDoc.CreateElement("BeginDay");
                    property.InnerText = monthlySchedule.BeginDay.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("BeginHour");
                    property.InnerText = monthlySchedule.BeginHour.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("BeginMinute");
                    property.InnerText = monthlySchedule.BeginMinute.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("BeginSecond");
                    property.InnerText = monthlySchedule.BeginSecond.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("EndDay");
                    property.InnerText = monthlySchedule.EndDay.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("EndHour");
                    property.InnerText = monthlySchedule.EndHour.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("EndMinute");
                    property.InnerText = monthlySchedule.EndMinute.ToString();
                    scheduleNode.AppendChild(property);

                    property = xmlDoc.CreateElement("EndSecond");
                    property.InnerText = monthlySchedule.EndSecond.ToString();
                    scheduleNode.AppendChild(property);
                }
            }
            else if (job.Schedule is SPOneTimeSchedule)
            {
                SPOneTimeSchedule oneTimeSchedule = job.Schedule as SPOneTimeSchedule;
                if (oneTimeSchedule != null)
                {
                    property = xmlDoc.CreateElement("Time");
                    property.InnerText = oneTimeSchedule.Time.ToString();
                    scheduleNode.AppendChild(property);
                }
            }
        }
    }
}