using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.StsAdmin;

namespace Edfinancial.SharePoint.STSADM.Commands.TimerJob
{

    /// <summary>
    /// Sets the Schedule of a specific Timer Job Definition.
    /// </summary>
    public class SetJobSchedule : ISPStsadmCommand
    {
        private enum ScheduleType
        {
            daily,
            minute,
            monthly,
            onetime,
            notspecified
        }

        public string GetHelpMessage(string command)
        {
            StringBuilder builder = new StringBuilder();

            builder.Append("\r\n\r\nSets the specified schedule to the specified job\r\n\r\nParameters:");
            builder.Append("\r\n\t-job <job name>\r\n\t-url <url of site collection>\r\n\t-scheduletype <daily|minute|monthly|onetime>\r\n");
            builder.Append("\r\nFor a daily schedule:\r\n\t-beginhour <hour at which the schedule starts>\r\n\t-beginminute <minute at which the schedule starts>\r\n\t-beginsecond <second at which the schedule starts>\r\n\t-endhour <hour at which the schedule ends>\r\n\t-endminute <minute at which the schedule ends>\r\n\t-endsecond <second at which the schedule ends>\r\n");
            builder.Append("\r\nFor a minute schedule:\r\n\t-beginsecond <second at which the schedule starts>\r\n\t-endsecond <second at which the schedule ends>\r\n\t-interval <interval in minutes>\r\n");
            builder.Append("\r\nFor a monthly schedule:\r\n\t-beginday <day at which the schedule starts>\r\n\t-beginhour <hour at which the schedule starts>\r\n\t-beginminute <minute at which the schedule starts>\r\n\t-beginsecond <second at which the schedule starts>\r\n\t-endday <day at which the schedule ends>\r\n\t-endhour <hour at which the schedule ends>\r\n\t-endminute <minute at which the schedule ends>\r\n\t-endsecond <second at which the schedule ends>\r\n");
            builder.Append("\r\nFor a one time schedule\r\n\t-time <a date in the format: YYYY-MM-DD HH:MM:SS>\r\n");
            return builder.ToString();
        }

        public int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            ScheduleType scheduleType;

            #region Check Arguments
            if (!keyValues.ContainsKey("job") || keyValues["job"].Length == 0)
            {
                output = "Please specify the job name.";
                output += GetHelpMessage(command);
                return 0;
            }

            if (!keyValues.ContainsKey("url") || keyValues["url"].Length == 0)
            {
                output = "Please specify a url of a site collection.";
                output += GetHelpMessage(command);
                return 0;
            }

            if (!keyValues.ContainsKey("scheduletype") || keyValues["scheduletype"].Length == 0)
            {
                output = "Please specify a schedule type.";
                output += GetHelpMessage(command);
                return 0;
            }
            else
            {
                try
                {
                    scheduleType = (ScheduleType) Enum.Parse(typeof(ScheduleType), keyValues["scheduletype"], true);
                }
                catch (Exception)
                {
                    output = "The scheduletype was not one of the known values.";
                    output += GetHelpMessage(command);
                    return 0;
                }
            }

            DateTime givenTime;
            int beginHour = -1;
            int beginMinute = -1;
            int beginSecond = -1;
            int endHour = -1;
            int endMinute = -1;
            int endSecond = -1;
            int interval = -1;
            int beginDay = -1;
            int endDay = -1;

            switch (scheduleType)
            {
                case ScheduleType.daily:
                    if (!keyValues.ContainsKey("beginhour") || keyValues["beginhour"].Length == 0)
                    {
                        output = "Please specify a value for begin hour.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["beginhour"], out beginHour);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for begin hour.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("beginminute") || keyValues["beginminute"].Length == 0)
                    {
                        output = "Please specify a value for begin minute.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["beginminute"], out beginMinute);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for begin minute.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("beginsecond") || keyValues["beginsecond"].Length == 0)
                    {
                        output = "Please specify a value for begin second.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["beginsecond"], out beginSecond);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for begin second.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("endhour") || keyValues["endhour"].Length == 0)
                    {
                        output = "Please specify a value for end hour.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["endhour"], out endHour);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for end hour.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("endminute") || keyValues["endminute"].Length == 0)
                    {
                        output = "Please specify a value for end minute.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["endminute"], out endMinute);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for end minute.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("endsecond") || keyValues["endsecond"].Length == 0)
                    {
                        output = "Please specify a value for end second.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["endsecond"], out endSecond);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for end second.";
                            return 0;
                        }
                    }
                    break;
                case ScheduleType.minute:
                    if (!keyValues.ContainsKey("beginsecond") || keyValues["beginsecond"].Length == 0)
                    {
                        output = "Please specify a value for begin second.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["beginsecond"], out beginSecond);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for begin second.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("endsecond") || keyValues["endsecond"].Length == 0)
                    {
                        output = "Please specify a value for end second.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["endsecond"], out endSecond);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for end second.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("interval") || keyValues["interval"].Length == 0)
                    {
                        output = "Please specify a value for interval.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["interval"], out interval);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for interval.";
                            return 0;
                        }
                    }
                    break;
                case ScheduleType.monthly:
                    if (!keyValues.ContainsKey("beginday") || keyValues["beginday"].Length == 0)
                    {
                        output = "Please specify a value for begin day.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["beginday"], out beginDay);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for begin day.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("beginhour") || keyValues["beginhour"].Length == 0)
                    {
                        output = "Please specify a value for begin hour.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["beginhour"], out beginHour);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for begin hour.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("beginminute") || keyValues["beginminute"].Length == 0)
                    {
                        output = "Please specify a value for begin minute.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["beginminute"], out beginMinute);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for begin minute.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("beginsecond") || keyValues["beginsecond"].Length == 0)
                    {
                        output = "Please specify a value for begin second.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["beginsecond"], out beginSecond);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for begin second.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("endday") || keyValues["endday"].Length == 0)
                    {
                        output = "Please specify a value for end day.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["endday"], out endDay);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for end day.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("endhour") || keyValues["endhour"].Length == 0)
                    {
                        output = "Please specify a value for end hour.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["endhour"], out endHour);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for end hour.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("endminute") || keyValues["endminute"].Length == 0)
                    {
                        output = "Please specify a value for end minute.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["endminute"], out endMinute);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for end minute.";
                            return 0;
                        }
                    }

                    if (!keyValues.ContainsKey("endsecond") || keyValues["endsecond"].Length == 0)
                    {
                        output = "Please specify a value for end second.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool valueCanBeParsed = int.TryParse(keyValues["endsecond"], out endSecond);
                        if (!valueCanBeParsed)
                        {
                            output = "Please specify a valid value for end second.";
                            return 0;
                        }
                    }
                    break;
                case ScheduleType.onetime:
                    if (!keyValues.ContainsKey("time") || keyValues["time"].Length == 0)
                    {
                        output = "Please specify a value for time.";
                        output += GetHelpMessage(command);
                        return 0;
                    }
                    else
                    {
                        bool givenTimeCanBeParsed = DateTime.TryParse(keyValues["time"], out givenTime);
                        if (!givenTimeCanBeParsed)
                        {
                            output = "Please specify a valid value for time.";
                            return 0;
                        }
                    }
                    break;
            }
            #endregion

            string specifiedJobName = keyValues["job"];
            string siteCollectionUrl = keyValues["url"];

            using (SPSite site = new SPSite(siteCollectionUrl))
            {
                List<SPJobDefinition> jobs = Helper.GetJobDefinition(site, specifiedJobName);

                if (jobs != null && jobs.Count > 1)
                {
                    throw new ApplicationException("More than one job found matching the specified name " +
                                                   specifiedJobName);
                }

                if (jobs != null && jobs.Count > 0)
                {
                    SPJobDefinition job = jobs[0];
                    switch (scheduleType)
                    {
                        case ScheduleType.daily:
                            SPDailySchedule dailySchedule = new SPDailySchedule();

                            dailySchedule.BeginHour = beginHour;
                            dailySchedule.BeginMinute = beginMinute;
                            dailySchedule.BeginSecond = beginSecond;
                            dailySchedule.EndHour = endHour;
                            dailySchedule.EndMinute = endMinute;
                            dailySchedule.EndSecond = endSecond;

                            job.Schedule = dailySchedule;
                            break;
                        case ScheduleType.minute:
                            SPMinuteSchedule minuteSchedule = new SPMinuteSchedule();

                            minuteSchedule.BeginSecond = beginSecond;
                            minuteSchedule.EndSecond = endSecond;
                            minuteSchedule.Interval = interval;

                            job.Schedule = minuteSchedule;
                            break;
                        case ScheduleType.monthly:
                            SPMonthlySchedule monthlySchedule = new SPMonthlySchedule();

                            monthlySchedule.BeginDay = beginDay;
                            monthlySchedule.BeginHour = beginHour;
                            monthlySchedule.BeginMinute = beginMinute;
                            monthlySchedule.BeginSecond = beginSecond;
                            monthlySchedule.EndDay = endDay;
                            monthlySchedule.EndHour = endHour;
                            monthlySchedule.EndMinute = endMinute;
                            monthlySchedule.EndSecond = endSecond;

                            job.Schedule = monthlySchedule;
                            break;
                        case ScheduleType.onetime:
                            SPOneTimeSchedule oneTimeSchedule = new SPOneTimeSchedule();

                            DateTime time = DateTime.Parse(keyValues["time"]);

                            oneTimeSchedule.Time = time;

                            job.Schedule = oneTimeSchedule;
                            break;
                    }

                    try
                    {
                        job.Update();
                    }
                    catch (Exception ex)
                    {
                        output += "\r\nAn Exception occured while updating the timer job.\r\n"+ex;
                    }
                }
                else
                {
                    output += "\r\nCould not find a job with name [" + specifiedJobName + "] for that site collection";
                }
            }

            return 1;
        }
    }
}
