using System;
using System.Collections.Specialized;
using System.Globalization;
using System.Reflection;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace Edfinancial.SharePoint.STSADM.Commands.TimerJob
{
    public class SetUserProfileImportSchedule : SPOperation
    {
        private enum OccuranceType
        {
            daily,
            weekly,
            monthly
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SetUserProfileImportSchedule"/> class.
        /// </summary>
        public SetUserProfileImportSchedule()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("type", "t", true, null, new SPRegexValidator("^incremental$|^full$")));
            parameters.Add(new SPParam("occurance", "o", true, null, new SPRegexValidator("^daily$|^weekly$|^monthly$")));
            parameters.Add(new SPParam("hour", "hour", true, null, new SPIntRangeValidator(0, 23)));
            parameters.Add(new SPParam("day", "day", false, null, new SPIntRangeValidator(1, 31)));
            string regex = "^" + string.Join("$|^", Enum.GetNames(typeof (DayOfWeek))) + "$";
            parameters.Add(new SPParam("dayofweek", "dayofweek", false, null, new SPRegexValidator(regex.ToLowerInvariant() + "|" + regex)));
           
            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the profile import schedule.\r\n\r\nParameters:");
            sb.Append("\r\n\t-sspname <SSP name>");
            sb.Append("\r\n\t-type <incremental|full>");
            sb.Append("\r\n\t-occurance <daily|weekly|monthly>");
            sb.Append("\r\n\t-hour <hour to run (0-23)>");
            sb.Append("\r\n\t[-day <the day to run if monthly is specified>]");
            sb.AppendFormat("\r\n\t[-dayofweek <the day of week to run if weekly is specified ({0})>]", string.Join("|", Enum.GetNames(typeof(DayOfWeek))).ToLowerInvariant());
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            #region Check Arguments

            OccuranceType occurance = (OccuranceType)Enum.Parse(typeof(OccuranceType), Params["occurance"].Value, true);
            if (occurance == OccuranceType.monthly && !Params["day"].UserTypedIn)
            {
                output = "Please specify the day to run the import.";
                output += GetHelpMessage(command);
                return 0;
            }
            if (occurance == OccuranceType.weekly && !Params["dayofweek"].UserTypedIn)
            {
                output = "Please specify the day of week to run the import.";
                output += GetHelpMessage(command);
                return 0;
            }

            #endregion

            string type = Params["type"].Value;
            string day = Params["day"].Value;
            string dayofweek = Params["dayofweek"].Value;
            string sspname = Params["sspname"].Value;
            int hour = int.Parse(Params["hour"].Value);


            ServerContext current = ServerContext.GetContext(sspname);

            // What follows is a whole lot of reflection which is required in order to get the SPScheduledJob object.
            // Problem is that the only way to get the correct instance of this object is to use several internal
            // classes, methods, and properties - why on earth these were not made public is absolutely beyond me!

            // The bulk of the reflection is recreating the following which was taken from 
            // Microsoft.SharePoint.Portal.UserProfiles.AdminUI.SetImport._InitSchedulerControl().
            // Once we have the job objects we can start setting properties.
            /*
            private void _InitSchedulerControl()
            {
                ServerContext current = ServerContext.Current;
                UserProfileApplication userProfileApplication = current.UserProfileApplication;
                try
                {
                    using (SecurityContext context2 = PortalApplication.BeginSecurityContext())
                    {
                        JobSchedulerSharedApplicationCollection applications = new JobSchedulerSharedApplicationCollection(SPFarm.Local.Services.GetValue<JobSchedulerService>(string.Empty));
                        JobSchedulerSharedApplication sharedApplication = (JobSchedulerSharedApplication) applications[current.SharedResourceProvider];
                        ScheduledJobCollection jobs = new ScheduledJobCollection(sharedApplication);
                        this.FullImportScheduler.Job = jobs[userProfileApplication.ProfileFullImportJobId];
                        this.IncrementalScheduler.Job = jobs[userProfileApplication.ProfileIncrementalImportJobId];
                    }
                }
                catch (Exception)
                {
                    throw;
                }
            }
            */

            // UserProfileApplication userProfileApplication = current.UserProfileApplication;
            PropertyInfo userProfileApplicationProp = current.GetType().GetProperty("UserProfileApplication",
                                                     BindingFlags.NonPublic |
                                                     BindingFlags.Instance |
                                                     BindingFlags.InvokeMethod |
                                                     BindingFlags.GetProperty);
            object userProfileApplication = userProfileApplicationProp.GetValue(current, null);

            // The SSP is locked down so we need to use reflection to get at it.
            object sharedResourceProvider = Utilities.GetSharedResourceProvider(current);

            // JobSchedulerService jobSchedulerService = SPFarm.Local.Services.GetValue(typeof(JobSchedulerService));
            Type jobSchedulerServiceType = Type.GetType("Microsoft.Office.Server.Administration.JobSchedulerService, Microsoft.Office.Server, Version=12.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c");

            MethodInfo getValue =
                SPFarm.Local.Services.GetType().GetMethod("GetValue",
                                                          BindingFlags.NonPublic | BindingFlags.Public |
                                                          BindingFlags.Instance | BindingFlags.InvokeMethod, null, new Type[] {typeof(Type), typeof(string)}, null);

            object jobSchedulerService = getValue.Invoke(SPFarm.Local.Services,
                                                          new object[]
                                                              {
                                                                  jobSchedulerServiceType, string.Empty
                                                              });


            // JobSchedulerSharedApplicationCollection application = new JobSchedulerSharedApplicationCollection(jobSchedulerServiceType);
            Type jobSchedulerSharedApplicationCollectionType = Type.GetType("Microsoft.Office.Server.Administration.JobSchedulerSharedApplicationCollection, Microsoft.Office.Server, Version=12.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c");

            ConstructorInfo jobSchedulerSharedApplicationCollectionConstructor =
                jobSchedulerSharedApplicationCollectionType.GetConstructor(
                    BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.InvokeMethod | BindingFlags.Public,
                    null,
                    new Type[] {jobSchedulerService.GetType()}, null);
            object applications = jobSchedulerSharedApplicationCollectionConstructor.Invoke(new object[] { jobSchedulerService });

            // JobSchedulerSharedApplication jobSchedulerSharedApplication = applications[sharedResourceProvider];
            PropertyInfo itemProp = applications.GetType().GetProperty("Item",
                                                                     BindingFlags.NonPublic |
                                                                     BindingFlags.Instance |
                                                                     BindingFlags.InvokeMethod |
                                                                     BindingFlags.GetProperty |
                                                                     BindingFlags.Public);
            object jobSchedulerSharedApplication = itemProp.GetValue(applications, new object[] { sharedResourceProvider });


            //ScheduledJobCollection scheduledJobCollection = new ScheduledJobCollection(sharedApplication);
            Type scheduledJobCollectionType = Type.GetType("Microsoft.Office.Server.Administration.ScheduledJobCollection, Microsoft.Office.Server, Version=12.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c");
            ConstructorInfo scheduledJobCollectionConstructor =
                scheduledJobCollectionType.GetConstructor(
                    BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.InvokeMethod | BindingFlags.Public,
                    null,
                    new Type[] {jobSchedulerSharedApplication.GetType()}, null);
            object scheduledJobCollection = scheduledJobCollectionConstructor.Invoke(new object[] { jobSchedulerSharedApplication });


            // userProfileApplication.ProfileFullImportJobId
            PropertyInfo profileFullImportJobIdProp = userProfileApplication.GetType().GetProperty("ProfileFullImportJobId",
                                         BindingFlags.NonPublic |
                                         BindingFlags.Instance |
                                         BindingFlags.InvokeMethod |
                                         BindingFlags.GetProperty);
            Guid profileFullImportJobId = (Guid)profileFullImportJobIdProp.GetValue(userProfileApplication, null);


            // userProfileApplication.ProfileIncrementalImportJobId
            PropertyInfo profileIncrementalImportJobIdProp = userProfileApplication.GetType().GetProperty("ProfileIncrementalImportJobId",
                             BindingFlags.NonPublic |
                             BindingFlags.Instance |
                             BindingFlags.InvokeMethod |
                             BindingFlags.GetProperty);
            Guid profileIncrementalImportJobId = (Guid)profileIncrementalImportJobIdProp.GetValue(userProfileApplication, null);

            // SPScheduledJob fullImportJob = scheduledJobCollection[profileFullImportJobId];
            // SPScheduledJob incrementalImportJob = scheduledJobCollection[profileIncrementalImportJobId];
            itemProp = scheduledJobCollection.GetType().GetProperty("Item",
                                                                    BindingFlags.NonPublic |
                                                                    BindingFlags.Instance |
                                                                    BindingFlags.InvokeMethod |
                                                                    BindingFlags.GetProperty |
                                                                    BindingFlags.Public);
            object fullImportJob = itemProp.GetValue(scheduledJobCollection, new object[] {profileFullImportJobId});
            object incrementalImportJob =
                itemProp.GetValue(scheduledJobCollection, new object[] {profileIncrementalImportJobId});


            PropertyInfo scheduleProp = fullImportJob.GetType().GetProperty("Schedule",
                                                                            BindingFlags.FlattenHierarchy |
                                                                            BindingFlags.NonPublic |
                                                                            BindingFlags.Instance |
                                                                            BindingFlags.InvokeMethod |
                                                                            BindingFlags.GetProperty |
                                                                            BindingFlags.Public);

            MethodInfo update =
                fullImportJob.GetType().GetMethod("Update",
                                                  BindingFlags.NonPublic | 
                                                  BindingFlags.Public |
                                                  BindingFlags.Instance | 
                                                  BindingFlags.InvokeMethod |
                                                  BindingFlags.FlattenHierarchy, 
                                                  null,
                                                  new Type[] {typeof (bool)}, null);
            
            // Woohoo!!! We are finally at a point where we can actually set the schedule - what a pain the @$$ that was!!!
            SPSchedule schedule;

            if (occurance == OccuranceType.daily)
            {
                schedule = ScheduledJobHelper.GetScheduleDaily(hour);
            }
            else if (occurance == OccuranceType.weekly)
            {
                schedule = ScheduledJobHelper.GetScheduleWeekly((DayOfWeek)Enum.Parse(typeof(DayOfWeek), dayofweek, true), hour);
            }
            else if (occurance == OccuranceType.monthly)
            {
                schedule = ScheduledJobHelper.GetScheduleMonthly(int.Parse(day), hour);
            }
            else
                throw new Exception("Unknown occurance type.");

            if (type == "full")
            {
                // fullImportJob.Schedule = schedule;
                scheduleProp.SetValue(fullImportJob, schedule, null);

                // fullImportJob.Update(true);
                update.Invoke(fullImportJob, new object[] { true });
            }
            else
            {
                // incrementalImportJob.Schedule = schedule;
                scheduleProp.SetValue(incrementalImportJob, schedule, null);

                // incrementalImportJob.Update(true);
                update.Invoke(incrementalImportJob, new object[] { true });
            }



            return 1;
        }

        #endregion

        internal class ScheduledJobHelper
        {
            /// <summary>
            /// Gets the schedule daily.
            /// </summary>
            /// <param name="hour">The hour.</param>
            /// <returns></returns>
            internal static SPSchedule GetScheduleDaily(int hour)
            {
                DateTime time = ConvertFromLocalToUtc(GetDateTimeForSchedule(hour));
                SPDailySchedule schedule = new SPDailySchedule();
                schedule.BeginHour = time.Hour;
                schedule.EndHour = schedule.BeginHour;
                schedule.BeginMinute = 0;
                schedule.EndMinute = schedule.BeginMinute;

                return schedule;
            }

            /// <summary>
            /// Gets the schedule weekly.
            /// </summary>
            /// <param name="dayOfWeek">The day of week.</param>
            /// <param name="hour">The hour.</param>
            /// <returns></returns>
            internal static SPSchedule GetScheduleWeekly(DayOfWeek dayOfWeek, int hour)
            {
                DateTime time = ConvertFromLocalToUtc(GetDateTimeForSchedule(dayOfWeek, hour));
                SPWeeklySchedule schedule = new SPWeeklySchedule();
                schedule.BeginDayOfWeek = time.DayOfWeek;
                schedule.EndDayOfWeek = schedule.BeginDayOfWeek;
                schedule.BeginHour = time.Hour;
                schedule.EndHour = schedule.BeginHour;

                return schedule;
            }

            /// <summary>
            /// Gets the schedule monthly.
            /// </summary>
            /// <param name="dayOfMonth">The day of month.</param>
            /// <param name="hour">The hour.</param>
            /// <returns></returns>
            internal static SPSchedule GetScheduleMonthly(int dayOfMonth, int hour)
            {
                DateTime time = ConvertFromLocalToUtc(GetDateTimeForSchedule(dayOfMonth, hour));
                SPMonthlySchedule schedule = new SPMonthlySchedule();
                schedule.BeginDay = time.Day;
                schedule.EndDay = schedule.BeginDay;
                schedule.BeginHour = time.Hour;
                schedule.EndHour = schedule.BeginHour;

                return schedule;
            }

            /// <summary>
            /// Converts from local to UTC.
            /// </summary>
            /// <param name="dateTime">The date time.</param>
            /// <returns></returns>
            internal static DateTime ConvertFromLocalToUtc(DateTime dateTime)
            {
                TimeSpan timeSpanDifferenceBetweenLocalAndUtc = GetTimeSpanDifferenceBetweenLocalAndUtc();
                return dateTime.Subtract(timeSpanDifferenceBetweenLocalAndUtc);
            }

            /// <summary>
            /// Gets the time span difference between local and UTC.
            /// </summary>
            /// <returns></returns>
            private static TimeSpan GetTimeSpanDifferenceBetweenLocalAndUtc()
            {
                DateTime now = DateTime.Now;
                return now.Subtract(now.ToUniversalTime());
            }

            /// <summary>
            /// Gets the date time for schedule.
            /// </summary>
            /// <param name="dayOfMonth">The day of month.</param>
            /// <param name="hour">The hour.</param>
            /// <returns></returns>
            internal static DateTime GetDateTimeForSchedule(int dayOfMonth, int hour)
            {
                return DateTime.ParseExact(hour.ToString(CultureInfo.InvariantCulture) + " " + dayOfMonth.ToString(CultureInfo.InvariantCulture) + " 1", "H d M", CultureInfo.InvariantCulture);
            }

            /// <summary>
            /// Gets the date time for schedule.
            /// </summary>
            /// <param name="hour">The hour.</param>
            /// <returns></returns>
            internal static DateTime GetDateTimeForSchedule(int hour)
            {
                return DateTime.ParseExact(hour.ToString(CultureInfo.InvariantCulture) + ":00", "H:mm", CultureInfo.InvariantCulture);
            }

            /// <summary>
            /// Gets the date time for schedule.
            /// </summary>
            /// <param name="dayOfWeek">The day of week.</param>
            /// <param name="hour">The hour.</param>
            /// <returns></returns>
            internal static DateTime GetDateTimeForSchedule(DayOfWeek dayOfWeek, int hour)
            {
                DateTime time = DateTime.ParseExact(hour.ToString(CultureInfo.InvariantCulture) + ":00", "H:mm", CultureInfo.InvariantCulture);
                return time.AddDays(dayOfWeek - time.DayOfWeek);
            }
        }
 

 


 

         

    }
}
