using System;
using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server;
using Microsoft.Office.Server.UserProfiles;

namespace Edfinancial.SharePoint.STSADM.Commands.UserProfiles
{
    public class EditProfileProperty : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EditProfileProperty"/> class.
        /// </summary>
        public EditProfileProperty()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("name", "name", false, null, new SPNonEmptyValidator(), "Please specify the profile property name."));
            parameters.Add(new SPParam("description", "desc", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("displayname", "dname", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("alias", "a", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("showchangesincolleaguetrackerwebpart", "collegueeventlog", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("indexed", "i", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("isusereditable", "ue", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("isvisibleoneditor", "ve", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("isvisibleonviewer", "vv", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("separator", "sep", false, null, new SPRegexValidator("(?i:^comma$|^semicolon$)")));
            parameters.Add(new SPParam("maximumshown", "max", false, null, new SPIntRangeValidator(0, 25)));
            parameters.Add(new SPParam("mapping", "map", false, null, new SPNullOrNonEmptyValidator()));
            parameters.Add(new SPParam("connectionname", "conn", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("associationname", "assoc", false, null, new SPNonEmptyValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nEdits a profile property.\r\n\r\nParameters:");
            sb.Append("\r\n\t-sspname <name of the SSP>");
            sb.Append("\r\n\t-name <name of the profile property to edit>");
            sb.Append("\r\n\t[-description <property description>]");
            sb.Append("\r\n\t[-displayname <display name>]");
            sb.Append("\r\n\t[-alias <true | false>]");
            sb.Append("\r\n\t[-showchangesincolleaguetrackerwebpart <true | false>]");
            sb.Append("\r\n\t[-indexed <true | false>]");
            sb.Append("\r\n\t[-isusereditable <true | false>]");
            sb.Append("\r\n\t[-isvisibleoneditor <true | false>]");
            sb.Append("\r\n\t[-isvisibleonviewer <true | false>]");
            sb.Append("\r\n\t[-separator <comma | semicolon>]");
            sb.Append("\r\n\t[-maximumshown <maximum number of values to show before displaying ellipsis for multi-value property>]");
            sb.Append("\r\n\t[-mapping <data source field to map to (case sensitive) - leave empty to clear mapping>]");
            sb.Append("\r\n\t[-connectionname <source data connection>]");
            sb.Append("\r\n\t[-associationname <data source associated entity>]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string name = Params["name"].Value;

            ServerContext serverContext = ServerContext.GetContext(Params["sspname"].Value);
            UserProfileConfigManager manager = new UserProfileConfigManager(serverContext);
            PropertyCollection properties = manager.GetProperties();
            Property property = properties.GetPropertyByName(name);

            if (property == null)
            {
                // We couldn't find using the system name so try using the display name.
                foreach (Property item in properties)
                {
                    if (item.DisplayName.ToLowerInvariant() == name.ToLowerInvariant())
                    {
                        if (property != null)
                            throw new ArgumentException(string.Format("Duplicate properties found matching the name {0}.  Use the system name instead (use enumprofileprivacypolicies to get system names).", name));
                        property = item;
                    }
                }
            }

            if (property == null)
            {
                throw new ArgumentException(string.Format("Could not find User Profile property '{0}'", name));
            }


            if (Params["description"].UserTypedIn)
                property.Description = Params["description"].Value;

            if (Params["displayname"].UserTypedIn)
                property.DisplayName = Params["displayname"].Value;

            if (Params["alias"].UserTypedIn)
                property.IsAlias = bool.Parse(Params["alias"].Value);

            if (Params["showchangesincolleaguetrackerwebpart"].UserTypedIn)
                property.IsColleagueEventLog = bool.Parse(Params["showchangesincolleaguetrackerwebpart"].Value);

            if (Params["indexed"].UserTypedIn)
                property.IsSearchable = bool.Parse(Params["indexed"].Value);

            if (Params["isusereditable"].UserTypedIn)
                property.IsUserEditable = bool.Parse(Params["isusereditable"].Value);

            if (Params["isvisibleoneditor"].UserTypedIn)
                property.IsVisibleOnEditor = bool.Parse(Params["isvisibleoneditor"].Value);

            if (Params["isvisibleonviewer"].UserTypedIn)
                property.IsVisibleOnViewer = bool.Parse(Params["isvisibleonviewer"].Value);

            if (Params["maximumshown"].UserTypedIn)
                property.MaximumShown = int.Parse(Params["maximumshown"].Value);

            if (Params["separator"].UserTypedIn)
                property.Separator = (MultiValueSeparator)Enum.Parse(typeof(MultiValueSeparator), Params["separator"].Value, true);


            if (Params["mapping"].UserTypedIn)
            {
                PropertyMapCollection propertyMapping = manager.GetDataSource().PropertyMapping;

                if (!string.IsNullOrEmpty(Params["mapping"].Value))
                {
                    PropertyMap map = propertyMapping[property.Name];

                    if (map == null)
                    {
                        string connectionName = string.Empty;
                        string associationName = string.Empty;
                        if (Params["connectionname"].UserTypedIn)
                            connectionName = Params["connectionname"].Value;
                        if (Params["associationname"].UserTypedIn)
                            associationName = Params["associationname"].Value;

                        propertyMapping.Add(property.Name, Params["mapping"].Value, connectionName, associationName);
                    }
                    else
                    {
                        map.DSPropName = Params["mapping"].Value;
                        if (Params["connectionname"].UserTypedIn)
                            map.ConnectionName = Params["connectionname"].Value;
                        if (Params["associationname"].UserTypedIn)
                            map.AssociationName = Params["associationname"].Value;
                        map.Commit();
                    }
                }
                else
                    propertyMapping.Remove(property.Name);
            }

            property.Commit();

            return 1;
        }

        #endregion

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["connectionname"].UserTypedIn && !Params["mapping"].UserTypedIn)
                throw new SPSyntaxException(
                    "Parameter 'connectionname' is only valid if parameter 'mapping' is specified.");

            if (Params["associationname"].UserTypedIn && !Params["mapping"].UserTypedIn)
                throw new SPSyntaxException(
                    "Parameter 'associationname' is only valid if parameter 'mapping' is specified.");

            if (Params["mapping"].UserTypedIn && string.IsNullOrEmpty(Params["mapping"].Value) && 
                (Params["connectionname"].UserTypedIn || Params["associationname"].UserTypedIn))
                throw new SPSyntaxException(
                    "Parameters 'connectionname' and 'associationname' are invalid when parameter 'mapping' is empty.");

            base.Validate(keyValues);
        }
    }
}
