using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server;
using Microsoft.Office.Server.UserProfiles;

namespace Edfinancial.SharePoint.STSADM.Commands.UserProfiles
{
    public class EnumProfilePrivacyPolicies : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EnumProfilePrivacyPolicies"/> class.
        /// </summary>
        public EnumProfilePrivacyPolicies()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("outputfile", "output", false, null, new SPDirectoryExistsAndValidFileNameValidator(), "Make sure the output directory exists and a valid filename is provided."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nLists the user profile privacy policies associated with the SSP.\r\n\r\nParameters:");
            sb.Append("\r\n\t-sspname <name of the SSP>");
            sb.Append("\r\n\t[-outputfile <file to output results to>]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            ServerContext serverContext = ServerContext.GetContext(Params["sspname"].Value);
            UserProfileConfigManager manager = new UserProfileConfigManager(serverContext);
            PropertyCollection properties = manager.GetProperties();

            StringBuilder sb = new StringBuilder();

            XmlTextWriter xmlWriter = new XmlTextWriter(new StringWriter(sb));
            xmlWriter.Formatting = Formatting.Indented;

            xmlWriter.WriteStartElement("PrivacyPolicies");
            xmlWriter.WriteAttributeString("ssp", Params["sspname"].Value);

            foreach (IPrivacyPolicyItem item in properties)
            {
                AddXml(xmlWriter, item);
            }

            PrivacyPolicyManager privacyPolicyManager = ProfileLoader.GetProfileLoader(serverContext).GetUserProfileManager().GetPrivacyPolicy();
            foreach (PrivacyPolicyItem item in privacyPolicyManager.GetAllItems())
            {
                AddXml(xmlWriter, item);
            }

            xmlWriter.WriteEndElement();
            xmlWriter.Flush();

            output += sb.ToString();

            if (Params["outputfile"].UserTypedIn)
            {
                File.WriteAllText(Params["outputfile"].Value, output);
                output = string.Empty;
            }

            return 1;
        }

        #endregion

        /// <summary>
        /// Adds the XML.
        /// </summary>
        /// <param name="xmlWriter">The XML writer.</param>
        /// <param name="item">The item.</param>
        private static void AddXml(XmlTextWriter xmlWriter, IPrivacyPolicyItem item)
        {
            xmlWriter.WriteStartElement("Policy");

            xmlWriter.WriteAttributeString("isProperty", (item is Property).ToString());

            if (item is PrivacyPolicyItem)
            {
                xmlWriter.WriteElementString("Id", ((PrivacyPolicyItem)item).Id.ToString());
                xmlWriter.WriteElementString("Group", ((PrivacyPolicyItem)item).Group);
                xmlWriter.WriteElementString("FilterPrivacyItems", ((PrivacyPolicyItem)item).FilterPrivacyItems.ToString());
            }
            else if (item is Property)
            {
                xmlWriter.WriteElementString("Name", ((Property)item).Name);
                xmlWriter.WriteElementString("Description", ((Property)item).Description);
                xmlWriter.WriteElementString("DisplayOrder", ((Property)item).DisplayOrder.ToString());
                xmlWriter.WriteElementString("IsReplicable", ((Property)item).IsReplicable.ToString());
                xmlWriter.WriteElementString("IsSection", ((Property)item).IsSection.ToString());
                xmlWriter.WriteElementString("URI", ((Property)item).URI);
            }

            xmlWriter.WriteElementString("DisplayName", item.DisplayName);
            xmlWriter.WriteElementString("AllowPolicyOverride", item.AllowPolicyOverride.ToString());
            xmlWriter.WriteElementString("DefaultPrivacy", item.DefaultPrivacy.ToString());
            xmlWriter.WriteElementString("PrivacyPolicy", item.PrivacyPolicy.ToString());
            xmlWriter.WriteElementString("UserOverridePrivacy", item.UserOverridePrivacy.ToString());

            xmlWriter.WriteEndElement();
        }
    }
}
