using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server;
using Microsoft.Office.Server.UserProfiles;

namespace Edfinancial.SharePoint.STSADM.Commands.UserProfiles
{
    public class EnumProfileProperties : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EnumProfilePrivacyPolicy"/> class.
        /// </summary>
        public EnumProfileProperties()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("outputfile", "output", false, null, new SPDirectoryExistsAndValidFileNameValidator(), "Make sure the output directory exists and a valid filename is provided."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nLists the user profile properties associated with the SSP.\r\n\r\nParameters:");
            sb.Append("\r\n\t-sspname <name of the SSP>");
            sb.Append("\r\n\t[-outputfile <file to output results to>]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            ServerContext serverContext = ServerContext.GetContext(Params["sspname"].Value);
            UserProfileConfigManager manager = new UserProfileConfigManager(serverContext);
            PropertyCollection properties = manager.GetProperties();

            StringBuilder sb = new StringBuilder();

            XmlTextWriter xmlWriter = new XmlTextWriter(new StringWriter(sb));
            xmlWriter.Formatting = Formatting.Indented;

            xmlWriter.WriteStartElement("Properties");
            xmlWriter.WriteAttributeString("ssp", Params["sspname"].Value);

            foreach (Property item in properties)
            {
                AddXml(xmlWriter, item, manager);
            }

            xmlWriter.WriteEndElement();
            xmlWriter.Flush();

            output += sb.ToString();

            if (Params["outputfile"].UserTypedIn)
            {
                File.WriteAllText(Params["outputfile"].Value, output);
                output = string.Empty;
            }

            return 1;
        }

        #endregion

        /// <summary>
        /// Adds the XML.
        /// </summary>
        /// <param name="xmlWriter">The XML writer.</param>
        /// <param name="item">The item.</param>
        /// <param name="manager">The manager.</param>
        private static void AddXml(XmlTextWriter xmlWriter, Property item, UserProfileConfigManager manager)
        {
            xmlWriter.WriteStartElement("Property");
            
            xmlWriter.WriteElementString("Name", item.Name);
            xmlWriter.WriteElementString("AllowPolicyOverride", item.AllowPolicyOverride.ToString());
            xmlWriter.WriteStartElement("ChoiceList");
            if (item.ChoiceList != null)
            {
                foreach (string s in item.ChoiceList.GetAllTerms(true))
                {
                    xmlWriter.WriteElementString("Choice", s);
                }
            }
            xmlWriter.WriteEndElement();
            xmlWriter.WriteElementString("ChoiceType", item.ChoiceType.ToString());
            xmlWriter.WriteElementString("DefaultPrivacy", item.DefaultPrivacy.ToString());
            xmlWriter.WriteElementString("Description", item.Description);
            xmlWriter.WriteElementString("DisplayName", item.DisplayName);
            xmlWriter.WriteElementString("DisplayOrder", item.DisplayOrder.ToString());
            xmlWriter.WriteElementString("IsAdminEditable", item.IsAdminEditable.ToString());
            xmlWriter.WriteElementString("IsAlias", item.IsAlias.ToString());
            xmlWriter.WriteElementString("IsColleagueEventLog", item.IsColleagueEventLog.ToString());
            xmlWriter.WriteElementString("IsImported", item.IsImported.ToString());
            xmlWriter.WriteElementString("IsMultivalued", item.IsMultivalued.ToString());
            xmlWriter.WriteElementString("IsReplicable", item.IsReplicable.ToString());
            xmlWriter.WriteElementString("IsRequired", item.IsRequired.ToString());
            xmlWriter.WriteElementString("IsSearchable", item.IsSearchable.ToString());
            xmlWriter.WriteElementString("IsSection", item.IsSection.ToString());
            xmlWriter.WriteElementString("IsSystem", item.IsSystem.ToString());
            xmlWriter.WriteElementString("IsUpgrade", item.IsUpgrade.ToString());
            xmlWriter.WriteElementString("IsUpgradePrivate", item.IsUpgradePrivate.ToString());
            xmlWriter.WriteElementString("IsUserEditable", item.IsUserEditable.ToString());
            xmlWriter.WriteElementString("IsVisibleOnEditor", item.IsVisibleOnEditor.ToString());
            xmlWriter.WriteElementString("IsVisibleOnViewer", item.IsVisibleOnViewer.ToString());
            xmlWriter.WriteElementString("Length", item.Length.ToString());
            xmlWriter.WriteElementString("ManagedPropertyName", item.ManagedPropertyName);
            xmlWriter.WriteElementString("MaximumShown", item.MaximumShown.ToString());
            xmlWriter.WriteElementString("PrivacyPolicy", item.PrivacyPolicy.ToString());
            xmlWriter.WriteElementString("Separator", item.Separator.ToString());
            xmlWriter.WriteElementString("Type", item.Type);
            xmlWriter.WriteElementString("URI", item.URI);
            xmlWriter.WriteElementString("UserOverridePrivacy", item.UserOverridePrivacy.ToString());

            xmlWriter.WriteStartElement("ImportMapping");
            PropertyMapCollection propertyMapping = manager.GetDataSource().PropertyMapping;
            PropertyMap map = propertyMapping[item.Name];

            if (map != null)
            {
                xmlWriter.WriteElementString("DSPropName", map.DSPropName);
                xmlWriter.WriteElementString("ConnectionName", map.ConnectionName);
                xmlWriter.WriteElementString("AssociationName", map.AssociationName);
            }
            xmlWriter.WriteEndElement();

            xmlWriter.WriteEndElement();
        }
    }
}
