using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Data.SqlClient;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Edfinancial.SharePoint.STSADM.Commands.UserProfiles.UserProfiles2003DataSetTableAdapters;
using Microsoft.Office.Server;
using Microsoft.Office.Server.Infrastructure;
using Microsoft.Office.Server.UserProfiles;

namespace Edfinancial.SharePoint.STSADM.Commands.UserProfiles
{
    public class Migrate2003ProfilesTo2007 : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Migrate2003ProfilesTo2007"/> class.
        /// </summary>
        public Migrate2003ProfilesTo2007()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("profdb2003", "server", true, null, new SPNonEmptyValidator(), "Please specify the Profiles Database Server for SharePoint 2003 (profdb2003)."));
            parameters.Add(new SPParam("profdbname2003", "dbname", true, null, new SPNonEmptyValidator(), "Please specify the Profiles Database Name for SharePoint 2003 (profdbname2003)."));
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("input", "i", true, null, new SPFileExistsValidator(), "Please specify the input file name (i.e., \"c:\\propertymap.xml\")."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nMigrates profile data and columns from a SharePoint 2003 profile database to SharePoint 2007.\r\n\r\nParameters:");
            sb.Append("\r\n\t-profdb2003 <database server containing the 2003 profile database>");
            sb.Append("\r\n\t-profdbname2003 <2003 profile database name>");
            sb.Append("\r\n\t-sspname <name of the 2007 SSP>");
            sb.Append("\r\n\t-input <mapping file>");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string profdb = Params["profdb2003"].Value;
            string profdbname = Params["profdbname2003"].Value;
            string sspname = Params["sspname"].Value;
            string inputFileName = Params["input"].Value;

            UserProfileConfigManager profConfigManager = new UserProfileConfigManager(ServerContext.GetContext(sspname));
            
            PropertyCollection propColl = profConfigManager.GetProperties();

            XmlDocument map = new XmlDocument();
            map.Load(inputFileName);

            #region Create any missing properties

            // First need to create any missing properties
            XmlNodeList propertiesXml = map.SelectNodes("/ProfileMap/Property");
            foreach (XmlElement propXml in propertiesXml)
            {
                string mossPropertyName = propXml.GetAttribute("propertyName-2007");
                if (string.IsNullOrEmpty(mossPropertyName)) // In case we've already created the property (result of running the command more than once with the same mapping file)
                    mossPropertyName = propXml.GetAttribute("PropertyName");

                Property prop = propColl.GetPropertyByName(mossPropertyName);
                if (prop == null)
                {
                    // We couldn't find a matching property so go ahead and create it
                    prop = propColl.Create(false);
                    prop.Name = mossPropertyName;
                    if (!string.IsNullOrEmpty(propXml.GetAttribute("DisplayName")))
                        prop.DisplayName = propXml.GetAttribute("DisplayName");
                    prop.IsAlias = bool.Parse(propXml.GetAttribute("IsAlias"));
                    prop.IsUserEditable = bool.Parse(propXml.GetAttribute("IsEditable"));
                    prop.Length = int.Parse(propXml.GetAttribute("Length"));
                    prop.Type = propXml.GetAttribute("DataTypeName");
                    bool isPrivate = bool.Parse(propXml.GetAttribute("IsPrivate"));
                    if (isPrivate)
                        prop.DefaultPrivacy = Privacy.Private;
                    else
                        prop.DefaultPrivacy = Privacy.Public;

                    if (!string.IsNullOrEmpty(propXml.GetAttribute("IsVisibleOnEditor")))
                        prop.IsVisibleOnEditor = bool.Parse(propXml.GetAttribute("IsVisibleOnEditor"));

                    if (!string.IsNullOrEmpty(propXml.GetAttribute("IsVisibleOnViewer")))
                        prop.IsVisibleOnViewer = bool.Parse(propXml.GetAttribute("IsVisibleOnViewer"));

                    propColl.Add(prop);
                    //output += "DEBUG - Property " + mossPropertyName + " added.\r\n";
                }
            }

            #endregion

            UserProfileManager profManager = new UserProfileManager(ServerContext.GetContext(sspname));

            UserProfileValueTableAdapter adapter = new UserProfileValueTableAdapter();
            adapter.Connection = new SqlConnection("Data Source=" + profdb + ";Initial Catalog=" + profdbname + ";Integrated Security=True");

            // Now we need to set the properties
            Dictionary<string, UserProfile> profiles = new Dictionary<string, UserProfile>();

            foreach (XmlElement propXml in propertiesXml)
            {
                string mossPropertyName = propXml.GetAttribute("propertyName-2007");
                if (string.IsNullOrEmpty(mossPropertyName)) // In case we've already created the property (result of running the command more than once with the same mapping file)
                    mossPropertyName = propXml.GetAttribute("PropertyName");

                //output += "DEBUG - Setting property " + mossPropertyName + "\r\n";

                UserProfiles2003DataSet.UserProfileValueDataTable table = adapter.GetData(long.Parse(propXml.GetAttribute("PropertyID")));

                foreach (UserProfiles2003DataSet.UserProfileValueRow row in table.Rows)
                {
                    string accountName = row.NTName;

                    try
                    {
                        UserProfile profile;

                        #region Make sure we can handle setting the property

                        if (profiles.ContainsKey(accountName))
                            profile = profiles[accountName];
                        else
                        {
                            if (profManager.UserExists(accountName))
                                profile = profManager.GetUserProfile(accountName);
                            else
                            {
                                //output += "WARNING - Profile not found: " + accountName + "\r\n";
                                continue;
                            }
                            profiles[accountName] = profile;
                        }
                        if (bool.Parse(propXml.GetAttribute("IsMultiValue")))
                        {
                            output += "WARNING - Multi-Value column found (currently not supported): " + mossPropertyName + "\r\n";
                            break;
                        }
                        Property userProperty = profile[mossPropertyName].Property;

                        int length = -1;
                        if (!string.IsNullOrEmpty(propXml.GetAttribute("Length")))
                            length = int.Parse(propXml.GetAttribute("Length"));

                        if (userProperty.Length < length && length != -1)
                        {
                            output += "WARNING - Length of " + mossPropertyName + " is too small by " + (length - userProperty.Length) + "\r\n";
                            break;
                        }
                        if (userProperty.Type == "HTML")
                        {
                            string safeHtml = SafeHtmlWrapper.MakeSafe(row.PropertyVal.ToString());
                            row.PropertyVal = safeHtml;

                            if (userProperty.Length < safeHtml.Length)
                            {
                                output += "WARNING - Length of HTML property " + mossPropertyName + " is too small by " + (safeHtml.Length - userProperty.Length) + " - Removing HTML...\r\n";
                                row.PropertyVal = SafeHtmlWrapper.StripHtmlToPlainText(safeHtml);
                            }
                        }

                        #endregion

                        #region Set Value

                        object val = row.PropertyVal;

                        // This is probably not necessary as I believe the Value property will do a basic conversion but it makes me feel safer.
                        switch (userProperty.Type)
                        {
                            case "int":
                                try
                                {
                                    val = (int)val;
                                }
                                catch
                                {
                                    val = int.Parse(val.ToString());
                                }
                                break;
                            case "big integer":
                                try
                                {
                                    val = (long)val;
                                }
                                catch
                                {
                                    val = long.Parse(val.ToString());
                                }
                                break;
                            case "date time":
                            case "date":
                                try
                                {
                                    val = (DateTime)val;
                                }
                                catch
                                {
                                    val = DateTime.Parse(val.ToString());
                                }
                                break;
                            case "float":
                                try
                                {
                                    val = (float)val;
                                }
                                catch
                                {
                                    val = float.Parse(val.ToString());
                                }
                                break;
                            case "unique identifier":
                                try
                                {
                                    val = (Guid)val;
                                }
                                catch
                                {
                                    val = new Guid(val.ToString());
                                }
                                break;
                            case "boolean":
                                try
                                {
                                    val = (bool)val;
                                }
                                catch
                                {
                                    try
                                    {
                                        val = (int)val != 0;
                                    }
                                    catch
                                    {
                                        val = val.ToString().ToLower() == "true";
                                    }
                                }
                                break;
                            default:
                                if (val.ToString().Length > userProperty.Length)
                                    val = val.ToString().Substring(0, userProperty.Length);
                                else
                                    val = val.ToString();
                                break;
                        }

                        if (userProperty.IsMultivalued)
                        {
                            if (val.ToString().Length > 0)
                                profile[mossPropertyName].Add(val);
                        }
                        else
                        {
                            profile[mossPropertyName].Value = val;
                        }
                        #endregion

                        profile.Commit();
                    }
                    catch (Exception ex)
                    {
                        output += "ERROR - " + ex.Message + "\r\n";
                        output += "Stack Trace: \r\n" + ex.StackTrace + "\r\n";
                        output += "Property: " + mossPropertyName + "\r\n";
                        output += "User: " + accountName + "\r\n";
                        output += "Text: " + row.PropertyVal + "\r\n\r\n";
                        output += "Text Length: " + row.PropertyVal.ToString().Length + "\r\n\r\n";
                        output += "SafeHTML Length: " + SafeHtmlWrapper.MakeSafe(row.PropertyVal.ToString()).Length + "\r\n\r\n";
                        return 0;
                    }
                }

                
            }


            return 1;
        }

        #endregion
    }
}
