using System;
using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server;
using Microsoft.Office.Server.UserProfiles;

namespace Edfinancial.SharePoint.STSADM.Commands.UserProfiles
{
    public class SetProfilePrivacyPolicy : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetProfilePrivacyPolicy"/> class.
        /// </summary>
        public SetProfilePrivacyPolicy()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("name", "name", false, null, new SPNonEmptyValidator(), "Please specify the property name."));
            SPEnumValidator defaultPrivacyValidator = new SPEnumValidator(typeof(Privacy));
            parameters.Add(new SPParam("defaultprivacy", "d", false, null, defaultPrivacyValidator));
            SPEnumValidator privacyValidator = new SPEnumValidator(typeof(PrivacyPolicy));
            parameters.Add(new SPParam("privacy", "p", false, null, privacyValidator));
            parameters.Add(new SPParam("allowuseroverride", "uo", false, null, new SPTrueFalseValidator()));
            parameters.Add(new SPParam("replicable", "r", false, null, new SPTrueFalseValidator()));
            
            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the privacy policy settings for a profile property.\r\n\r\nParameters:");
            sb.Append("\r\n\t-sspname <name of the SSP>");
            sb.Append("\r\n\t-name <name of the property to edit>");
            sb.AppendFormat("\r\n\t[-defaultprivacy <{0}>]", defaultPrivacyValidator.DisplayValue);
            sb.AppendFormat("\r\n\t[-privacy <{0}>]", privacyValidator.DisplayValue);
            sb.Append("\r\n\t[-allowuseroverride <true | false>]");
            sb.Append("\r\n\t[-replicable <true | false>]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string name = Params["name"].Value;

            ServerContext serverContext = ServerContext.GetContext(Params["sspname"].Value);
            UserProfileConfigManager manager = new UserProfileConfigManager(serverContext);
            PropertyCollection properties = manager.GetProperties();
            IPrivacyPolicyItem property = properties.GetPropertyByName(name);

            if (property == null)
            {
                // We couldn't find using the system name so try using the display name.
                foreach (IPrivacyPolicyItem item in properties)
                {
                    if (item.DisplayName.ToLowerInvariant() == name.ToLowerInvariant())
                    {
                        if (property != null)
                            throw new ArgumentException(string.Format("Duplicate properties found matching the name {0}.  Use the system name instead (use enumprofileprivacypolicies to get system names).", name));
                        property = item;
                    }
                }
            }

            if (property == null)
            {
                PrivacyPolicyManager privacyPolicyManager = ProfileLoader.GetProfileLoader(serverContext).GetUserProfileManager().GetPrivacyPolicy();

                foreach (PrivacyPolicyItem item in privacyPolicyManager.GetAllItems())
                {
                    if (item.DisplayName.ToLowerInvariant() == name.ToLowerInvariant())
                    {
                        property = item;
                        break;
                    }
                }
                if (property == null)
                    throw new ArgumentException(
                        string.Format("Could not find a policy for the User Profile property '{0}'", name));

                
                if (Params["privacy"].UserTypedIn)
                    property.PrivacyPolicy = (PrivacyPolicy)Enum.Parse(typeof(PrivacyPolicy), Params["privacy"].Value, true);

                if (((PrivacyPolicyItem)property).Id != PrivacyPolicyIdConstants.MyColleaguesRecommendations)
                {
                    if (Params["defaultprivacy"].UserTypedIn)
                        property.DefaultPrivacy = (Privacy)Enum.Parse(typeof(Privacy), Params["defaultprivacy"].Value, true);

                    if (Params["allowuseroverride"].UserTypedIn)
                        property.UserOverridePrivacy = bool.Parse(Params["allowuseroverride"].Value);
                }

                if (Params["replicable"].UserTypedIn)
                    Console.WriteLine("WARNING: Property does not support replication. \"replicable\" parameter ignored.\r\n");

                property.Commit();
            }
            else
            {
                if (Params["privacy"].UserTypedIn)
                {
                    if (property.AllowPolicyOverride)
                        property.PrivacyPolicy =(PrivacyPolicy) Enum.Parse(typeof (PrivacyPolicy), Params["privacy"].Value, true);
                    else 
                        Console.WriteLine("WARNING: Property does not allow policy override. \"privacy\" parameter ignored.\r\n");
                }

                if (Params["defaultprivacy"].UserTypedIn)
                    property.DefaultPrivacy = (Privacy)Enum.Parse(typeof(Privacy), Params["defaultprivacy"].Value, true);

                if (Params["replicable"].UserTypedIn)
                    ((Property)property).IsReplicable = bool.Parse(Params["replicable"].Value);

                if (Params["allowuseroverride"].UserTypedIn)
                {
                    if (property.AllowPolicyOverride && !((Property)property).IsReplicable)
                        property.UserOverridePrivacy = bool.Parse(Params["allowuseroverride"].Value);
                    else
                    {
                        if (((Property)property).IsReplicable)
                            Console.WriteLine("WARNING: Property does not allow policy override because it is marked as Replicable. \"allowuseroverride\" parameter ignored.\r\n");
                        else
                            Console.WriteLine("WARNING: Property does not allow policy override. \"allowuseroverride\" parameter ignored.\r\n");
                    }
                }

                property.Commit();
            }
            return 1;
        }

        #endregion
    }
}
