using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace Edfinancial.SharePoint.STSADM.Commands.WebApplications
{
    public class AddUserPolicyForWebApp : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="AddUserPolicyForWebApp"/> class.
        /// </summary>
        public AddUserPolicyForWebApp()
        {
            SPEnumValidator urlZoneValidator = new SPEnumValidator(typeof(SPUrlZone), new string[] {"All"});
            
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the web application url."));
            parameters.Add(new SPParam("zone", "z", true, null, urlZoneValidator));
            parameters.Add(new SPParam("userlogin", "u", true, null, new SPNonEmptyValidator(), "Please specify the users login."));
            parameters.Add(new SPParam("username", "username", true, null, new SPNonEmptyValidator(), "Please specify the users name."));
            parameters.Add(new SPParam("permissions", "p", true, null, new SPNonEmptyValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nAdds a user policy for a web application.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <web application url>");
            sb.AppendFormat("\r\n\t-zone <{0}>", urlZoneValidator.DisplayValue);
            sb.Append("\r\n\t-userlogin <DOMAIN\\user>");
            sb.Append("\r\n\t-username <display name>");
            sb.Append("\r\n\t-permissions <comma separated list of policy permissions>");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');
            string login = Params["userlogin"].Value;
            string username = Params["username"].Value;

            SPWebApplication webApp = SPWebApplication.Lookup(new Uri(url));
            SPPolicyCollection policies = GetZonePolicyCollection(Params["zone"].Value, webApp);
            login = Utilities.TryGetNT4StyleAccountName(login, webApp);

            List<SPPolicyRole> roles = new List<SPPolicyRole>();
            foreach (string roleName in Params["permissions"].Value.Split(','))
            {
                SPPolicyRole role = webApp.PolicyRoles[roleName.Trim()];
                if (role == null)
                    throw new SPException(string.Format("The policy permission '{0}' was not found.", roleName.Trim()));

                roles.Add(role);
            }
            SPPolicy policy = policies.Add(login, username);
            
            foreach (SPPolicyRole role in roles)
                policy.PolicyRoleBindings.Add(role);

            webApp.Update();

            return 1;
        }

        #endregion

        /// <summary>
        /// Gets the zone policy collection.
        /// </summary>
        /// <param name="zoneName">Name of the zone.</param>
        /// <param name="application">The application.</param>
        /// <returns></returns>
        internal static SPPolicyCollection GetZonePolicyCollection(string zoneName, SPWebApplication application)
        {
            zoneName = zoneName.ToLowerInvariant();
            return (!zoneName.Equals("all") ? application.ZonePolicies((SPUrlZone)Enum.Parse(typeof(SPUrlZone), zoneName, true)) : application.Policies);
        }


    }
}
