using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebPartPages;
using WebPart=System.Web.UI.WebControls.WebParts.WebPart;

namespace Edfinancial.SharePoint.STSADM.Commands.WebParts
{
    public class MoveWebPart : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="MoveWebPart"/> class.
        /// </summary>
        public MoveWebPart()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPNonEmptyValidator(), "Please specify the page url."));
            parameters.Add(new SPParam("id", "id", false, null, new SPNonEmptyValidator(), "Please specify the web part ID."));
            parameters.Add(new SPParam("title", "t", false, null, new SPNonEmptyValidator(), "Please specify the web part title."));
            parameters.Add(new SPParam("zone", "z", false, null, new SPNonEmptyValidator(), "Please specify the zone to move to."));
            parameters.Add(new SPParam("zoneindex", "zi", false, null, new SPIntRangeValidator(0, int.MaxValue)));
            parameters.Add(new SPParam("publish", "p", false, null, null));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nMoves a web part on a page.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <web part page URL>");
            sb.Append("\r\n\t{-id <web part ID> |\r\n\t -title <web part title>}");
            sb.Append("\r\n\t[-zone <zone ID>]");
            sb.Append("\r\n\t[-zoneindex <zone index>]");
            sb.Append("\r\n\t[-publish]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            SPBinaryParameterValidator.Validate("id", Params["id"].Value, "title", Params["title"].Value);
            if (!Params["zone"].UserTypedIn && !Params["zoneindex"].UserTypedIn)
                throw new SPSyntaxException("You must specify at least the zone or zoneindex parameters.");

            string url = Params["url"].Value;

            using (SPSite site = new SPSite(url))
            using (SPWeb web = site.OpenWeb()) // The url contains a filename so AllWebs[] will not work unless we want to try and parse which we don't
            {
                SPFile file = web.GetFile(url);

                if (file.InDocumentLibrary)
                {
                    if (!Utilities.IsCheckedOut(file.Item) || !Utilities.IsCheckedOutByCurrentUser(file.Item))
                        file.CheckOut();
                            // If it's checked out by another user then this will throw an informative exception so let it do so.
                }

                string displayTitle = string.Empty;
                try
                {
                    WebPart wp;
                    SPLimitedWebPartManager manager;
                    if (Params["id"].UserTypedIn)
                    {
                        wp = Utilities.GetWebPartById(web, url, Params["id"].Value, out manager);
                    }
                    else
                    {
                        wp = Utilities.GetWebPartByTitle(web, url, Params["title"].Value, out manager);
                        if (wp == null)
                        {
                            throw new SPException(
                                "Unable to find specified web part.  Try specifying the -id parameter instead (use enumpagewebparts to get the ID)");
                        }
                    }

                    if (wp == null)
                    {
                        throw new SPException("Unable to find specified web part.");
                    }

                    string zoneID = manager.GetZoneID(wp);
                    int zoneIndex = wp.ZoneIndex;

                    if (Params["zone"].UserTypedIn)
                        zoneID = Params["zone"].Value;
                    if (Params["zoneindex"].UserTypedIn)
                        zoneIndex = int.Parse(Params["zoneindex"].Value);

                    // Set this so that we can add it to the check-in comment.
                    displayTitle = wp.DisplayTitle;

                    manager.MoveWebPart(wp, zoneID, zoneIndex);
                    manager.SaveChanges(wp);
                }
                finally
                {
                    file.CheckIn("Checking in changes to page layout due to moving of web part " + displayTitle);

                    if (Params["publish"].UserTypedIn && file.InDocumentLibrary)
                    {
                        file.Publish("Publishing changes to page layout due to moving of web part " + displayTitle);
                        if (file.Item.ModerationInformation != null)
                        {
                            file.Approve("Approving changes to page layout due to moving of web part " + displayTitle);
                        }
                    }
                }
            }

            return 1;
        }

        #endregion
    }
}
