using System;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Web.UI.WebControls.WebParts;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.Lists;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using System.Text.RegularExpressions;
using Microsoft.SharePoint.Portal.WebControls;
using Microsoft.SharePoint.Publishing;
using Microsoft.SharePoint.Publishing.Fields;
using Microsoft.SharePoint.Publishing.WebControls;
using Microsoft.SharePoint.WebPartPages;
using WebPart=Microsoft.SharePoint.WebPartPages.WebPart;

namespace Edfinancial.SharePoint.STSADM.Commands.WebParts
{
    public class ReplaceWebPartContent : SPOperation
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ReplaceWebPartContent"/> class.
        /// </summary>
        public ReplaceWebPartContent()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", false, null, new SPNonEmptyValidator(), "Please specify the url to search."));
            parameters.Add(new SPParam("searchstring", "search", true, null, new SPNonEmptyValidator(), "Please specify the search string."));
            parameters.Add(new SPParam("replacestring", "replace", true, null, new SPNullOrNonEmptyValidator(), "Please specify the search string."));
            parameters.Add(new SPParam("scope", "scope", true, null, new SPRegexValidator("(?i:^Farm$|^WebApplication$|^Site$|^Web$|^Page$)")));
            parameters.Add(new SPParam("webpartname", "part", false, null, new SPNonEmptyValidator(), "Please enter the web part name."));
            parameters.Add(new SPParam("quiet", "q", false, null, null));
            parameters.Add(new SPParam("test", "t", false, null, null));
            parameters.Add(new SPParam("publish", "p", false, null, null));
            parameters.Add(new SPParam("logfile", "log", false, null, new SPDirectoryExistsAndValidFileNameValidator()));
            parameters.Add(new SPParam("unsafexml", "unsafexml"));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nReplaces all occurances of the search string with the replacement string.  Supports use of regular expressions.  Use -test to verify your replacements before executing.\r\n\r\nParameters:");
            sb.Append("\r\n\t[-url <url to search>]");
            sb.Append("\r\n\t-searchstring <regular expression string to search for>");
            sb.Append("\r\n\t-replacestring <replacement string>");
            sb.Append("\r\n\t-scope <Farm | WebApplication | Site | Web | Page>");
            sb.Append("\r\n\t[-webpartname <web part name>]");
            sb.Append("\r\n\t[-quiet]");
            sb.Append("\r\n\t[-test]");
            sb.Append("\r\n\t[-logfile <log file>]");
            sb.Append("\r\n\t[-publish]");
            sb.Append("\r\n\t[-unsafexml (treats known XML data as a string)]");

            Init(parameters, sb.ToString());
        }

        #endregion

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            if (url != null)
                url = url.TrimEnd('/');

            Settings settings = new Settings();
            settings.SearchString = Params["searchstring"].Value;
            settings.ReplaceString = Params["replacestring"].Value;
            settings.LogFile = Params["logfile"].Value;
            settings.WebPartName = Params["webpartname"].Value;
            settings.Publish = Params["publish"].UserTypedIn;
            settings.Quiet = Params["quiet"].UserTypedIn;
            settings.Test = Params["test"].UserTypedIn;
            settings.UnsafeXml = Params["unsafexml"].UserTypedIn;

            string scope = Params["scope"].Value.ToLowerInvariant();

            if (scope == "farm")
            {
                foreach (SPService svc in SPFarm.Local.Services)
                {
                    if (!(svc is SPWebService))
                        continue;

                    foreach (SPWebApplication webApp in ((SPWebService)svc).WebApplications)
                    {
                        ReplaceValues(webApp, settings);
                    }
                }
            }
            else if (scope == "webapplication")
            {
                SPWebApplication webApp = SPWebApplication.Lookup(new Uri(url));
                ReplaceValues(webApp, settings);
            }
            else if (scope == "site")
            {
                using (SPSite site = new SPSite(url))
                {
                    ReplaceValues(site, settings);
                }
            }
            else if (scope == "web")
            {
                using (SPSite site = new SPSite(url))
                using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                {
                    ReplaceValues(web, settings);
                }
            }
            else if (scope == "page")
            {
                using (SPSite site = new SPSite(url))
                using (SPWeb web = site.OpenWeb())
                {
                    ReplaceValues(web, web.GetFile(url), settings);
                }

            }

            return 1;
        }

        #endregion

        #region SPOperation Misc Overrides

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["quiet"].UserTypedIn && Params["test"].UserTypedIn && !Params["logfile"].UserTypedIn)
                throw new SPSyntaxException("The quiet parameter and the test parameter are incompatible if no log file is specified.");

            if (Params["scope"].UserTypedIn)
            {
                if (Params["scope"].Value.ToLowerInvariant() == "farm" && Params["url"].UserTypedIn)
                    throw new SPSyntaxException("The url parameter is not compatible with a scope of Farm.");
                if (Params["scope"].Value.ToLowerInvariant() != "farm" && !Params["url"].UserTypedIn)
                    throw new SPSyntaxException("The url parameter is required if the scope is not Farm.");
            }
            base.Validate(keyValues);
        }

        #endregion

        #region ReplaceValues Methods

        #region Looping Methods

        /// <summary>
        /// Replaces the content of the various web parts on every file of every web of every site of a given web application.
        /// </summary>
        /// <param name="webApp">The web app that should be searched.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        private static void ReplaceValues(SPWebApplication webApp, Settings settings)
        {
            Log(settings, "Processing Web Application: " + webApp.DisplayName);
            
            foreach (SPSite site in webApp.Sites)
            {
                try
                {
                    ReplaceValues(site, settings);
                }
                finally
                {
                    site.Dispose();
                }
            }

            Log(settings, "Finished Processing Web Application: " + webApp.DisplayName + "\r\n");
        }

        /// <summary>
        /// Replaces the content of the various web parts on every file of every web of a given site.
        /// </summary>
        /// <param name="site">The site that should be searched.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        private static void ReplaceValues(SPSite site, Settings settings)
        {
            Log(settings, "Processing Site: " + site.ServerRelativeUrl);
            
            foreach (SPWeb web in site.AllWebs)
            {
                try
                {
                    ReplaceValues(web, settings);
                }
                finally
                {
                    web.Dispose();
                }
            }

            Log(settings, "Finished Processing Site: " + site.ServerRelativeUrl + "\r\n");
        }

        /// <summary>
        /// Replaces the content of the various web parts on every file of a given web.
        /// </summary>
        /// <param name="web">The web which should be searched.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        internal static void ReplaceValues(SPWeb web, Settings settings)
        {
            if (settings.Verbose)
                Log(settings, "Processing Web: " + web.ServerRelativeUrl);

            foreach (SPFile file in web.Files)
            {
                ReplaceValues(web, file, settings);
            }

            foreach (SPList list in web.Lists)
            {
                foreach (SPListItem item in list.Items)
                {
                    if (item.File != null && item.File.Url.ToLowerInvariant().EndsWith(".aspx"))
                    {
                        ReplaceValues(web, item.File, settings);
                    }
                }
            }

            if (settings.Verbose)
                Log(settings, "Finished Processing Web: " + web.ServerRelativeUrl + "\r\n");
        }

        /// <summary>
        /// Replaces the content of the various web parts on a given page (file).  This is the main
        /// decision maker method which calls the various worker methods based on web part type.
        /// The following web part types are currently supported (all others will be ignored):
        /// <see cref="ContentEditorWebPart"/>, <see cref="PageViewerWebPart"/>, <see cref="ImageWebPart"/>,
        /// <see cref="SiteDocuments"/>, <see cref="SummaryLinkWebPart"/>, <see cref="DataFormWebPart" />,
        /// <see cref="ContentByQueryWebPart"/>
        /// </summary>
        /// <param name="web">The web to which the file belongs.</param>
        /// <param name="file">The file containing the web parts to search.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        internal static void ReplaceValues(SPWeb web, SPFile file, Settings settings)
        {
            if (file == null)
            {
                return; // This should never be the case.
            }

            if (!Utilities.EnsureAspx(file.Url, true, false))
                return; // We can only handle aspx and master pages.

            SPLimitedWebPartManager manager = null;
            try
            {
                manager = web.GetLimitedWebPartManager(file.Url, PersonalizationScope.Shared);

                if (settings.Verbose)
                    Log(settings, "Processing File: " + manager.ServerRelativeUrl);
                Regex regex = new Regex(settings.SearchString);

                if (file.InDocumentLibrary && Utilities.IsCheckedOut(file.Item) &&
                    !Utilities.IsCheckedOutByCurrentUser(file.Item))
                {
                    return; // The item is checked out by a different user so leave it alone.
                }

                bool fileModified = false;
                bool wasCheckedOut = true;

                SPLimitedWebPartCollection webParts = manager.WebParts;
                for (int i = 0; i < webParts.Count; i++)
                {
                    WebPart webPart = null;
                    try
                    {
                        webPart = webParts[i] as WebPart;

                        if (webPart == null)
                            continue;

                        bool modified = false;
                        string webPartName = webPart.Title.ToLowerInvariant();
                        if (settings.WebPartName == null || settings.WebPartName.ToLowerInvariant() == webPartName)
                        {
                            // As every web part has different requirements we are only going to consider a small subset.
                            // Custom web parts will not be addressed as there's no interface that can utilized.
                            if (webPart is ContentEditorWebPart)
                            {
                                ContentEditorWebPart wp = (ContentEditorWebPart)webPart;
                                webPart = ReplaceValues(web, file, settings, wp, regex, ref manager, ref wasCheckedOut,
                                                  ref modified);
                            }
                            else if (webPart is PageViewerWebPart)
                            {
                                PageViewerWebPart wp = (PageViewerWebPart)webPart;
                                webPart = ReplaceValues(web, file, settings, wp, regex, ref manager, ref wasCheckedOut,
                                                  ref modified);
                            }
                            else if (webPart is ImageWebPart)
                            {
                                ImageWebPart wp = (ImageWebPart)webPart;
                                webPart = ReplaceValues(web, file, settings, wp, regex, ref manager, ref wasCheckedOut,
                                                  ref modified);
                            }
                            else if (webPart is SiteDocuments)
                            {
                                SiteDocuments wp = (SiteDocuments)webPart;
                                webPart = ReplaceValues(web, file, settings, wp, regex, ref manager, ref wasCheckedOut,
                                                  ref modified);
                            }
                            else if (webPart is SummaryLinkWebPart)
                            {
                                SummaryLinkWebPart wp = (SummaryLinkWebPart)webPart;
                                webPart = ReplaceValues(web, file, settings, wp, regex, ref manager, ref wasCheckedOut,
                                                  ref modified);
                            }
                            else if (webPart is ContentByQueryWebPart)
                            {
                                ContentByQueryWebPart wp = (ContentByQueryWebPart)webPart;
                                webPart = ReplaceValues(web, file, settings, wp, regex, ref manager, ref wasCheckedOut,
                                                  ref modified);
                            }
                            else if (webPart is DataFormWebPart)
                            {
                                DataFormWebPart wp = (DataFormWebPart)webPart;
                                webPart = ReplaceValues(web, file, settings, wp, regex, ref manager, ref wasCheckedOut,
                                                  ref modified);
                            }

                            if (modified && !settings.Test)
                                manager.SaveChanges(webPart);

                            if (modified)
                                fileModified = true;
                        }
                    }
                    finally
                    {
                        if (webPart != null)
                            webPart.Dispose();
                    }
                }

                if (!settings.Test)
                {
                    if (fileModified)
                        file.CheckIn("Checking in changes to list item due to automated search and replace (\"" +
                                     settings.SearchString + "\" replaced with \"" + settings.ReplaceString + "\").");

                    if (file.InDocumentLibrary && fileModified && settings.Publish && !wasCheckedOut)
                        PublishItems.PublishListItem(file.Item, file.Item.ParentList,
                                                     new PublishItems.Settings(settings.Quiet, settings.Test,
                                                                               settings.LogFile),
                                                     "\"stsadm.exe -o replacewebpartcontent\"");
                }
                if (settings.Verbose)
                    Log(settings, "Finished Processing File: " + manager.ServerRelativeUrl + "\r\n");
            }
            finally
            {
                if (manager != null)
                {
                    manager.Web.Dispose(); // manager.Dispose() does not dispose of the SPWeb object and results in a memory leak.
                    manager.Dispose();
                }
            }
        }

        #endregion

        #region Primary Worker Methods

        /// <summary>
        /// Replaces the content of a <see cref="ContentEditorWebPart"/>.
        /// </summary>
        /// <param name="web">The web that the file belongs to.</param>
        /// <param name="file">The file that the web part is associated with.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        /// <param name="wp">The web part whose content will be replaced.</param>
        /// <param name="regex">The regular expression object which contains the search pattern.</param>
        /// <param name="manager">The web part manager.  This value may get updated during this method call.</param>
        /// <param name="wasCheckedOut">if set to <c>true</c> then the was checked out prior to this method being called.</param>
        /// <param name="modified">if set to <c>true</c> then the web part was modified as a result of this method being called.</param>
        /// <returns>The modified web part.  This returned web part is what must be used when saving any changes.</returns>
        internal static WebPart ReplaceValues(SPWeb web, 
            SPFile file, 
            Settings settings, 
            ContentEditorWebPart wp,
            Regex regex,
            ref SPLimitedWebPartManager manager,
            ref bool wasCheckedOut,
            ref bool modified)
        {
            if (wp.Content.FirstChild == null && string.IsNullOrEmpty(wp.ContentLink))
                return wp;

            // The first child of a the content XmlElement for a ContentEditorWebPart is a CDATA section
            // so we want to work with that to make sure we don't accidentally replace the CDATA text itself.
            bool isContentMatch = false;
            if (wp.Content.FirstChild != null)
                isContentMatch = regex.IsMatch(wp.Content.FirstChild.InnerText);
            bool isLinkMatch = false;
            if (!string.IsNullOrEmpty(wp.ContentLink))
                isLinkMatch = regex.IsMatch(wp.ContentLink);

            if (!isContentMatch && !isLinkMatch)
                return wp;

            string content;
            if (isContentMatch)
                content = wp.Content.FirstChild.InnerText;
            else
                content = wp.ContentLink;

            string result = content;
            if (!string.IsNullOrEmpty(content))
                result = regex.Replace(content, settings.ReplaceString);

            Log(settings,
                string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                              file.ServerRelativeUrl, wp.Title, content, result));
            if (!settings.Test)
            {
                if (file.CheckOutStatus == SPFile.SPCheckOutStatus.None)
                {
                    file.CheckOut();
                    wasCheckedOut = false;
                }
                // We need to reset the manager and the web part because a checkout (now or from an earlier call) 
                // could mess things up so safest to just reset every time.
                manager.Web.Dispose(); // manager.Dispose() does not dispose of the SPWeb object and results in a memory leak.
                manager.Dispose();
                manager = web.GetLimitedWebPartManager(file.Url, PersonalizationScope.Shared);

                wp.Dispose();
                wp = (ContentEditorWebPart)manager.WebParts[wp.ID];

                if (isContentMatch)
                    wp.Content = GetDataAsXmlElement("Content", "http://schemas.microsoft.com/WebPart/v2/ContentEditor", result);
                else
                    wp.ContentLink = result;

                modified = true;
            }
            return wp;
        }

        /// <summary>
        /// Replaces the content of a <see cref="PageViewerWebPart" /> web part.
        /// </summary>
        /// <param name="web">The web that the file belongs to.</param>
        /// <param name="file">The file that the web part is associated with.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        /// <param name="wp">The web part whose content will be replaced.</param>
        /// <param name="regex">The regular expression object which contains the search pattern.</param>
        /// <param name="manager">The web part manager.  This value may get updated during this method call.</param>
        /// <param name="wasCheckedOut">if set to <c>true</c> then the was checked out prior to this method being called.</param>
        /// <param name="modified">if set to <c>true</c> then the web part was modified as a result of this method being called.</param>
        /// <returns>The modified web part.  This returned web part is what must be used when saving any changes.</returns>
        internal static WebPart ReplaceValues(SPWeb web,
            SPFile file,
            Settings settings,
            PageViewerWebPart wp,
            Regex regex,
            ref SPLimitedWebPartManager manager,
            ref bool wasCheckedOut,
            ref bool modified)
        {
            if (string.IsNullOrEmpty(wp.ContentLink))
                return wp;

            bool isLinkMatch = regex.IsMatch(wp.ContentLink);

            if (!isLinkMatch)
                return wp;

            string content = wp.ContentLink;
            string result = content;

            if (!string.IsNullOrEmpty(content))
                result = regex.Replace(content, settings.ReplaceString);

            Log(settings,
                string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                              file.ServerRelativeUrl, wp.Title, content, result));
            if (!settings.Test)
            {
                if (file.CheckOutStatus == SPFile.SPCheckOutStatus.None)
                {
                    file.CheckOut();
                    wasCheckedOut = false;
                }
                // We need to reset the manager and the web part because a checkout (now or from an earlier call) 
                // could mess things up so safest to just reset every time.
                manager.Web.Dispose(); // manager.Dispose() does not dispose of the SPWeb object and results in a memory leak.
                manager.Dispose();
                manager = web.GetLimitedWebPartManager(file.Url, PersonalizationScope.Shared);

                wp.Dispose();
                wp = (PageViewerWebPart)manager.WebParts[wp.ID];

                wp.ContentLink = result;

                modified = true;
            }
            return wp;
        }


        /// <summary>
        /// Replaces the content of a <see cref="ContentByQueryWebPart"/> web part.
        /// </summary>
        /// <param name="web">The web that the file belongs to.</param>
        /// <param name="file">The file that the web part is associated with.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        /// <param name="wp">The web part whose content will be replaced.</param>
        /// <param name="regex">The regular expression object which contains the search pattern.</param>
        /// <param name="manager">The web part manager.  This value may get updated during this method call.</param>
        /// <param name="wasCheckedOut">if set to <c>true</c> then the was checked out prior to this method being called.</param>
        /// <param name="modified">if set to <c>true</c> then the web part was modified as a result of this method being called.</param>
        /// <returns>The modified web part.  This returned web part is what must be used when saving any changes.</returns>
        internal static WebPart ReplaceValues(SPWeb web,
           SPFile file,
           Settings settings,
           ContentByQueryWebPart wp,
           Regex regex,
           ref SPLimitedWebPartManager manager,
           ref bool wasCheckedOut,
           ref bool modified)
        {
            if (string.IsNullOrEmpty(wp.ListGuid) && string.IsNullOrEmpty(wp.WebUrl))
                return wp;

            bool isListGuidMatch = false;
            if (!string.IsNullOrEmpty(wp.ListGuid))
                isListGuidMatch = regex.IsMatch(wp.ListGuid);

            bool isWebUrlMatch = false;
            if (!string.IsNullOrEmpty(wp.WebUrl))
                isWebUrlMatch = regex.IsMatch(wp.WebUrl);

            if (!isListGuidMatch && !isWebUrlMatch)
                return wp;

            string listGuidContent = wp.ListGuid;
            string webUrlContent = wp.WebUrl;

            string listGuidResult = listGuidContent;
            string webUrlResult = webUrlContent;

            if (!string.IsNullOrEmpty(listGuidContent))
                listGuidResult = regex.Replace(listGuidContent, settings.ReplaceString);
            if (!string.IsNullOrEmpty(webUrlContent))
                webUrlResult = regex.Replace(webUrlContent, settings.ReplaceString);

            if (isListGuidMatch)
                Log(settings,
                    string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                                  file.ServerRelativeUrl, wp.Title, listGuidContent, listGuidResult));
            if (isWebUrlMatch)
                Log(settings,
                    string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                                  file.ServerRelativeUrl, wp.Title, webUrlContent, webUrlResult));
            if (!settings.Test)
            {
                if (file.CheckOutStatus == SPFile.SPCheckOutStatus.None)
                {
                    file.CheckOut();
                    wasCheckedOut = false;
                }
                // We need to reset the manager and the web part because a checkout (now or from an earlier call) 
                // could mess things up so safest to just reset every time.
                manager.Web.Dispose(); // manager.Dispose() does not dispose of the SPWeb object and results in a memory leak.
                manager.Dispose();
                manager = web.GetLimitedWebPartManager(file.Url, PersonalizationScope.Shared);

                wp.Dispose();
                wp = (ContentByQueryWebPart)manager.WebParts[wp.ID];

                if (isListGuidMatch)
                    wp.ListGuid = listGuidResult;
                if (isWebUrlMatch)
                    wp.WebUrl = webUrlResult;

                modified = true;
            }
            return wp;
        }


        /// <summary>
        /// Replaces the content of a <see cref="DataFormWebPart"/> web part.
        /// </summary>
        /// <param name="web">The web that the file belongs to.</param>
        /// <param name="file">The file that the web part is associated with.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        /// <param name="wp">The web part whose content will be replaced.</param>
        /// <param name="regex">The regular expression object which contains the search pattern.</param>
        /// <param name="manager">The web part manager.  This value may get updated during this method call.</param>
        /// <param name="wasCheckedOut">if set to <c>true</c> then the was checked out prior to this method being called.</param>
        /// <param name="modified">if set to <c>true</c> then the web part was modified as a result of this method being called.</param>
        /// <returns>
        /// The modified web part.  This returned web part is what must be used when saving any changes.
        /// </returns>
        internal static WebPart ReplaceValues(SPWeb web,
           SPFile file,
           Settings settings,
           DataFormWebPart wp,
           Regex regex,
           ref SPLimitedWebPartManager manager,
           ref bool wasCheckedOut,
           ref bool modified)
        {
            if (string.IsNullOrEmpty(wp.DataSourcesString) && string.IsNullOrEmpty(wp.ParameterBindings))
                return wp;

            bool isDataSourcesMatch = false;
            if (!string.IsNullOrEmpty(wp.DataSourcesString))
                isDataSourcesMatch = regex.IsMatch(wp.DataSourcesString);
            
            bool isParameterBindingsMatch = false;
            if (!string.IsNullOrEmpty(wp.ParameterBindings))
                isParameterBindingsMatch = regex.IsMatch(wp.ParameterBindings);

            bool isListNameMatch = false;
            if (!string.IsNullOrEmpty(wp.ListName))
                isListNameMatch = regex.IsMatch(wp.ListName);

            if (!isDataSourcesMatch && !isParameterBindingsMatch && !isListNameMatch)
                return wp;

            string dataSourcesContent = wp.DataSourcesString;
            string parameterBindingsContent = wp.ParameterBindings;
            string listNameContent = wp.ListName;

            string dataSourcesResult = dataSourcesContent;
            string parameterBindingsResult = parameterBindingsContent;
            string listNameResult = listNameContent;

            if (!string.IsNullOrEmpty(dataSourcesContent))
                dataSourcesResult = regex.Replace(dataSourcesContent, settings.ReplaceString);
            if (!string.IsNullOrEmpty(listNameContent))
                listNameResult = regex.Replace(listNameContent, settings.ReplaceString);
            try
            {
                if (!string.IsNullOrEmpty(parameterBindingsContent))
                {
                    if (!settings.UnsafeXml)
                        parameterBindingsResult = ReplaceXmlValues(regex, settings.ReplaceString, parameterBindingsContent);
                    else
                        parameterBindingsResult = regex.Replace(parameterBindingsContent, settings.ReplaceString);
                }
            }
            catch (XmlException ex)
            {
                isParameterBindingsMatch = false;
                string msg = string.Format(
                    "WARNING: An error occured replacing data in a DataFormWebPart: File={0}, WebPart={1}\r\n{2}",
                    file.ServerRelativeUrl, wp.Title, Utilities.FormatException(ex));
                if (settings.Quiet && settings.LogFile == null)
                    Console.WriteLine(msg);
                else
                    Log(settings, msg);

            }
            if (isDataSourcesMatch)
                Log(settings,
                    string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                                  file.ServerRelativeUrl, wp.Title, dataSourcesContent, dataSourcesResult));
            if (isParameterBindingsMatch)
                Log(settings,
                    string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                                  file.ServerRelativeUrl, wp.Title, parameterBindingsContent, parameterBindingsResult));
            if (isListNameMatch)
                Log(settings,
                    string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                                  file.ServerRelativeUrl, wp.Title, listNameContent, listNameResult));
            if (!settings.Test)
            {
                if (file.CheckOutStatus == SPFile.SPCheckOutStatus.None)
                {
                    file.CheckOut();
                    wasCheckedOut = false;
                }
                // We need to reset the manager and the web part because a checkout (now or from an earlier call) 
                // could mess things up so safest to just reset every time.
                manager.Web.Dispose(); // manager.Dispose() does not dispose of the SPWeb object and results in a memory leak.
                manager.Dispose();
                manager = web.GetLimitedWebPartManager(file.Url, PersonalizationScope.Shared);

                wp.Dispose();
                wp = (DataFormWebPart)manager.WebParts[wp.ID];

                if (isDataSourcesMatch)
                    wp.DataSourcesString = dataSourcesResult;
                if (isParameterBindingsMatch)
                    wp.ParameterBindings = parameterBindingsResult;
                if (isListNameMatch)
                    wp.ListName = listNameResult;

                modified = true;
            }
            return wp;
        }

        /// <summary>
        /// Replaces the content of a <see cref="ImageWebPart"/> web part.
        /// </summary>
        /// <param name="web">The web that the file belongs to.</param>
        /// <param name="file">The file that the web part is associated with.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        /// <param name="wp">The web part whose content will be replaced.</param>
        /// <param name="regex">The regular expression object which contains the search pattern.</param>
        /// <param name="manager">The web part manager.  This value may get updated during this method call.</param>
        /// <param name="wasCheckedOut">if set to <c>true</c> then the was checked out prior to this method being called.</param>
        /// <param name="modified">if set to <c>true</c> then the web part was modified as a result of this method being called.</param>
        /// <returns>The modified web part.  This returned web part is what must be used when saving any changes.</returns>
        internal static WebPart ReplaceValues(SPWeb web,
           SPFile file,
           Settings settings,
           ImageWebPart wp,
           Regex regex,
           ref SPLimitedWebPartManager manager,
           ref bool wasCheckedOut,
           ref bool modified)
        {
            if (string.IsNullOrEmpty(wp.ImageLink) && string.IsNullOrEmpty(wp.AlternativeText))
                return wp;

            bool isAltTextMatch = false;
            if (!string.IsNullOrEmpty(wp.AlternativeText))
                isAltTextMatch = regex.IsMatch(wp.AlternativeText);

            bool isLinkMatch = false;
            if (!string.IsNullOrEmpty(wp.ImageLink))
                isLinkMatch = regex.IsMatch(wp.ImageLink);

            if (!isAltTextMatch && !isLinkMatch)
                return wp;

            string altTextConent = wp.AlternativeText;
            string linkContent = wp.ImageLink;

            string altTextResult = altTextConent;
            string linkResult = linkContent;

            if (!string.IsNullOrEmpty(altTextConent))
                altTextResult = regex.Replace(altTextConent, settings.ReplaceString);
            if (!string.IsNullOrEmpty(linkContent))
                linkResult = regex.Replace(linkContent, settings.ReplaceString);

            if (isAltTextMatch)
                Log(settings,
                    string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                                  file.ServerRelativeUrl, wp.Title, altTextConent, altTextResult));
            if (isLinkMatch)
                Log(settings,
                    string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                                  file.ServerRelativeUrl, wp.Title, linkContent, linkResult));
            if (!settings.Test)
            {
                if (file.CheckOutStatus == SPFile.SPCheckOutStatus.None)
                {
                    file.CheckOut();
                    wasCheckedOut = false;
                }
                // We need to reset the manager and the web part because a checkout (now or from an earlier call) 
                // could mess things up so safest to just reset every time.
                manager.Web.Dispose(); // manager.Dispose() does not dispose of the SPWeb object and results in a memory leak.
                manager.Dispose();
                manager = web.GetLimitedWebPartManager(file.Url, PersonalizationScope.Shared);

                wp.Dispose();
                wp = (ImageWebPart)manager.WebParts[wp.ID];

                if (isAltTextMatch)
                    wp.AlternativeText = altTextResult;
                if (isLinkMatch)
                    wp.ImageLink = linkResult;

                modified = true;
            }
            return wp;
        }

        /// <summary>
        /// Replaces the content of a <see cref="SiteDocuments"/> web part.
        /// </summary>
        /// <param name="web">The web that the file belongs to.</param>
        /// <param name="file">The file that the web part is associated with.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        /// <param name="wp">The web part whose content will be replaced.</param>
        /// <param name="regex">The regular expression object which contains the search pattern.</param>
        /// <param name="manager">The web part manager.  This value may get updated during this method call.</param>
        /// <param name="wasCheckedOut">if set to <c>true</c> then the was checked out prior to this method being called.</param>
        /// <param name="modified">if set to <c>true</c> then the web part was modified as a result of this method being called.</param>
        /// <returns>The modified web part.  This returned web part is what must be used when saving any changes.</returns>
        internal static WebPart ReplaceValues(SPWeb web,
            SPFile file,
            Settings settings,
            SiteDocuments wp,
            Regex regex,
            ref SPLimitedWebPartManager manager,
            ref bool wasCheckedOut,
            ref bool modified)
        {
            if (string.IsNullOrEmpty(wp.UserTabs))
                return wp;

            bool isLinkMatch = regex.IsMatch(wp.UserTabs);

            if (!isLinkMatch)
                return wp;

            string content = wp.UserTabs;
            string result = content;
            if (!string.IsNullOrEmpty(content))
                result = regex.Replace(content, settings.ReplaceString);

            Log(settings,
                string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                              file.ServerRelativeUrl, wp.Title, content, result));
            if (!settings.Test)
            {
                if (file.CheckOutStatus == SPFile.SPCheckOutStatus.None)
                {
                    file.CheckOut();
                    wasCheckedOut = false;
                }
                // We need to reset the manager and the web part because a checkout (now or from an earlier call) 
                // could mess things up so safest to just reset every time.
                manager.Web.Dispose(); // manager.Dispose() does not dispose of the SPWeb object and results in a memory leak.
                manager.Dispose();
                manager = web.GetLimitedWebPartManager(file.Url, PersonalizationScope.Shared);

                wp.Dispose();
                wp = (SiteDocuments)manager.WebParts[wp.ID];

                wp.UserTabs = result;

                modified = true;
            }
            return wp;
        }

        /// <summary>
        /// Replaces the content of a <see cref="SummaryLinkWebPart"/> web part.
        /// </summary>
        /// <param name="web">The web that the file belongs to.</param>
        /// <param name="file">The file that the web part is associated with.</param>
        /// <param name="settings">The settings object containing user provided parameters.</param>
        /// <param name="wp">The web part whose content will be replaced.</param>
        /// <param name="regex">The regular expression object which contains the search pattern.</param>
        /// <param name="manager">The web part manager.  This value may get updated during this method call.</param>
        /// <param name="wasCheckedOut">if set to <c>true</c> then the was checked out prior to this method being called.</param>
        /// <param name="modified">if set to <c>true</c> then the web part was modified as a result of this method being called.</param>
        /// <returns>The modified web part.  This returned web part is what must be used when saving any changes.</returns>
        internal static WebPart ReplaceValues(SPWeb web,
            SPFile file,
            Settings settings,
            SummaryLinkWebPart wp,
            Regex regex,
            ref SPLimitedWebPartManager manager,
            ref bool wasCheckedOut,
            ref bool modified)
        {
            if (string.IsNullOrEmpty(wp.SummaryLinkStore))
                return wp;

            bool isLinkMatch = false;
            string originalContent = wp.SummaryLinkStore;

            SummaryLinkFieldValue links = wp.SummaryLinkValue;

            // Make all appropriate changes here and then reset the value below.
            // We don't want to manipulate the store itself because we risk messing up the XML.
            foreach (SummaryLink link in links.SummaryLinks)
            {
                if (!string.IsNullOrEmpty(link.Description) && regex.IsMatch(link.Description))
                {
                    link.Description = regex.Replace(link.Description, settings.ReplaceString);
                    isLinkMatch = true;
                }
                if (!string.IsNullOrEmpty(link.ImageUrl) && regex.IsMatch(link.ImageUrl))
                {
                    link.ImageUrl = regex.Replace(link.ImageUrl, settings.ReplaceString);
                    isLinkMatch = true;
                }
                if (!string.IsNullOrEmpty(link.ImageUrlAltText) && regex.IsMatch(link.ImageUrlAltText))
                {
                    link.ImageUrlAltText = regex.Replace(link.ImageUrlAltText, settings.ReplaceString);
                    isLinkMatch = true;
                }
                if (!string.IsNullOrEmpty(link.LinkUrl) && regex.IsMatch(link.LinkUrl))
                {
                    link.LinkUrl = regex.Replace(link.LinkUrl, settings.ReplaceString);
                    isLinkMatch = true;
                }
                if (!string.IsNullOrEmpty(link.Title) && regex.IsMatch(link.Title))
                {
                    link.Title = regex.Replace(link.Title, settings.ReplaceString);
                    isLinkMatch = true;
                }
                if (!string.IsNullOrEmpty(link.LinkToolTip) && regex.IsMatch(link.LinkToolTip))
                {
                    link.LinkToolTip = regex.Replace(link.LinkToolTip, settings.ReplaceString);
                    isLinkMatch = true;
                }
            }
            if (!isLinkMatch)
                return wp;

            Log(settings,
                string.Format("Match found: File={0}, WebPart={1}, Replacement={2} => {3}",
                              file.ServerRelativeUrl, wp.Title, originalContent, links));
            if (!settings.Test)
            {
                if (file.CheckOutStatus == SPFile.SPCheckOutStatus.None)
                {
                    file.CheckOut();
                    wasCheckedOut = false;
                }
                // We need to reset the manager and the web part because a checkout (now or from an earlier call) 
                // could mess things up so safest to just reset every time.
                manager.Web.Dispose(); // manager.Dispose() does not dispose of the SPWeb object and results in a memory leak.
                manager.Dispose();
                manager = web.GetLimitedWebPartManager(file.Url, PersonalizationScope.Shared);

                wp.Dispose();
                wp = (SummaryLinkWebPart)manager.WebParts[wp.ID];

                wp.SummaryLinkValue = links;

                modified = true;
            }
            return wp;
        }

        #endregion

        #endregion

        #region Utility Methods

        /// <summary>
        /// Replaces XML attribute and text node values.
        /// </summary>
        /// <param name="regex">The regular expression object.</param>
        /// <param name="replaceString">The replacement string.</param>
        /// <param name="xmlContent">The XML to replace.</param>
        /// <returns></returns>
        private static string ReplaceXmlValues(Regex regex, string replaceString, string xmlContent)
        {
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.PreserveWhitespace = true;
            
            try
            {
                // The incomming xml may not have a root node so we need to add a temp root in order to load it.
                xmlDoc.LoadXml("<STSADM_TEMP>" + xmlContent + "</STSADM_TEMP>");
            }
            catch (XmlException ex)
            {
                ex.Data.Add("XmlContent", xmlContent);

                throw;
            }

            ReplaceXmlValues(regex, replaceString, xmlDoc.DocumentElement);

            return xmlDoc.DocumentElement.InnerXml;
        }

        /// <summary>
        /// Replaces XML attribute and text node values.
        /// </summary>
        /// <param name="regex">The regular expression object.</param>
        /// <param name="replaceString">The replacement string.</param>
        /// <param name="node">The node.</param>
        private static void ReplaceXmlValues(Regex regex, string replaceString, XmlNode node)
        {
            foreach (XmlNode childNode in node.ChildNodes)
            {
                ReplaceXmlValues(regex, replaceString, childNode);
            }

            if (node.Attributes != null)
            {
                foreach (XmlAttribute attr in node.Attributes)
                {
                    if (!string.IsNullOrEmpty(attr.Value) && regex.IsMatch(attr.Value))
                    {
                        attr.Value = regex.Replace(attr.Value, replaceString);
                    }
                }
            }
            if (node.ChildNodes.Count == 0 && !string.IsNullOrEmpty(node.Value))
            {
                if (regex.IsMatch(node.Value))
                {
                    node.Value = regex.Replace(node.Value, replaceString);
                }
            }
        }

        /// <summary>
        /// Gets the data as XML element.  This method is critical for the <see cref="ContentEditorWebPart"/>.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="ns">The ns.</param>
        /// <param name="data">The data.</param>
        /// <returns></returns>
        internal static XmlElement GetDataAsXmlElement(string name, string ns, string data)
        {
            XmlDocument xdoc = new XmlDocument();
            XmlElement element = xdoc.CreateElement(name, ns);
            if (data != null)
            {
                if (data.Length <= 0)
                {
                    return element;
                }
                if (data.IndexOf("]]>") != -1)
                {
                    element.InnerText = data;
                    return element;
                }
                element.InnerXml = "<![CDATA[" + data + "]]>";
            }
            return element;
        }


        /// <summary>
        /// Logs the specified message using the provided settings.
        /// </summary>
        /// <param name="settings">The settings.</param>
        /// <param name="message">The message.</param>
        private static void Log(Settings settings, string message)
        {
            if (!settings.Quiet)
                Console.WriteLine(message);

            if (settings.LogFile != null)
            {
                File.AppendAllText(settings.LogFile, DateTime.Now + ": " + message + "\r\n");
            }
        }

        #endregion

        #region Internal Classes

        /// <summary>
        /// Data class used for passing user provided settings into the various worker methods.
        /// </summary>
        internal class Settings
        {
            public string SearchString;
            public string ReplaceString;
            public string WebPartName;
            public bool Publish;
            public bool Quiet;
            public bool Test;
            public string LogFile;
            public bool Verbose = true;
            public bool UnsafeXml = false;
        }

        #endregion
    }
}
