using System;
using System.Collections.Specialized;
using System.Text;
using System.Web.UI.WebControls.WebParts;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Edfinancial.SharePoint.STSADM.Commands.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Portal.WebControls;
using Microsoft.SharePoint.Publishing;
using Microsoft.SharePoint.WebPartPages;
using WebPart = Microsoft.SharePoint.WebPartPages.WebPart;

namespace Edfinancial.SharePoint.STSADM.Commands.WebParts
{
    public class RetargetGroupedListingsWebPart : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="RetargetGroupedListingsWebPart"/> class.
        /// </summary>
        public RetargetGroupedListingsWebPart()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", false, null, new SPNonEmptyValidator(), "Please specify the url to search."));
            parameters.Add(new SPParam("scope", "scope", true, null, new SPRegexValidator("(?i:^Farm$|^WebApplication$|^Site$|^Web$|^Page$)")));
            parameters.Add(new SPParam("title", "t", false, "Grouped Listings", new SPNonEmptyValidator(), "Please specify the web part title."));
            parameters.Add(new SPParam("publish", "p", false, null, null));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nRetargets grouped listings web parts upgraded from V2 (points web parts with the specified title to the \"Listings\" list).\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <url to search>");
            sb.Append("\r\n\t-scope <Farm | WebApplication | Site | Web | Page>");
            sb.Append("\r\n\t[-title <title of web parts to search for (default is \"Grouped Listings\")>]");
            sb.Append("\r\n\t[-publish]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            try
            {
                output = string.Empty;

                InitParameters(keyValues);

                string url = Params["url"].Value;
                if (url != null)
                    url = url.TrimEnd('/');

                string scope = Params["scope"].Value.ToLowerInvariant();
                string title = Params["title"].Value.ToLowerInvariant();

                if (scope == "farm")
                {
                    foreach (SPService svc in SPFarm.Local.Services)
                    {
                        if (!(svc is SPWebService))
                            continue;

                        foreach (SPWebApplication webApp in ((SPWebService) svc).WebApplications)
                        {
                            RetargetGroupedListings(webApp, title);
                        }
                    }
                }
                else if (scope == "webapplication")
                {
                    SPWebApplication webApp = SPWebApplication.Lookup(new Uri(url));
                    RetargetGroupedListings(webApp, title);
                }
                else if (scope == "site")
                {
                    using (SPSite site = new SPSite(url))
                    {
                        RetargetGroupedListings(site, title);
                    }
                }
                else if (scope == "web")
                {
                    using (SPSite site = new SPSite(url))
                    using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
                    {
                        RetargetGroupedListings(web, title);
                    }
                }
                else if (scope == "page")
                {
                    using (SPSite site = new SPSite(url))
                    using (SPWeb web = site.OpenWeb())
                    {
                        RetargetGroupedListings(web, web.GetFile(url), title);
                    }
                }
            }
            catch(Exception ex)
            {
#if DEBUG
                Console.WriteLine(Utilities.FormatException(ex));
#endif
                throw;
            }
            return 1;
        }


        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (Params["scope"].UserTypedIn)
            {
                if (Params["scope"].Value.ToLowerInvariant() == "farm" && Params["url"].UserTypedIn)
                    throw new SPSyntaxException("The url parameter is not compatible with a scope of Farm.");
                if (Params["scope"].Value.ToLowerInvariant() != "farm" && !Params["url"].UserTypedIn)
                    throw new SPSyntaxException("The url parameter is required if the scope is not Farm.");
            }
            base.Validate(keyValues);
        }        

        #endregion


        #region Retarget Grouped Listings

        /// <summary>
        /// Retargets the grouped listings.
        /// </summary>
        /// <param name="webApp">The web app that should be searched.</param>
        /// <param name="webPartTitle">The web part title.</param>
        private static void RetargetGroupedListings(SPWebApplication webApp, string webPartTitle)
        {
            foreach (SPSite site in webApp.Sites)
            {
                try
                {
                    RetargetGroupedListings(site, webPartTitle);
                }
                finally
                {
                    site.Dispose();
                }
            }
        }

        /// <summary>
        /// Retargets the grouped listings.
        /// </summary>
        /// <param name="site">The site that should be searched.</param>
        /// <param name="webPartTitle">The web part title.</param>
        private static void RetargetGroupedListings(SPSite site, string webPartTitle)
        {
            foreach (SPWeb web in site.AllWebs)
            {
                try
                {
                    RetargetGroupedListings(web, webPartTitle);
                }
                finally
                {
                    web.Dispose();
                }
            }
        }

        /// <summary>
        /// Retargets the grouped listings.  This is necessary for sites that have been upgraded from V2.
        /// When these sites are moved the content query web part still references the old list ID so we
        /// have to repoint the web part to the new list.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="webPartTitle">The web part title.</param>
        internal static void RetargetGroupedListings(SPWeb web, string webPartTitle)
        {
            foreach (SPWeb subweb in web.Webs)
            {
                RetargetGroupedListings(subweb, webPartTitle);
            }

            SPList list;
            try
            {
                list = web.Lists["Listings"];
            }
            catch (Exception)
            {
                list = null;
            }
            if (list == null || list.ContentTypes["Listing"] == null)
                return;

            foreach (SPFile file in web.Files)
            {
                RetargetGroupedListings(web, list, file, webPartTitle);
            }

            if (PublishingWeb.IsPublishingWeb(web))
            {
                PublishingWeb pubWeb = PublishingWeb.GetPublishingWeb(web);

                foreach (PublishingPage page in pubWeb.GetPublishingPages())
                {
                    RetargetGroupedListings(web, list, page.ListItem.File, webPartTitle);
                }
            }
        }

        /// <summary>
        /// Retargets the grouped listings.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="file">The file.</param>
        /// <param name="webPartTitle">The web part title.</param>
        internal static void RetargetGroupedListings(SPWeb web, SPFile file, string webPartTitle)
        {
            foreach (SPWeb subweb in web.Webs)
            {
                RetargetGroupedListings(subweb, file, webPartTitle);
            }

            SPList list;
            try
            {
                list = web.Lists["Listings"];
            }
            catch (Exception)
            {
                list = null;
            }
            if (list == null || list.ContentTypes["Listing"] == null)
                return;

            RetargetGroupedListings(web, list, file, webPartTitle);
        }

        /// <summary>
        /// Retargets the grouped listings.
        /// </summary>
        /// <param name="web">The web.</param>
        /// <param name="list">The list.</param>
        /// <param name="file">The file.</param>
        /// <param name="webPartTitle">The web part title.</param>
        private static void RetargetGroupedListings(SPWeb web, SPList list, SPFile file, string webPartTitle)
        {
            bool wasCheckedOut = true;

            if (!Utilities.EnsureAspx(file.Url, true, false))
                return; // We can only handle aspx and master pages.

            if (file.CheckOutStatus == SPFile.SPCheckOutStatus.None)
            {
                file.CheckOut();
                wasCheckedOut = false;
                // If it's checked out by another user then this will throw an informative exception so let it do so.
            }
            else if (!Utilities.IsCheckedOutByCurrentUser(file.Item))
            {
                // We don't want to mess with files that are checked out to a different user so skip to the next.
                return;
            }
            bool modified = false;

            SPLimitedWebPartManager manager = null;
            try
            {
                manager = web.GetLimitedWebPartManager(file.ServerRelativeUrl, PersonalizationScope.Shared);
                foreach (WebPart webPart in manager.WebParts)
                {
#pragma warning disable 612
                    ListingSummary cqwp = webPart as ListingSummary;
#pragma warning restore 612

                    if (cqwp == null)
                        continue;

                    if (cqwp.DisplayTitle.ToLowerInvariant() == webPartTitle.ToLowerInvariant())
                    {
                        Guid existingGuid = Guid.Empty;
                        if (!string.IsNullOrEmpty(cqwp.ListGuid))
                            existingGuid = new Guid(cqwp.ListGuid);

                        if (!existingGuid.Equals(list.ID))
                        {
                            RetargetContentQueryWebPart.AdjustWebPart(cqwp, list, web);

                            manager.SaveChanges(cqwp);
                            modified = true;
                        }
                    }
                    webPart.Dispose();
                }

            }
            finally
            {
                if (manager != null)
                {
                    manager.Web.Dispose();
                    manager.Dispose();
                }

                if (modified)
                    file.CheckIn("Checking in changes to page layout due to retargeting of grouped listings web part.");
                else if (!wasCheckedOut)
                    file.UndoCheckOut();

                if (modified && !wasCheckedOut)
                {
                    file.Publish("Publishing changes to page layout due to retargeting of grouped listings web part.");
                    if (file.Item.ModerationInformation != null)
                        file.Approve("Auto-approving changes to page layout due to retargeting of grouped listings web part.");
                }
            }
        }

        #endregion


    }
}
